/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : BrowserDbg.h
 * DATE   : 13.11.2006
 *******************************************************************/
#ifndef INCLUDED_BrowserDbg_h
#define INCLUDED_BrowserDbg_h

/*------- forward declarations:
-------------------------------------------------------------------*/
class QSplitter;
class QPushButton;
class DbgCallStack;
class DbgRegisters;
class DbgDisasm;
class DbgMemory;

/*------- include files:
-------------------------------------------------------------------*/
#include <QWidget>

/*------- class declaration:
-------------------------------------------------------------------*/
class BrowserDbg : public QWidget
{
   Q_OBJECT

// ******* CONSTRUCTION / DESTRUCTION *******
public:
   explicit BrowserDbg( QWidget* = 0 );
   ~BrowserDbg();
private:
   BrowserDbg( const BrowserDbg& );
   BrowserDbg& operator=( const BrowserDbg& );

// ******* CONSTANTS *******
private:
   static const char* const CALL_STACK;
   static const char* const REGISTERS;
   static const char* const MEMORY;
   static const char* const DISASM;
   static const qint32      BTN_MARGIN;
   
   const qint32 d_label_width;

// ******* MEMBERS *******
private:
      QSplitter*   const d_splitter;
      QPushButton* const d_call_stack_btn;
      QPushButton* const d_regs_btn;
      QPushButton* const d_memory_btn;
      QPushButton* const d_disasm_btn;
      DbgCallStack*      d_call_stack_dialog;
      DbgRegisters*      d_registers_dialog;
      DbgDisasm*         d_disasm_dialog;
      DbgMemory*         d_memory_dialog;

// ******* METHODS *******
private:
   void       customEvent ( QEvent* );
   QPushButton* create_button( const QString& );
private slots:
   void        call_stack ( bool );
   void call_stack_closed ();
   void         registers ( bool );
   void  registers_closed ();
   void            memory ( bool );
   void     memory_closed ();
   void            disasm ( bool );
   void     disasm_closed ();
   void            finish ();
};

#endif // INCLUDED_BrowserDbg_h
