/********************************************************************
 * Copyright (C) 2006-2008 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR  : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT : Cobras
 * FILE    : CBtPoints.cpp
 * DATE    : 12.02.2007
 * CHANGED : 02.07.2008
 *-------------------------------------------------------------------
 * OPIS   : PURE VIRTUAL
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "CBtPoints.h"
#include <QVector>
#include <QtDebug>

//*******************************************************************
// add                                                        PUBLIC
//-------------------------------------------------------------------
// Jezeli przyslano punkt o statusie UNSET, zostaje on usuniety z
// kolekcji. W przeciwnym przypadku punkt i jego status sa
// zapamietywane.
//*******************************************************************
void CBtPoints::add( const QString&     in_fpath ,
                     const int          in_row   ,
                     const TPointStatus in_status,
                     const QString&     in_text   )
{
   if( UNSET == in_status ) {
      if( collection_.contains( in_fpath ) ) {
         collection_[ in_fpath ].remove( in_row );
         if( collection_[ in_fpath ].empty() ) {
            collection_.remove( in_fpath );
         }
      }
   }
   else {
      collection_[ in_fpath ][ in_row ] = CBtPointInfo( in_status, in_text );
   }
}
// end of add

//*******************************************************************
// prev                                                       PUBLIC
//*******************************************************************
qint32 CBtPoints::prev( const QString& in_fpath, const int in_row ) const
{
   if( in_row != -1 ) {
      if( collection_.contains( in_fpath ) ) {
         QList<int> points = collection_[ in_fpath ].keys();
         if( points.empty() ) return -1;
         
         qSort( points.begin(), points.end(), qGreater<int>() );
         const QList<int>::iterator end = points.end();
         const QList<int>::const_iterator it = qUpperBound( points.begin(), end, in_row, qGreater<int>() );
         if( it != end ) return (*it);
      }
   }
   return -1;
}
// end of prev

//*******************************************************************
// next                                                       PUBLIC
//*******************************************************************
qint32 CBtPoints::next( const QString& in_fpath, const int in_row ) const
{
   if( in_row != -1 ) {
      if( collection_.contains( in_fpath ) ) {
         QList<int> points = collection_[ in_fpath ].keys();
         if( points.empty() ) return -1;
         
         qSort( points );
         const QList<int>::const_iterator it = qUpperBound( points, in_row );
         if( it != points.end() ) return (*it);
      }
   }
   return -1;
}
// end of next

//*******************************************************************
// status                                                     PUBLIC
//-------------------------------------------------------------------
// Jezeli wskazany punkt jest w kolekcji zwracany jest jego status.
// W przecienym przypadku zwracany jest 'status' UNKNOWN.
//*******************************************************************
CBtPoints::TPointStatus CBtPoints::status( const QString& in_fpath, const int in_row ) const
{
   if( collection_.contains( in_fpath ) ) {
      if( collection_[ in_fpath ].contains( in_row ) ) {
         return collection_[ in_fpath ][ in_row ].status();
      }
   }
   return UNKNOWN;
}
// end of status

//*******************************************************************
// file_data                                                  PUBLIC
//-------------------------------------------------------------------
// Zwraca informacje o punktach dla konkretnego pliku.
//*******************************************************************
void CBtPoints::file_data ( const QString& in_fpath, QHash<int,CBtPointInfo>& out_data ) const
{
   out_data.clear();
   if( collection_.contains( in_fpath ) ) {
      out_data = collection_[ in_fpath ];
   }
}
// end of file_data
