#ifndef INCLUDED_CBtPoints_h
#define INCLUDED_CBtPoints_h
/********************************************************************
 * Copyright (C) 2006-2008 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR  : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT : Cobras
 * FILE    : CBtPoints.h
 * DATE    : 12.02.2007
 * CHANGED : 02.07.2008
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include <QObject>
#include <QString>
#include <QVector>
#include <QHash>

/*------- class declaration:
-------------------------------------------------------------------*/
class CBtPoints : public QObject
{
// ******* TYPES *******
public:
   typedef enum {
      UNKNOWN = 0,
      UNSET,
      SET,
      DISABLED
   } TPointStatus;

   class CBtPointInfo
   {
   public:
      CBtPointInfo( const TPointStatus in_status = UNKNOWN, const QString& in_text = QString() )
      : status_( in_status )
      , text_  ( in_text )
      {}
      CBtPointInfo( const CBtPointInfo& rhs )
      : status_ ( rhs.status_ )
      , text_   ( rhs.text_   )
      {}
      CBtPointInfo& operator=( const CBtPointInfo& rhs ) {
         if( this != &rhs ) {
            status_ = rhs.status_;
            text_   = rhs.text_;
         }
         return *this;
      }
   public:
      TPointStatus   status () const { return status_; }
      const QString& text   () const { return text_;   }
   private:
      TPointStatus status_;
      QString     text_;
   };

    //               sciezka do pliku
    //               |           numer linii
    //               |           |
   typedef QHash<QString, QHash<int, CBtPointInfo> > TCollection;

   
// ******* CONSTRUCTION / DESTRUCTION *******
protected:
   CBtPoints( QObject* const in_parent = 0 ) : QObject( in_parent )
   , collection_ ( TCollection() )
   {}
private:
   CBtPoints( const CBtPoints& );
   CBtPoints& operator=( const CBtPoints& );

// ******* MEMBERS *******
private:
   TCollection collection_;

// ******* METHODS *******
public:
   virtual void change_status( const QString&, int, const QString& ) = 0;

   void         add       ( const QString&, int, TPointStatus, const QString& );
   int          prev      ( const QString&, int ) const;
   int          next      ( const QString&, int ) const;
   TPointStatus status    ( const QString&, int ) const;
   void         file_data ( const QString&, QHash<int,CBtPointInfo>& ) const;


   void clear() {
      collection_.clear();
   }   
   void clear( const QString& in_fpath ) {
      collection_.remove( in_fpath );
   }
   const TCollection& data() {
      return collection_;
   }
};

#endif //  INCLUDED_CBtPoints_h
