/********************************************************************
 * Copyright (C) 2006-2008 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : CBtTextEdit.cpp
 * DATE   : 12.10.2006
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "CBtTextEdit.h"
#include "CBtUserData.h"
#include "Editor.h"
#include "Config.h"
#include <QTextBlock>
#include <QPainter>

//*******************************************************************
// paintEvent                                      PRIVATE inherited
//*******************************************************************
void CBtTextEdit::paintEvent( QPaintEvent* const in_event )
{
   QPainter p( viewport() );

   const Config* const cfg = Config::instance();
   const bool   use_margin = cfg->rmargin_get_use();
   const int    mpos       = cfg->rmargin_get_n() * fontMetrics().width( "X" );
   const QColor color      = cfg->rmargin_get_color();
   const int    vp_height  = viewport()->height();
   const int    vp_width   = viewport()->width();

   if( vp_width > mpos ) {
      if( use_margin ) {
         p.fillRect( QRect( mpos, 0, vp_width - mpos, vp_height ), color.lighter() );
      }
      if( Config::instance()->line_highlight() ) {
         const QTextCursor cursor = textCursor();
         QRect             rect   = cursorRect( cursor );
         rect.setLeft( 0 );
         rect.setWidth( vp_width );
         p.fillRect( rect, QColor( cfg->currline_background() ) );
      }
      if( use_margin ) {
         const QPen org_pen = p.pen();
         p.setPen( color );
         p.drawLine( mpos, 0, mpos, vp_height - 1 );
         p.setPen( org_pen );
      }
      p.end();
   }

   QTextEdit::paintEvent( in_event );   
}
// end of paintEvent

//*******************************************************************
// set_cursor_abs_pos                                         PUBLIC
//*******************************************************************
void CBtTextEdit::set_cursor_abs_pos( const int in_pos )
{
   const QTextBlock beg_block = document()->begin();
   const QTextBlock end_block = document()->end().previous();

   if( beg_block.isValid() && end_block.isValid() ) {
      QTextCursor cursor = textCursor();
      if( in_pos >= beg_block.position() ) {
         if( in_pos <= ( end_block.position() + end_block.length() ) ) {
            cursor.setPosition( in_pos );
         }
         else {
            // Jesli wskazana pozycja kursora znajduje sie za koncem
            // pliku, to ustawiamy kursor na poczatku ostatniej linii tekstu.
            cursor.setPosition( end_block.position() );
         }
      }
      else {
         // Jesli wskazana pozycja kursora znajduje sie przed poczatkiem
         // pliku, to ustawiamy kursor na poczatku pierwszej linii tekstu.
         cursor.setPosition( beg_block.position() );
      }
      setTextCursor( cursor );
      ensureCursorVisible();
   }
}
// end of set_cursor_abs_pos

//*******************************************************************
// set_cursor_pos                                          PROTECTED
//*******************************************************************
void CBtTextEdit::set_cursor_pos( const int in_row, const int in_col )
{
   if( in_row < 0 ) return;
   if( in_col < 0 ) return;
   
   QTextBlock block = document()->begin();
   while( block.isValid() && block.userData() ) {
      const CBtUserData* const data = dynamic_cast<CBtUserData*>( block.userData() );
      if( data->row() == in_row  ) {
         set_cursor_abs_pos( block.position() + in_col );
         return;
      }
      block = block.next();
   }
}
// end of set_cursor_pos

//*******************************************************************
// current_row                                               PRIVATE
//*******************************************************************
int CBtTextEdit::current_row() const
{
   const QTextBlock block = textCursor().block();
   if( block.isValid() && block.userData() ) {
      const CBtUserData* const data = dynamic_cast<CBtUserData*>( block.userData() );
      return data->row();
   }
   return -1;
}
// end of current_row
