/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : Classer.h
 * DATE   : 8.10.2006
 *******************************************************************/
#ifndef INCLUDED_Classer_h
#define INCLUDED_Classer_h

/*------- include files:
-------------------------------------------------------------------*/
#ifndef INCLUDED_ClassParser_h
#include "ClassParser.h"
#endif // INCLUDED_ClassParser_h

#ifndef INCLUDED_Shared_h
#include "Shared.h"
#endif // INCLUDED_Shared_h

#ifndef INCLUDED_QObject
#define INCLUDED_QObject
#include <QObject>
#endif // INCLUDED_QObject

#ifndef INCLUDED_QStringList
#define INCLUDED_QStringList
#include <QStringList>
#endif // INCLUDED_QStringList

#ifndef INCLUDED_QMutex
#define INCLUDED_QMutex
#include <QMutex>
#endif // INCLUDED_QMutex

#ifndef INCLUDED_QVector
#define INCLUDED_QVector
#include <QVector>
#endif // INCLUDED_QVector

/*------- class declaration:
-------------------------------------------------------------------*/
class Classer : public QObject
{
	Q_OBJECT

private:
	enum {
		THREAD_MAX_COUNT = 3,
	};
// ******* CONSTRUCTION / DESTRUCTION *******
private:
	Classer();
	~Classer();
private:
	Classer( const Classer& );
	Classer& operator=( const Classer& );

// ******* MEMBERS *******
private:
	static Classer *d_instance;
	static QMutex           d_work_mtx;
	ClassParser             d_thread[ THREAD_MAX_COUNT ];
	qint32                  d_used[ THREAD_MAX_COUNT ];
	QStringList             d_files;
	qint32                  d_idx;
	QStringList             d_del_classes;
	QStringList             d_new_classes;
	QStringList             d_mod_classes;
	ClassParser::ClassItems d_classes;
	ClassParser::ClassItems d_uptodate_classes;
	bool                    d_reparse;

// ******* METHODS *******
public:
	static Classer*            instance ();
	void                          parse ( const QStringList& );
	void                     parse_next ();
	void                          parse ( const QString& );
	void                        reparse ( const QString&, const QString& );
	const ClassParser::ClassItems &data () const { return d_uptodate_classes; }
	QStringList&            del_classes () { return d_del_classes; }
	QStringList&            new_classes () { return d_new_classes; }
	QStringList&            mod_classes () { return d_mod_classes; }
private:
	qint32                   get_thread () const;
	bool                   all_finished () const;
	void                end_of_proccess ();
	void                       add_data ( const ClassParser::ClassItems& in_src );
	void                    remove_data ( ClassParser::ClassItems& inout_classes, const QString& in_fpath );
	void                squeeze_classes ( ClassParser::ClassItems& in_classes );
	void                          print ( const ClassParser::ClassItems& in_src );
private slots:
	void               finished_handler ( qint32 );
};

#endif // INCLUDED_Classer_h
