/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : ClassesView.h
 * DATE   : 3.09.2006
 *******************************************************************/
#ifndef INCLUDED_ClassesView_h
#define INCLUDED_ClassesView_h

/*------- include files:
-------------------------------------------------------------------*/
#include "ClassParser.h"

#ifndef INCLUDED_Event_h
#include "Event.h"
#endif // INCLUDED_Event_h

#ifndef INCLUDED_Shared_h
#include "Shared.h"
#endif // INCLUDED_Shared_h

#ifndef INCLUDED_QFont
#define INCLUDED_QFont
#include <QFont>
#endif // INCLUDED_QFont

#ifndef INCLUDED_QPalette
#define INCLUDED_QPalette
#include <QPalette>
#endif // INCLUDED_QPalette

#ifndef INCLUDED_QTreeWidget
#define INCLUDED_QTreeWidget
#include <QTreeWidget>
#endif // INCLUDED_QTreeWidget

/*------- class declaration:
-------------------------------------------------------------------*/
class ClassesView : public QTreeWidget
{
   Q_OBJECT
   
// ******* CONSTRUCTION/DESTRUCTION *******
public:
   explicit ClassesView( QWidget* );
   ~ClassesView();
private:
   ClassesView( const ClassesView& );
   ClassesView& operator=( const ClassesView& );

// ******* MEMBERS *******
private:
	QTreeWidgetItem* d_root_item;
	QPalette         d_org_palette;
	QFont            d_org_font;

// ******* TYPES *******
private:
	enum {
		TYPE_ROLE = Qt::UserRole + 1,
		SCOPE_ROLE,
		FILE_ROLE
	};
// ******* METHODS *******
private:
	void              customEvent ( QEvent* );
	void           set_properties ();
	void                   opened ( Event* );
	void                   closed ( Event* );
	void           project_parsed ();
	QTreeWidgetItem*    add_class ( QTreeWidgetItem*, const QString& in_class_name );
	void                 add_item ( QTreeWidgetItem* in_parent, const ParsedItem& in_info );
	void               setup_item ( QTreeWidgetItem* in_item, const ParsedItem& in_info );
	void         project_reparsed ();
	void             update_class ( const QString& in_class_name );
	qint32              cmp_items ( QTreeWidgetItem*, QTreeWidgetItem* ) const;
	qint32               cmp_name ( const QString&, const QString& ) const;
	qint32             cmp_access ( quint32, quint32 ) const;
	void               swap_items ( QTreeWidgetItem*, QTreeWidgetItem* );
	void              sort_childs ( QTreeWidgetItem* );
	void           selected_class ( const QTreeWidgetItem* );
	void          selected_member ( const QTreeWidgetItem* );
	void          selected_method ( const QTreeWidgetItem* );
	QTreeWidgetItem*  search_item ( qint32, const QString& ) const;
	QTreeWidgetItem* search_child ( QTreeWidgetItem*, const ParsedItem& ) const;
private slots:
	void          double_clicked ( QTreeWidgetItem* );
};

#endif // INCLUDED_ClassesView_h
