/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : CppSyntax.h
 * DATE   : 13.09.2006
 *******************************************************************/
#ifndef INCLUDED_CppSyntax_h
#define INCLUDED_CppSyntax_h

/*------- include files:
-------------------------------------------------------------------*/
#include "Config.h"
#include <QSyntaxHighlighter>
#include <QSet>
#include <QString>


/*-------- forward declaration:
-------------------------------------------------------------------*/
class QTextDocument;

/*------- class declaration:
-------------------------------------------------------------------*/
class CppSyntax : public QSyntaxHighlighter
{
   Q_OBJECT

// ******* CONSTRUCTION / DESTRUCTION ******
public:
   explicit CppSyntax( QTextDocument* );
private:
   CppSyntax( const CppSyntax& );
   CppSyntax& operator=( const CppSyntax& );

// ******* MEMBERS *******
private:
   static const char* const STLS [];
   static const char* const TYPES [];
   static const char* const KEYWORDS [];
   static const char* const QT [];

   static QSet< QString > stl_set_;
   static QSet< QString > type_set_;
   static QSet< QString > keyword_set_;
   static QSet< QString > qt_set_;
   static bool            initialised_;

   static QTextCharFormat format_[ Config::EndMarker ];
   

// ******* METHODS *******
private:
   void init            ();
   void set_properties  ();
   void highlightBlock  ( const QString& );
   void save_info       ( qint32, qint32& );
   void save_info       ( qint32, qint32&, qint32 );
   bool is_qt           ( const QString& ) const;
   bool is_class_name   ( const QString&, const int ) const;
public:
   void rehighlight     ();
};

#endif // INCLUDED_CppSyntax_h
