/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : DbgRegisters.h
 * DATE   : 16.02.2007
 *******************************************************************/

/*------- include files:
--------------------------------------------------------------------*/
#include "DbgRegisters.h"
#include "EventsController.h"
#include <QApplication>
#include <QDesktopWidget>
#include <QHBoxLayout>
#include <QtDebug>

/*------- local constants:
-------------------------------------------------------------------*/
const char* const DbgRegisters::CAPTION = QT_TR_NOOP( "Registers" );
const QString     DbgRegisters::ERROR   = "error";


//*******************************************************************
// DbgRegisters                                          CONSTRUCTOR
//*******************************************************************
DbgRegisters::DbgRegisters( QWidget* const in_parent )
: DbgToolDialog ( in_parent )
, d_table       ( new DbgDataTable( this, 3, false ) )
{
	setWindowTitle( tr( CAPTION ) );

	QHBoxLayout* const mlay = new QHBoxLayout( this );
	mlay->setMargin( 0 );
	mlay->setSpacing( 0 );
	mlay->addWidget( d_table );

	d_table->add_row();

	EventsController::instance()->append( this, Event::DBG_REGISTERS_DATA );
}
// end of DbgCallStack

//*******************************************************************
// DbgRegisters                                          CONSTRUCTOR
//*******************************************************************
DbgRegisters::~DbgRegisters()
{
	EventsController::instance()->remove( this );
}
// end of ~DbgRegisters

//*******************************************************************
// showEvent                                       PRIVATE inherited
//*******************************************************************
void DbgRegisters::showEvent( QShowEvent* const in_event )
{
	DbgToolDialog::showEvent( in_event );
	move( qApp->desktop()->screenGeometry().width() - size().width() - 4 - 60, 100 );

	EventsController::instance()->send_event( Event::DBG_REGISTERS_START );
	EventsController::instance()->send_event( Event::DBG_REGISTERS_READ );
}
// end of showEvent

//*******************************************************************
// closeEvent                                      PRIVATE inherited
//*******************************************************************
void DbgRegisters::closeEvent( QCloseEvent* const in_event )
{
	EventsController::instance()->send_event( Event::DBG_REGISTERS_STOP );
	DbgToolDialog::closeEvent( in_event );
}
// end of closeEvent

//*******************************************************************
// customEvent                                     PRIVATE inherited
//*******************************************************************
void DbgRegisters::customEvent( QEvent* const in_event )
{
	Event* const event = dynamic_cast< Event* >( in_event );
	const qint32 type = static_cast<qint32>( event->type() );
         
	switch( type ) {
		case Event::DBG_REGISTERS_DATA:
			parse_data( event );
			break;
	}
}
// end of customEvent

//*******************************************************************
// parse_data                                                PRIVATE
//*******************************************************************
void DbgRegisters::parse_data( Event* const in_event )
{
	const QStringList data = in_event->d_data[ 0 ].toStringList();
	d_table->get_prv_data();

	qint32  row  = 0;
	QString text = "";
	bool    ok   = true;

	QStringList::const_iterator it = data.begin();
	const QStringList::const_iterator end = data.end();
	while( it != end ) {
		if( ERROR == (*it) ) {
			ok = false;
			break;
		}
		if( (*it)[ 0 ].isLetter()  ) {
			if( !text.isEmpty() ) {
				set_item( row++, text.simplified() );
				text = (*it);
			}
			else {
				text += ' ';
				text += (*it);
			}
		}
		else {
			text += ' ';
			text += (*it);
		}
		++it;
	}
	if( ok ) set_item( row++, text.simplified() );
	d_table->remove_rows( row );
}
// end of parse_data

//*******************************************************************
// set_item                                                  PRIVATE
//*******************************************************************
void DbgRegisters::set_item( const qint32 in_idx, const QString& in_text )
{
	if( !in_text.isEmpty() ) {
		const QString var   = in_text.section( ' ', 0, 0 ).trimmed();
		const QString value = in_text.section( ' ', 1 ).trimmed();
		const qint32  row   = ( in_idx < d_table->rowCount() ) ? in_idx : d_table->add_row( in_idx );

		d_table->set_item( row, 0, var );
		d_table->set_item( row, 1, value.section( ' ', 0, 0 ) );
		d_table->set_item( row, 2, value.section( ' ', 1 ) );
		d_table->resize_columns();
		d_table->colors( row );
	}
}
// end of set_item
