/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : EventsController.h
 * DATE   : 3.09.2006
 *******************************************************************/
#ifndef INCLUDED_EventsController_h
#define INCLUDED_EventsController_h

/*------- include files:
-------------------------------------------------------------------*/
#ifndef INCLUDED_Event_h
#include "Event.h"
#endif // INCLUDED_Event_h

#ifndef INCLUDED_QObject
#define INCLUDED_QObject
#include <QObject>
#endif // QObject

#ifndef INCLUDED_QMutex
#define INCLUDED_QMutex
#include <QMutex>
#endif // INCLUDED_QMutex

#ifndef INCLUDED_QHash
#define INCLUDED_QHash
#include <QHash>
#endif // INCLUDED_QHash

#ifndef INCLUDED_QSet
#define INCLUDED_QSet
#include <QSet>
#endif // INCLUDED_QSet


/*------- class declaration:
-------------------------------------------------------------------*/
class EventsController : public QObject
{
	Q_OBJECT

// ******* CONSTRUCTION/DESTRUCTION *******
private:
	EventsController();
	~EventsController();
	EventsController( const EventsController& );
	EventsController& operator=( const EventsController& );

// ******* TYPES *******
private:
	typedef QHash< qint32, QSet< QObject* > > EventRegister;

// ******* MEMBERS *******
private:
	static EventsController* d_instance;
	EventRegister            d_items;
	QMutex                   d_mutex;

// ******* METHODS *******
public:
	static EventsController* instance  ();

	bool     append ( QObject* in_receiver, qint32 in_event_id );
	void     remove ( QObject* in_receiver );
	void send_event ( qint32 in_event_id );
	void send_event ( qint32 in_event_id, QVariant in_val );
	void send_event ( qint32 in_event_id, QVariant in_val1, QVariant in_val2 );
	void send_event ( qint32 in_event_id, QVariant in_val1, QVariant in_val2, QVariant in_val3 );
	void send_event ( qint32 in_event_id, QVariant in_val1, QVariant in_val2, QVariant in_val3, QVariant in_val4 );
private:
	bool     exists ( QObject* in_receiver, qint32 in_event_id );
};

#endif // INCLUDED_EventsController_h
