/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : FilesView.cpp
 * DATE   : 3.09.2006
 *******************************************************************/
#ifndef INCLUDED_FilesView_h
#define INCLUDED_FilesView_h

/*------- include files:
-------------------------------------------------------------------*/
#ifndef INCLUDED_Shared_h
#include "Shared.h"
#endif // INCLUDED_Shared_h

#ifndef INCLUDED_QFont
#define INCLUDED_QFont
#include <QFont>
#endif // INCLUDED_QFont

#ifndef INCLUDED_QPalette
#define INCLUDED_QPalette
#include <QPalette>
#endif // INCLUDED_QPalette

#ifndef INCLUDED_QTreeWidget
#define INCLUDED_QTreeWidget
#include <QTreeWidget>
#endif // INCLUDED_QTreeWidget

/*------- forward declarations:
-------------------------------------------------------------------*/
class Event;

/*------- class declaration:
-------------------------------------------------------------------*/
class FilesView : public QTreeWidget
{
   Q_OBJECT
   
// ******* CONSTRUCTION/DESTRUCTION *******
public:
   explicit FilesView( QWidget* );
   ~FilesView();
private:
   FilesView( const FilesView& );
   FilesView& operator=( const FilesView& );

// ******* TYPES *******
private:
	enum {
		PATH_ROLE = Qt::UserRole + 1,
	};

// ******* CONSTANTS *******
private:
	static const char* const SOURCES;
	static const char* const HEADERS;
	static const char* const FORMS;
	static const char* const RESOURCES;
	static const char* const SUBMENU_OPEN;
	static const char* const SUBMENU_RENAME;
	static const char* const SUBMENU_REMOVE;

// ******* MEMBERS *******
private:
	QTreeWidgetItem* d_root_item;
	QTreeWidgetItem* d_src_item;
	QTreeWidgetItem* d_hdr_item;
	QTreeWidgetItem* d_ui_item;
	QTreeWidgetItem* d_qrc_item;
	QString          d_tmp_buffer;
	QPalette         d_org_palette;
	QFont            d_org_font;
	QString          d_dir;
	QString          d_buffer;

// ******* METHODS *******
private:
	void                 customEvent ( QEvent* );
	void               keyPressEvent ( QKeyEvent* );
	void             mousePressEvent ( QMouseEvent* );
	void              set_properties ();
	void                      opened ( Event* );
	void                      closed ( Event* );
	void              src_file_added ( Event* );
	void              hdr_file_added ( Event* );
	void               ui_file_added ( Event* );
	void              qrc_file_added ( Event* );
	void                added_finish ();
	QTreeWidgetItem* parent_for_type ( const QString& ) const;
	QTreeWidgetItem*    search_child ( QTreeWidgetItem*, const QString& );
	bool                is_file_item ( const QTreeWidgetItem* ) const;
	const QString&             fpath ( const QTreeWidgetItem* );
	void                file_renamed ( Event* );
	void                file_removed ( Event* );
	const QString&         abs_fpath ( const QTreeWidgetItem* );
private slots:
	void              double_clicked ( QTreeWidgetItem* );
	void                submenu_open ();
	void              submenu_rename ();
	void              submenu_remove ();
};

#endif // INCLUDED_FilesView_h
