/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : GetQmake.cpp
 * DATE   : 25.09.2006
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "GetQmake.h"
#include "Config.h"
#include "Shared.h"
#include <QLabel>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QPushButton>
#include <QLineEdit>
#include <QFileDialog>

/*------- local constants:
-------------------------------------------------------------------*/
const char* const GetQmake::CAPTION = QT_TR_NOOP( "Select qmake location" );
const char* const GetQmake::PROMPT  = QT_TR_NOOP( "Here you must type where is your qmake.\nWhen your environmet knows where he is, type simply 'qmake'.\nOr type whole path (or select) to your 'qmake' " );
const char* const GetQmake::GOTO    = QT_TR_NOOP( "&Select" );
const char* const GetQmake::CANCEL  = QT_TR_NOOP( "&Cancel" );
const char* const GetQmake::FILTER  = QT_TR_NOOP( "All files (*)" );


//*******************************************************************
// GotoLine                                              CONSTRUCTOR
//*******************************************************************
GetQmake::GetQmake( QWidget* const in_parent )
: QDialog( in_parent )
, d_input( 0 )
{
	setWindowTitle( tr( CAPTION ) );

	QVBoxLayout* const mlay = new QVBoxLayout( this );

	// Prompt ------------------------------------------------
	mlay->addWidget( new QLabel( tr( PROMPT ), this ) );

	// Pole edycyjne z przyciskiem wywolania dialogu ---------
	QHBoxLayout* const ilay = new QHBoxLayout;
	mlay->addLayout( ilay );
	ilay->addWidget( d_input = new QLineEdit( this ) );
	QPushButton* const select = new QPushButton( "...", this );
	ilay->addWidget( select );

	// Przyciski ---------------------------------------------
	QHBoxLayout* const blay = new QHBoxLayout;
	mlay->addLayout( blay );
	blay->addStretch( 100 );
	QPushButton* const go = new QPushButton( tr( GOTO ), this );
	blay->addWidget( go );
	QPushButton* const cancel = new QPushButton( tr( CANCEL ), this );
	blay->addWidget( cancel );

	connect( select, SIGNAL( clicked() ), this, SLOT( select() ));
	connect( go    , SIGNAL( clicked() ), this, SLOT( accept() ));
	connect( cancel, SIGNAL( clicked() ), this, SLOT( reject() ));
}
// end of GotoLine

//*******************************************************************
// accept                                               PRIVATE slot
//*******************************************************************
void GetQmake::accept()
{
	const QString input_text = d_input->text();
	if( !input_text.isEmpty() ) {
		Config::instance()->qmake_path( input_text );
		Config::instance()->save();
		QDialog::accept();
	}
}
// end of accept

//*******************************************************************
// select                                               PRIVATE slot
//-------------------------------------------------------------------
// Uzytkownik moze wskazac lokalizacje qmake'a poprzez dialog.
//*******************************************************************
void GetQmake::select()
{
	QString fpath = QFileDialog::getOpenFileName( this, tr( CAPTION ), "/", tr( FILTER ) );

	if( fpath != QString::null ) {
		fpath = fpath.trimmed();
		if( !fpath.isEmpty() ) {
			d_input->setText( fpath );
		}
	}
}
// end of select
