/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : GotoLine.cpp
 * DATE   : 23.09.2006
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "GotoLine.h"
#include "Settings.h"
#include "Config.h"
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QPushButton>
#include <QIntValidator>
#include <QStringList>
#include <QLineEdit>
#include <QShowEvent>
#include <QtDebug>

/*------- local constants:
-------------------------------------------------------------------*/
const char* const GotoLine::CAPTION = QT_TR_NOOP( "Goto Line" );
const char* const GotoLine::GOTO    = QT_TR_NOOP( "&Go" );
const char* const GotoLine::CANCEL  = QT_TR_NOOP( "&Cancel" );


//*******************************************************************
// GotoLine                                              CONSTRUCTOR
//*******************************************************************
GotoLine::GotoLine( QWidget* const in_parent )
: QDialog       ( in_parent )
, d_main_layout ( new QVBoxLayout( this ) )
, d_input_layout( new QHBoxLayout )
, d_btn_layout  ( new QHBoxLayout )
, d_input       ( new QComboBox( this ) )
, d_go          ( new QPushButton( tr( GOTO ), this ) )
, d_cancel      ( new QPushButton( tr( CANCEL ), this ) )
{
	setWindowTitle( tr( CAPTION ) );

	d_input->setEditable( true );
	d_input->setMaxCount( MAX_ITEMS_NUMBER );
	d_input->setMaxVisibleItems( MAX_ITEMS_NUMBER );
	d_input->setDuplicatesEnabled( false );
	d_input->setInsertPolicy( QComboBox::InsertAtTop );
	d_input->setValidator( new QIntValidator( 0, 999999, this ) );
	d_go->setDefault( true );
	d_cancel->setDefault( false );

	d_input_layout->addWidget( d_input );
	d_btn_layout->addStretch( 100 );
	d_btn_layout->addWidget( d_go );
	d_btn_layout->addWidget( d_cancel );

	d_main_layout->addLayout( d_input_layout );
	d_main_layout->addLayout( d_btn_layout );

	connect( d_go    , SIGNAL( clicked() ), this, SLOT( accept() ));
	connect( d_cancel, SIGNAL( clicked() ), this, SLOT( reject() ));
}
// end of GotoLine

//*******************************************************************
// ~GotoLine                                              DESTRUCTOR
//*******************************************************************
GotoLine::~GotoLine()
{
	save_data();
}
// end of ~GotoLine

//*******************************************************************
// showEvent                                       PRIVATE inherited
//*******************************************************************
void GotoLine::showEvent( QShowEvent* const in_event )
{
	QDialog::showEvent( in_event );
	restore_data();
	d_input->setCurrentIndex( -1 );
}
// end of showEvent

//*******************************************************************
// save_data                                                 PRIVATE
//*******************************************************************
void GotoLine::save_data()
{
	QStringList slist;
	for( qint32 i = 0; i < d_input->count(); ++i ) {
		slist << d_input->itemText( i );
	}

	Settings stt;
	stt.save( Config::GOTO_LINE_GROUP + Config::GOTO_LINE_KEY, slist );
}
// end of save_data

//*******************************************************************
// restore_data                                              PRIVATE
//*******************************************************************
void GotoLine::restore_data()
{
	Settings stt;
	QVariant data;
	if( stt.read( Config::GOTO_LINE_GROUP + Config::GOTO_LINE_KEY, data ) ) {
		d_input->addItems( data.toStringList() );
	}
}
// end of restore_data

//*******************************************************************
// accept                                               PRIVATE slot
//*******************************************************************
void GotoLine::accept()
{	
	const QString input_text = d_input->currentText();

	if( !input_text.isEmpty() ) {
		if( -1 == d_input->findText( input_text ) ) {
			d_input->insertItem( 0, input_text );
		}
		QDialog::accept();
	}
}
// end of accept
