/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : MainWindow.cpp
 * DATE   : 2.09.2006
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "QBtWorkspace.h"
#include "Translator.h"
#include "QBtSidekick.h"
#include "QBtBottomkick.h"
#include "MainWindow.h"
#include "EventsController.h"
#include "Shared.h"
#include "Config.h"
#include "Settings.h"
#include "QBtParser.h"
#include "Breakpoints.h"
#include <QMenu>
#include <QVector>
#include <QMenuBar>
#include <QToolBar>
#include <QAction>
#include <QFileInfo>
#include <QApplication>
#include <QCloseEvent>
#include <QStatusBar>
#include <QLabel>
#include <QFontMetrics>
#include <QSplitter>
#include <QFontDialog>
#include <QMessageBox>
#include <QActionGroup>
#include <QTranslator>
#include <QtDebug>

/*------- local constants:
-------------------------------------------------------------------*/
const char* const MainWindow::MENU_FILE_LABEL                = QT_TR_NOOP( "&File" );
const char* const MainWindow::MENU_EDIT_LABEL                = QT_TR_NOOP( "&Edit" );
const char* const MainWindow::MENU_SEARCH_LABEL              = QT_TR_NOOP( "&Search" );
const char* const MainWindow::MENU_PROJECT_LABEL             = QT_TR_NOOP( "&Project" );
const char* const MainWindow::MENU_DEBUG_LABEL               = QT_TR_NOOP( "&Debugger" );
const char* const MainWindow::MENU_DOCUMENTS_LABEL           = QT_TR_NOOP( "&Documents" );
const char* const MainWindow::MENU_LANGUAGES                 = QT_TR_NOOP( "&Languages" );
const char* const MainWindow::MENU_HELP_LABEL                = QT_TR_NOOP( "&Help" );
// File .........................................
const char* const MainWindow::OPEN_FILE_LABEL                = QT_TR_NOOP( "&Open File" );
const char* const MainWindow::NEW_FILE_LABEL                 = QT_TR_NOOP( "&New File" );
const char* const MainWindow::SAVE_FILE_LABEL                = QT_TR_NOOP( "&Save File" );
const char* const MainWindow::SAVE_FILE_AS_LABEL             = QT_TR_NOOP( "Save File &As" );
const char* const MainWindow::SAVE_ALL_FILES_LABEL           = QT_TR_NOOP( "&Save All Files" );
const char* const MainWindow::PRINT_LABEL                    = QT_TR_NOOP( "&Print" );
const char* const MainWindow::LAST_OPENED_FILES_LABEL        = QT_TR_NOOP( "Last Opened &Files" );
const char* const MainWindow::LAST_OPENED_PROJECTS_LABEL     = QT_TR_NOOP( "Last Opened &Projects" );
const char* const MainWindow::EXIT_LABEL                     = QT_TR_NOOP( "&Exit" );
// Edit .........................................
const char* const MainWindow::UNDO_LABEL                     = QT_TR_NOOP( "&Undo" );
const char* const MainWindow::REDO_LABEL                     = QT_TR_NOOP( "&Redo" );
const char* const MainWindow::CUT_LABEL                      = QT_TR_NOOP( "Cu&t" );
const char* const MainWindow::COPY_LABEL                     = QT_TR_NOOP( "&Copy" );
const char* const MainWindow::PASTE_LABEL                    = QT_TR_NOOP( "&Paste" );
const char* const MainWindow::DELETE_LABEL                   = QT_TR_NOOP( "&Delete" );
const char* const MainWindow::SELECT_ALL_LABEL               = QT_TR_NOOP( "Select &All" );
const char* const MainWindow::PROPERTIES_LABEL               = QT_TR_NOOP( "&Properties" );
// Search .......................................
const char* const MainWindow::FIND_LABEL                     = QT_TR_NOOP( "&Find" );
const char* const MainWindow::FIND_NEXT_LABEL                = QT_TR_NOOP( "Find Ne&xt" );
const char* const MainWindow::FIND_PREVIOUS_LABEL            = QT_TR_NOOP( "Find Pre&vious" );
const char* const MainWindow::REPLACE_LABEL                  = QT_TR_NOOP( "&Replace" );
const char* const MainWindow::BOOKMARK_NEXT_LABEL            = QT_TR_NOOP( "&Next bookmark" );
const char* const MainWindow::BOOKMARK_PREV_LABEL            = QT_TR_NOOP( "&Previous bookmark" );
const char* const MainWindow::BOOKMARK_TOGGLE_LABEL          = QT_TR_NOOP( "&Toggle bookmark" );
const char* const MainWindow::BOOKMARK_ALL_LABEL             = QT_TR_NOOP( "&List of all bookmarks" );
const char* const MainWindow::GOTO_LINE_LABEL                = QT_TR_NOOP( "Goto &Line" );
const char* const MainWindow::SWITCH_FILES_LABEL             = QT_TR_NOOP( "S&witch Files" );
const char* const MainWindow::FIND_IN_PROJECT_FILES_LABEL    = QT_TR_NOOP( "Find In &Project Files" );
// Project ......................................
const char* const MainWindow::OPEN_PROJECT_LABEL             = QT_TR_NOOP( "&Open Project" );
const char* const MainWindow::CLOSE_PROJECT_LABEL            = QT_TR_NOOP( "&Close Project" );
const char* const MainWindow::NEW_PROJECT_LABEL              = QT_TR_NOOP( "&New Project" );
const char* const MainWindow::ADD_FILE_TO_PROJECT_LABEL      = QT_TR_NOOP( "&Add File" );
const char* const MainWindow::REMOVE_FILE_FROM_PROJECT_LABEL = QT_TR_NOOP( "Remo&ve File" );
const char* const MainWindow::RUN_LABEL                      = QT_TR_NOOP( "&Run" );
const char* const MainWindow::BUILD_LABEL                    = QT_TR_NOOP( "&Build" );
const char* const MainWindow::REBUILD_LABEL                  = QT_TR_NOOP( "Reb&uild" );
const char* const MainWindow::BREAK_LABEL                    = QT_TR_NOOP( "B&reak" );
// Debugger
const char* const MainWindow::DBG_BREAKPOINT_LABEL           = QT_TR_NOOP( "&Breakpoint (toggle)" );
const char* const MainWindow::DBG_BREAKPOINT_CLEAR_LABEL     = QT_TR_NOOP( "&Clear all breakpoints" );
const char* const MainWindow::DBG_LIST_BREAKPOINT_LABEL      = QT_TR_NOOP( "&List of all breakpoint" );
const char* const MainWindow::DBG_RUN_LABEL                  = QT_TR_NOOP( "&Start" );
const char* const MainWindow::DBG_STEP_OVER_LABEL            = QT_TR_NOOP( "Step &Over" );
const char* const MainWindow::DBG_STEP_INTO_LABEL            = QT_TR_NOOP( "Step &Into" );
const char* const MainWindow::DBG_STEP_OUT_LABEL             = QT_TR_NOOP( "Step O&ut" );
const char* const MainWindow::DBG_CONTINUE_LABEL             = QT_TR_NOOP( "Co&ntinue" );
const char* const MainWindow::DBG_PAUSE_LABEL                = QT_TR_NOOP( "&Pause" );
const char* const MainWindow::DBG_STOP_LABEL                 = QT_TR_NOOP( "Brea&k" );
// Documents ....................................
const char* const MainWindow::CLOSE_LABEL                    = QT_TR_NOOP( "&Close" );
const char* const MainWindow::CLOSE_ALL_LABEL                = QT_TR_NOOP( "Close &All" );
const char* const MainWindow::TILE_LABEL                     = QT_TR_NOOP( "&Tile" );
const char* const MainWindow::CASCADE_LABEL                  = QT_TR_NOOP( "Casca&de" );
// Languages ....................................
const char* const MainWindow::ENGLISH_LABEL                  = QT_TR_NOOP( "&English" );
const char* const MainWindow::POLISH_LABEL                   = QT_TR_NOOP( "&Polish" );
// Help .........................................
const char* const MainWindow::HELP_LABEL                     = QT_TR_NOOP( "&Info" );
const char* const MainWindow::ABOUT_PROGRAM_LABEL            = QT_TR_NOOP( "&About Program" );
const char* const MainWindow::ABOUT_QT_LABEL                 = QT_TR_NOOP( "About &Qt" );
// SHORTCUTS
// File .........................................
const char* const MainWindow::OPEN_FILE_SHORTCUT             = QT_TR_NOOP( "Ctrl+O" );
const char* const MainWindow::NEW_FILE_SHORTCUT              = QT_TR_NOOP( "Ctrl+N" );
const char* const MainWindow::SAVE_FILE_SHORTCUT             = QT_TR_NOOP( "Ctrl+S" );
const char* const MainWindow::SAVE_FILE_AS_SHORTCUT          = QT_TR_NOOP( "Shift+Ctrl+S" );
const char* const MainWindow::SAVE_ALL_FILES_SHORTCUT        = QT_TR_NOOP( "Shift+Ctrl+L" );
const char* const MainWindow::PRINT_FAST_SHORTCUT            = QT_TR_NOOP( "Ctrl+P" );
const char* const MainWindow::EXIT_SHORTCUT                  = QT_TR_NOOP( "Ctrl+Q" );
// Edit .........................................
const char* const MainWindow::UNDO_SHORTCUT                  = QT_TR_NOOP( "Ctrl+Z" );
const char* const MainWindow::REDO_SHORTCUT                  = QT_TR_NOOP( "Shift+Ctrl+Z" );
const char* const MainWindow::CUT_SHORTCUT                   = QT_TR_NOOP( "Ctrl+X" );
const char* const MainWindow::COPY_SHORTCUT                  = QT_TR_NOOP( "Ctrl+C" );
const char* const MainWindow::PASTE_SHORTCUT                 = QT_TR_NOOP( "Ctrl+V" );
const char* const MainWindow::SELECT_ALL_SHORTCUT            = QT_TR_NOOP( "Ctrl+A" );
// Search .......................................
const char* const MainWindow::FIND_SHORTCUT                  = QT_TR_NOOP( "Ctrl+F" );
const char* const MainWindow::FIND_NEXT_SHORTCUT             = QT_TR_NOOP( "F3" );
const char* const MainWindow::FIND_PREVIOUS_SHORTCUT         = QT_TR_NOOP( "F2" );
const char* const MainWindow::REPLACE_SHORTCUT               = QT_TR_NOOP( "Ctrl+H" );
const char* const MainWindow::BOOKMARK_NEXT_SHORTCUT         = QT_TR_NOOP( "Ctrl+Down" );
const char* const MainWindow::BOOKMARK_PREV_SHORTCUT         = QT_TR_NOOP( "Ctrl+Up" );
const char* const MainWindow::BOOKMARK_TOGGLE_SHORTCUT       = QT_TR_NOOP( "Ctrl+B" );
const char* const MainWindow::BOOKMARK_ALL_SHORTCUT          = QT_TR_NOOP( "Ctrl+Alt+B" );
const char* const MainWindow::GOTO_LINE_SHORTCUT             = QT_TR_NOOP( "Ctrl+G" );
const char* const MainWindow::SWITCH_FILES_SHORTCUT          = QT_TR_NOOP( "Ctrl+M" );
// Project ......................................
const char* const MainWindow::RUN_SHORTCUT                   = QT_TR_NOOP( "Ctrl+F5" );
const char* const MainWindow::BUILD_SHORTCUT                 = QT_TR_NOOP( "F7" );
const char* const MainWindow::REBUILD_SHORTCUT               = QT_TR_NOOP( "Ctrl+F7" );
// Debugger .....................................
const char* const MainWindow::DBG_BREAKPOINT_SHORTCUT        = QT_TR_NOOP( "F6" );
const char* const MainWindow::DBG_LIST_BREAKPOINT_SHORTCUT   = QT_TR_NOOP( "Ctrl+F6" );
const char* const MainWindow::DBG_RUN_SHORTCUT               = QT_TR_NOOP( "F5" );
const char* const MainWindow::DBG_STEP_OVER_SHORTCUT         = QT_TR_NOOP( "F8" );
const char* const MainWindow::DBG_STEP_INTO_SHORTCUT         = QT_TR_NOOP( "F9" );
const char* const MainWindow::DBG_STEP_OUT_SHORTCUT          = QT_TR_NOOP( "F10" );
// Documents ....................................
const char* const MainWindow::CLOSE_SHORTCUT                 = QT_TR_NOOP( "Ctrl+W" );
const char* const MainWindow::CLOSE_ALL_SHORTCUT             = QT_TR_NOOP( "Shift+Ctrl+W" );
// Help .........................................
const char* const MainWindow::HELP_SHORTCUT                  = QT_TR_NOOP( "F1" );
//............................................
const char* const MainWindow::COL_LABEL                      = QT_TR_NOOP( "Col: " );
const char* const MainWindow::ROW_LABEL                      = QT_TR_NOOP( "Row: " );
const char* const MainWindow::WINDOW_KEY                     = "window";
const char* const MainWindow::WINDOW_POS_KEY                 = "position";
const char* const MainWindow::WINDOW_SIZE_KEY                = "size";
const char* const MainWindow::HOR_SPLITTER_KEY               = "splitter-h";
const char* const MainWindow::VER_SPLITTER_KEY               = "splitter-v";
const char* const MainWindow::MENU_FILES_KEY                 = "menu files";
const char* const MainWindow::MENU_PROJECTS_KEY              = "menu project";


//*******************************************************************
// MainWindow                                            CONSTRUCTOR
//*******************************************************************
MainWindow::MainWindow() : QMainWindow()
, menu_file_           ( new QMenu )
, menu_edit_           ( new QMenu )
, menu_search_         ( new QMenu )
, menu_project_        ( new QMenu )
, menu_debug_          ( new QMenu )
, menu_documents_      ( new QMenu )
, menu_languages_      ( new QMenu )
, menu_help_           ( new QMenu )
// Toolbars .....................................
, file_toolbar_       ( new QToolBar )
, edit_toolbar_       ( new QToolBar )
, search_toolbar_     ( new QToolBar )
, bookmark_toolbar_   ( new QToolBar )
, project_toolbar_    ( new QToolBar )
, documents_toolbar_  ( new QToolBar )
, dbg_toolbar_        ( new QToolBar )
// File .........................................
, open_file_           ( new QAction( QIcon( ":/img/fileopen.png" ), "", this ) )
, new_file_            ( new QAction( QIcon( ":/img/new.png" )     , "", this ) )
, save_file_           ( new QAction( QIcon( ":/img/filesave.png" ), "", this ) )
, save_file_as_        ( new QAction( this ) )
, save_all_files_      ( new QAction( QIcon( ":/img/save_all.png" ), "", this ) )
, print_               ( new QAction( QIcon( ":/img/print.png" )   , "", this ) )
, print_fast_          ( new QAction( QIcon( ":/img/print.png" )   , "", this ) )
, last_opened_files_   ( new QMenu  ( this ) )
, last_opened_projects_( new QMenu  ( this ) )
, exit_                ( new QAction( QIcon( ":/img/exit.png" )    , "", this ) )
// Edit .........................................
, undo_                ( new QAction( QIcon( ":/img/undo.png" )    , "", this ) )
, redo_                ( new QAction( QIcon( ":/img/redo.png" )    , "", this ) )
, cut_                 ( new QAction( QIcon( ":/img/cut.png" )     , "", this ) )
, copy_                ( new QAction( QIcon( ":/img/copy.png" )    , "", this ) )
, paste_               ( new QAction( QIcon( ":/img/paste.png" )   , "", this ) )
, delete_              ( new QAction( this ) )
, select_all_          ( new QAction( this ) )
, properties_          ( new QAction( QIcon( ":img/configure.png" ), "", this ) )
// Search .......................................
, Find                 ( new QAction( QIcon( ":/img/find.png" )    , "", this ) )
, FindNext             ( new QAction( QIcon( ":/img/next.png" )    , "", this ) )
, FindPrevious         ( new QAction( QIcon( ":/img/prev.png" )    , "", this ) )
, Replace              ( new QAction( this ))
, BookmarkNext         ( new QAction( QIcon( ":/img/bookmark_next.png" )  , "", this ) )
, BookmarkPrev         ( new QAction( QIcon( ":/img/bookmark_prev.png" )  , "", this ) )
, BookmarkToggle       ( new QAction( QIcon( ":/img/bookmark_toggle.png" ), "", this ) )
, BookmarkAll          ( new QAction( QIcon( ":/img/bookmark_list.png" )  , "", this ) )
, GotoLine             ( new QAction( this ) )
, SwitchFiles          ( new QAction( this ) )
, FindInProjectFiles   ( new QAction( this ) )
// Project ......................................
, OpenProject          ( new QAction( this ) )
, CloseProject         ( new QAction( this ) )
, NewProject           ( new QAction( this ) )
, ProjectProperties    ( new QAction( this ) )
, AddFileToProject     ( new QAction( this ) )
, RemoveFileFromProject( new QAction( this ) )
, Run                  ( new QAction( QIcon( ":/img/run.png" )    , "", this ) )
, Build                ( new QAction( QIcon( ":/img/make.png" )   , "", this ) )
, Rebuild              ( new QAction( QIcon( ":/img/rebuild.png" ), "", this ) )
, Break                ( new QAction( QIcon( ":/img/break.png" )  , "", this ) )
// Debugger .................................
, DbgBreakpoint        ( new QAction( this ) )
, DbgBreakpointClear   ( new QAction( this ) )
, DbgListBreakpoint    ( new QAction( this ) )
, DbgRun               ( new QAction( QIcon( ":/img/dbgrun.png" )    , "", this ) )
, DbgPause             ( new QAction( QIcon( ":/img/dbgpause.png" )  , "", this ) )
, DbgStepOver          ( new QAction( QIcon( ":/img/dbgnext.png" )   , "", this ) )
, DbgStepInto          ( new QAction( QIcon( ":/img/dbgstep.png" )   , "", this ) )
, DbgStepOut           ( new QAction( QIcon( ":/img/dbgstepout.png" ), "", this ) )
, DbgContinue          ( new QAction( QIcon( ":/img/go-down.png" )   , "", this ) )
, DbgStop              ( new QAction( QIcon( ":/img/dbgstop.png" )   , "", this ) )
// Documents ....................................
, Close                ( new QAction( this ) )
, CloseAll             ( new QAction( this ) )
, Tile                 ( new QAction( this ) )
, Cascade              ( new QAction( this ) )
, document_separator_  ( new QAction( this ) )
// Languages ....................................
, english_             ( new QAction( this ) )
, polish_              ( new QAction( this ) )
// Help .........................................
, d_help               ( new QAction( this ) )
, d_about_program      ( new QAction( this ) )
, d_about_qt           ( new QAction( this ) )
, d_document_group     ( new QActionGroup( this ) )
, d_last_project_group ( new QActionGroup( this ) )
, d_last_file_group    ( new QActionGroup( this ) )
//...............................................
, workspace_           ( new QBtWorkspace )
, sidekick_            ( new QBtSidekick )
, bottomkick_          ( new QBtBottomkick )
, ColPrompt            ( new QLabel( COL_LABEL, statusBar() ))
, d_col_number         ( new QLabel( statusBar() ))
, RowPrompt            ( new QLabel( ROW_LABEL, statusBar() ))
, d_row_number         ( new QLabel( statusBar() ))
, d_first_time         ( true )
, d_debugging          ( false )
{
   //menu_debug_->setEnabled( false );
   
   file_toolbar_     ->setObjectName( "file toolbar"      );
   edit_toolbar_     ->setObjectName( "edit toolbar"      );
   search_toolbar_   ->setObjectName( "search toolbar"    );
   bookmark_toolbar_ ->setObjectName( "bookmark toolbar"  );
   project_toolbar_  ->setObjectName( "project toolbar"   );
   documents_toolbar_->setObjectName( "documents toolbar" );
   dbg_toolbar_      ->setObjectName( "debug toolbar"     );
   
   document_separator_->setSeparator( true );
   document_separator_->setVisible( true );
   last_opened_files_->setEnabled( false );
   last_opened_projects_->setEnabled( false );
   undo_->setEnabled( false );
   redo_->setEnabled( false );
   copy_->setEnabled( false );
   cut_->setEnabled( false );

   english_->setCheckable( true );
   polish_->setCheckable( true );
   set_initial_lang_menu_item();
   QActionGroup* const agrp = new QActionGroup( this );
   if( agrp ) {
      agrp->setExclusive( true );
      agrp->addAction( english_ );
      agrp->addAction( polish_  );
      connect( agrp, SIGNAL( triggered( QAction* ) ), this, SLOT( language_slot( QAction* ) ) );
   }

   translate();
   update_actions();   
   create_menu();
   create_toolbar();
   init_status_bar();

   //setCentralWidget( d_v_splitter );
   setCentralWidget( workspace_ );
   addDockWidget( Qt::LeftDockWidgetArea, sidekick_ );
   addDockWidget( Qt::BottomDockWidgetArea, bottomkick_ );

   EventsController::instance()->append( this, Event::FILE_OPENED );
   EventsController::instance()->append( this, Event::FILE_CLOSED );
   EventsController::instance()->append( this, Event::ALL_FILES_CLOSED );
   EventsController::instance()->append( this, Event::FILE_RENAMED );
   EventsController::instance()->append( this, Event::ACTIVE_DOCUMENT_CHANGED );
   EventsController::instance()->append( this, Event::CURSOR_POSITION_CHANGED );
   EventsController::instance()->append( this, Event::PROJECT_OPENED );
   EventsController::instance()->append( this, Event::PROJECT_CLOSED );
   EventsController::instance()->append( this, Event::UNDO_AVAILABLE );
   EventsController::instance()->append( this, Event::REDO_AVAILABLE );
   EventsController::instance()->append( this, Event::COPY_AVAILABLE );
   EventsController::instance()->append( this, Event::PROCESS_STARTED );
   EventsController::instance()->append( this, Event::PROCESS_BROKEN );
   EventsController::instance()->append( this, Event::DBG_STARTED );
   EventsController::instance()->append( this, Event::DBG_FINISHED );
   EventsController::instance()->append( this, Event::DBG_DISABLE_ALL_REQUEST );
   EventsController::instance()->append( this, Event::DBG_DISABLE_RUN_REQUEST );
   EventsController::instance()->append( this, Event::DBG_FINISHED );
   EventsController::instance()->append( this, Event::RUN_REQUEST_EXTERNAL );
   EventsController::instance()->append( this, Event::DBG_RUN_REQUEST_EXTERNAL );

   active_debug_items( DBG_DISABLE_ALL );
}
// end of MainWindow

//*******************************************************************
// ~MainWindow                                            DESTRUCTOR
//*******************************************************************
MainWindow::~MainWindow()
{
   EventsController::instance()->remove( this );
   save_settings();
  
   while( d_last_files.size() > 0 ) {
      delete d_last_files.takeLast();
   }
}
// end of ~MainWindow

//*******************************************************************
// set_initial_lang_menu_item                                PRIVATE
//*******************************************************************
void MainWindow::set_initial_lang_menu_item()
{
   QAction* action = 0;
   Translator::Language lang = Translator::instance()->current_lang();
   switch( lang ) {
      case Translator::POLISH:
         action = polish_;
         break;
      default:
         action = english_;
   }
   action->setChecked( true );
}
// end of set_initial_lang_menu_item

//*******************************************************************
// save_settings                                             PRIVATE
//*******************************************************************
void MainWindow::save_settings()
{
   Settings stt;
   stt.save( Config::MAIN_WINDOW_GROUP + WINDOW_KEY, saveState() );
   stt.save( Config::MAIN_WINDOW_GROUP + WINDOW_POS_KEY, pos() );
   stt.save( Config::MAIN_WINDOW_GROUP + WINDOW_SIZE_KEY, size() );

   // ostatnio otwarte pliki   
   QStringList files;
   foreach( QAction* action, d_last_files ) {
      files << action->statusTip();
   }
   stt.save( Config::MAIN_WINDOW_GROUP + Config::MENU_FILES_KEY, files );
   
   // ostatnio otwarte projekty
   QStringList projects;
   foreach( QAction* action, d_last_projects ) {
      projects << action->statusTip();
   }
   stt.save( Config::MAIN_WINDOW_GROUP + Config::MENU_PROJECTS_KEY, projects );
}
// end of save_settings

//*******************************************************************
// restore_settings                                          PRIVATE
//*******************************************************************
void MainWindow::restore_settings()
{
   Settings stt;
   QVariant data;

   if( stt.read( Config::MAIN_WINDOW_GROUP + WINDOW_KEY, data ) ) {
      restoreState( data.toByteArray() );
   }
   // Ostatnio otwarte pliki
   if( stt.read( Config::MAIN_WINDOW_GROUP + Config::MENU_FILES_KEY, data ) ) {
      const QStringList files = data.toStringList();
      if( !files.isEmpty() ) {
         for( qint32 i = ( files.size() - 1 ); i >= 0; --i ) {
            add_to_last_opened_files( files.at( i ) );
         }
      }
   }
   // Ostatnio otwarte projekty.
   if( stt.read( Config::MAIN_WINDOW_GROUP + Config::MENU_PROJECTS_KEY, data ) ) {
      const QStringList projects = data.toStringList();
      if( !projects.isEmpty() ) {
         for( qint32 i = ( projects.size() - 1 ); i >= 0; --i ) {
            add_to_last_opened_projects( projects.at( i ) );
         }
      }
   }
}
// end of restore_settings

//*******************************************************************
// project_from_command_line                                 PRIVATE
//-------------------------------------------------------------------
// Wyszukanie nazwy projektu startowego wsrod paramentrow
// wywolania programu. Interesuje nas pierwszy parametr
// konczacy sie rozszerzeniem zastrzezonym dla projektow.
//*******************************************************************
void MainWindow::project_from_command_line( QString& out_path )
{
   const QStringList argv = QApplication::arguments();
   QStringList::const_iterator it = argv.begin();
   const QStringList::const_iterator end = argv.end();
   while( it != end ) {
      if( it->endsWith( ".pro" ) ) {
         out_path = *it;
         break;
      }
      ++it;
   }
}
// end of project_from_command_line

//*******************************************************************
// showEvent                                       PRIVATE inherited
//-------------------------------------------------------------------
// W czasie uruchomienia programu odtwarzamy wszystkie ustawienia.
// Robimy to tylko przy PIERWSZYM wyswietleniu okna programu.
//*******************************************************************
void MainWindow::showEvent( QShowEvent* const in_event )
{
   if( d_first_time ) {
      restore_settings();

      Settings stt;
      
      Shared::resize( this, 90, 90 );
      QVariant data;
      if( stt.read( Config::MAIN_WINDOW_GROUP + WINDOW_POS_KEY, data ) ) {
         move( data.toPoint() );
      }
      if( stt.read( Config::MAIN_WINDOW_GROUP + WINDOW_SIZE_KEY, data ) ) {
         resize( data.toSize() );
      }

      // Otwarcie ostatnio otwartego projektu.
      // Oczywiscie jesli tak wynika z konfiguracji.
      // Pierwszenstwo ma projekt podany w linii wywolania programu.
      QString project_on_start = "";
      project_from_command_line( project_on_start );
      
      // Jesli nie podano sciezki do projektu w linii wywolania
      // programu wykorzystujemy informacje konfiguracyjne.
      if( project_on_start.isEmpty() ) {
         if( Config::instance()->load_project() ) {
            if( !d_last_projects.isEmpty() ) {
               const QAction* const action = d_last_projects.at( 0 );
               if( action ) {
                  project_on_start = action->statusTip();
               }
            }
         }
      }
      // Jesli mamy jakis projekt do wczytania to robimy to.
      if( !project_on_start.isEmpty() ) {
         EventsController::instance()->send_event( Event::PROJECT_SELECTED_REQUEST, project_on_start );
      }

      // Otwarcie wszystkich ostanio otwartych edytorow.
      // Oczywiscie jesli tak wynika z konfiguracji.
      if( Config::instance()->load_files() ) {
         workspace_->restore_all_windows();
      }
      // Ta praca byla wykonana tylko teraz.
      // Wiecej tego juz nie robimy.
      d_first_time = false;
   }
   if( d_debugging ) dbg_toolbar_->show();
   else              dbg_toolbar_->hide();

   showMaximized();
   QMainWindow::showEvent( in_event );
}
// end of showEvent

//*******************************************************************
// closeEvent                                      PRIVATE inherited
//*******************************************************************
void MainWindow::closeEvent( QCloseEvent* const in_event )
{
   workspace_->save_info_about_windows();
   workspace_->close_all();
   workspace_->activeWindow() ? in_event->ignore() : in_event->accept();
}
// end of closeEvent

//*******************************************************************
// update_actions                                            PRIVATE
//*******************************************************************
void MainWindow::update_actions()
{
   active_file_items( false );
   active_project_items( false );

   // ACTIONS ===================================
   
   // File ......................................
   connect( open_file_     , SIGNAL( triggered()           ),
            this           , SLOT  ( open_file_slot()      ));
   connect( new_file_      , SIGNAL( triggered()           ),
            this           , SLOT  ( new_file_slot()       ));
   connect( save_file_     , SIGNAL( triggered()           ),
            this           , SLOT  ( save_file_slot()      ));
   connect( save_file_as_  , SIGNAL( triggered()           ),
            this           , SLOT  ( save_file_as_slot()   ));
   connect( save_all_files_, SIGNAL( triggered()           ),
            this           , SLOT  ( save_all_files_slot() ));
   connect( print_         , SIGNAL( triggered()           ),
            this           , SLOT  ( print()               ));
   connect( print_fast_    , SIGNAL( triggered()           ),
            this           , SLOT  ( print_fast()          ));
   connect( exit_          , SIGNAL( triggered()           ),
            qApp           , SLOT  ( closeAllWindows()     ));   // !!! wyjatek
   // Edit ......................................
   connect( undo_      , SIGNAL( triggered()       ),
            this       , SLOT  ( undo_slot()       ));
   connect( redo_      , SIGNAL( triggered()       ),
            this       , SLOT  ( redo_slot()       ));
   connect( copy_      , SIGNAL( triggered()       ),
            this       , SLOT  ( copy_slot()       ));
   connect( cut_       , SIGNAL( triggered()       ),
            this       , SLOT  ( cut_slot()        ));
   connect( paste_     , SIGNAL( triggered()       ),
            this       , SLOT  ( paste_slot()      ));
   connect( delete_    , SIGNAL( triggered()       ),
            this       , SLOT  ( delete_slot()     ));
   connect( select_all_, SIGNAL( triggered()       ),
            this       , SLOT  ( select_all_slot() ));
   connect( properties_, SIGNAL( triggered()       ),
            this       , SLOT  ( properties_slot() ));
   // Search ....................................
   connect( Find               , SIGNAL( triggered()                  ),
            this               , SLOT  ( find_slot()                  ));
   connect( FindNext           , SIGNAL( triggered()                  ),
            this               , SLOT  ( find_next_slot()             ));
   connect( FindPrevious       , SIGNAL( triggered()                  ),
            this               , SLOT  ( find_previous_slot()         ));
   connect( Replace            , SIGNAL( triggered()                  ),
            this               , SLOT  ( replace_slot()               ));
   connect( BookmarkNext       , SIGNAL( triggered()                  ),
            this               , SLOT  ( bookmark_next_slot()         ));
   connect( BookmarkPrev       , SIGNAL( triggered()                  ),
            this               , SLOT  ( bookmark_prev_slot()         ));
   connect( BookmarkToggle     , SIGNAL( triggered()                  ),
            this               , SLOT  ( bookmark_toggle_slot()       ));
   connect( BookmarkAll        , SIGNAL( triggered()                  ),
            this               , SLOT  ( bookmark_all_slot()          ));
   connect( GotoLine           , SIGNAL( triggered()                  ),
            this               , SLOT  ( goto_line_slot()             ));
   connect( SwitchFiles        , SIGNAL( triggered()                  ),
            this               , SLOT  ( switch_files_slot()          ));
   connect( FindInProjectFiles , SIGNAL( triggered()                  ),
            this               , SLOT  ( find_in_project_files_slot() ));
   // Project ...................................
   connect( OpenProject          , SIGNAL( triggered()                     ),
            this                 , SLOT  ( open_project_slot()             ));
   connect( CloseProject         , SIGNAL( triggered()                     ),
            this                 , SLOT  ( close_project_slot()            ));
   connect( NewProject           , SIGNAL( triggered()                     ),
            this                 , SLOT  ( new_project_slot()              ));
   connect( ProjectProperties    , SIGNAL( triggered()                     ),
            this                 , SLOT  ( project_properties_slot()       ));
   connect( AddFileToProject     , SIGNAL( triggered()                     ),
            this                 , SLOT  ( add_file_to_project_slot()      ));
   connect( RemoveFileFromProject, SIGNAL( triggered()                     ),
            this                 , SLOT  ( remove_file_from_project_slot() ));
   connect( Run                  , SIGNAL( triggered()                     ),
            this                 , SLOT  ( run_slot()                      ));
   connect( Build                , SIGNAL( triggered()                     ),
            this                 , SLOT  ( build_slot()                    ));
   connect( Rebuild              , SIGNAL( triggered()                     ),
            this                 , SLOT  ( rebuild_slot()                  ));
   connect( Break                , SIGNAL( triggered()                     ),
            this                 , SLOT  ( break_slot()                    ));
   // Debugger ..................................
   connect( DbgBreakpoint     , SIGNAL( triggered()                 ),
            this              , SLOT  ( dbg_breakpoint_slot()       ));
   connect( DbgBreakpointClear, SIGNAL( triggered()                 ),
            this              , SLOT  ( dbg_breakpoint_clear_slot() ));
   connect( DbgListBreakpoint , SIGNAL( triggered()                 ),
            this              , SLOT  ( dbg_list_breakpoint_slot()  ));
   connect( DbgRun            , SIGNAL( triggered()                 ),
            this              , SLOT  ( dbg_run_slot()              ));
   connect( DbgPause          , SIGNAL( triggered()                 ),
            this              , SLOT  ( dbg_pause_slot()            ));
   connect( DbgStepOver       , SIGNAL( triggered()                 ),
            this              , SLOT  ( dbg_step_over_slot()        ));
   connect( DbgStepInto       , SIGNAL( triggered()                 ),
            this              , SLOT  ( dbg_step_into_slot()        ));
   connect( DbgStepOut        , SIGNAL( triggered()                 ),
            this              , SLOT  ( dbg_step_out_slot()         ));
   connect( DbgContinue       , SIGNAL( triggered()                 ),
            this              , SLOT  ( dbg_continue_slot()         ));
   connect( DbgStop           , SIGNAL( triggered()                 ),
            this              , SLOT  ( dbg_stop_slot()             ));
   // Documents .................................
   connect( Close   , SIGNAL( triggered()      ),
            this    , SLOT  ( close_slot()     ));
   connect( CloseAll, SIGNAL( triggered()      ),
            this    , SLOT  ( close_all_slot() ));
   connect( Tile    , SIGNAL( triggered()      ),
            this    , SLOT  ( tile_slot()      ));
   connect( Cascade , SIGNAL( triggered()      ),
            this    , SLOT  ( cascade_slot()   ));
   // Help ........................................
   connect( d_help         , SIGNAL( triggered()        ),
            this           , SLOT( help_slot()          ));
   connect( d_about_program, SIGNAL( triggered()        ),
            this           , SLOT( about_program_slot() ));
   connect( d_about_qt     , SIGNAL( triggered()        ),
            this           , SLOT( about_qt_slot()      ));

   connect( d_document_group    , SIGNAL( triggered( QAction* )),
            this                , SLOT  ( document_group_triggered( QAction* )));
   connect( d_last_file_group   , SIGNAL( triggered( QAction* )               ),
            this                , SLOT  ( last_documents_triggered( QAction* )));
   connect( d_last_project_group, SIGNAL( triggered( QAction* )               ),
            this                , SLOT  ( last_projects_triggered( QAction*  )));
}
// end of update_actions

//*******************************************************************
// active_file_items                                         PRIVATE
//*******************************************************************
void MainWindow::active_file_items( const bool in_status )
{
   // File .........................................
   save_file_->setEnabled( in_status );
   save_file_as_->setEnabled( in_status );
   save_all_files_->setEnabled( in_status );
   // Edit .........................................
   paste_->setEnabled( in_status );
   delete_->setEnabled( in_status );
   select_all_->setEnabled( in_status );
   // Search .......................................
   Find->setEnabled( in_status );
   FindNext->setEnabled( in_status );
   FindPrevious->setEnabled( in_status );
   Replace->setEnabled( in_status );
   BookmarkNext->setEnabled( in_status );
   BookmarkPrev->setEnabled( in_status );
   BookmarkToggle->setEnabled( in_status );
   BookmarkAll->setEnabled( in_status );
   GotoLine->setEnabled( in_status );
   SwitchFiles->setEnabled( in_status );
   // Documents ....................................
   Close->setEnabled( in_status );
   CloseAll->setEnabled( in_status );
   Tile->setEnabled( in_status );
   Cascade->setEnabled( in_status );
}
// end of active_file_items

//*******************************************************************
// active_project_items                                      PRIVATE
//*******************************************************************
void MainWindow::active_project_items( const bool in_status )
{
   CloseProject->setEnabled( in_status );
   ProjectProperties->setEnabled( in_status );
   AddFileToProject->setEnabled( in_status );
   RemoveFileFromProject->setEnabled( in_status );
   Run->setEnabled( in_status );
   Build->setEnabled( in_status );
   Rebuild->setEnabled( in_status );
   Break->setEnabled( in_status );
   FindInProjectFiles->setEnabled( in_status );
}
// end of set_project_items

//*******************************************************************
// active_debug_items                                        PRIVATE
//*******************************************************************
void MainWindow::active_debug_items( const qint32 in_status )
{
   bool enable = ( DBG_ENABLE_RUN == in_status );
   DbgBreakpoint->setEnabled     ( enable );
   DbgBreakpointClear->setEnabled( enable );
   DbgListBreakpoint->setEnabled ( enable );
   DbgRun->setEnabled            ( enable );

   //---------------------------------------
   enable = ( DBG_DISABLE_RUN == in_status );
   DbgPause->setEnabled         ( enable );
   DbgStepOver->setEnabled      ( enable );
   DbgStepInto->setEnabled      ( enable );
   DbgStepOut->setEnabled       ( enable );
   DbgStop->setEnabled          ( enable );
   DbgContinue->setEnabled      ( enable );
}
// end of active_debug_items

//*******************************************************************
// process_items                                             PRIVATE
//*******************************************************************
void MainWindow::process_items( const bool in_status )
{
   Run->setEnabled( in_status );
   Build->setEnabled( in_status );
   Rebuild->setEnabled( in_status );
   Break->setEnabled( !in_status );
}
// end of process_items

//*******************************************************************
// create_menu                                               PRIVATE
//*******************************************************************
void MainWindow::create_menu()
{
   menuBar()->addMenu( menu_file_ );
   menu_file_->addAction( open_file_ );
   menu_file_->addAction( new_file_ );
   menu_file_->addAction( save_file_ );
   menu_file_->addAction( save_file_as_ );
   menu_file_->addAction( save_all_files_ );
   menu_file_->addSeparator();
   menu_file_->addAction( print_ );
   menu_file_->addSeparator();
   menu_file_->addMenu( last_opened_files_ );
   menu_file_->addMenu( last_opened_projects_ );
   menu_file_->addSeparator();
   menu_file_->addAction( exit_ );
   //............................................
   menuBar()->addMenu( menu_edit_ );
   menu_edit_->addAction( sidekick_->toggleViewAction() );
   menu_edit_->addAction( bottomkick_->toggleViewAction() );
   menu_edit_->addSeparator();
   menu_edit_->addAction( undo_ );
   menu_edit_->addAction( redo_ );
   menu_edit_->addAction( copy_ );
   menu_edit_->addAction( cut_ );
   menu_edit_->addAction( paste_ );
   menu_edit_->addAction( delete_ );
   menu_edit_->addSeparator();
   menu_edit_->addAction( select_all_ );
   menu_edit_->addSeparator();
   menu_edit_->addAction( properties_ );
   //............................................
   menuBar()->addMenu( menu_search_ );
   menu_search_->addAction( Find );
   menu_search_->addAction( FindNext );
   menu_search_->addAction( FindPrevious );
   menu_search_->addAction( Replace );
   menu_search_->addSeparator();
   menu_search_->addAction( BookmarkNext );
   menu_search_->addAction( BookmarkPrev );
   menu_search_->addAction( BookmarkToggle );
   menu_search_->addAction( BookmarkAll );
   menu_search_->addSeparator();
   menu_search_->addAction( GotoLine );
   menu_search_->addAction( SwitchFiles );
   menu_search_->addSeparator();
   menu_search_->addAction( FindInProjectFiles );
   //............................................
   menuBar()->addMenu( menu_project_ );
   menu_project_->addAction( OpenProject );
   menu_project_->addAction( CloseProject );
   menu_project_->addAction( NewProject );
   menu_project_->addAction( ProjectProperties );
   menu_project_->addSeparator();
   menu_project_->addAction( AddFileToProject );
   menu_project_->addAction( RemoveFileFromProject );
   menu_project_->addSeparator();
   menu_project_->addAction( Run );
   menu_project_->addAction( Build );
   menu_project_->addAction( Rebuild );
   menu_project_->addAction( Break );
   //............................................
   menuBar()->addMenu( menu_debug_ );
   menu_debug_->addAction( DbgBreakpoint );
   menu_debug_->addAction( DbgBreakpointClear );
   menu_debug_->addAction( DbgListBreakpoint );
   menu_debug_->addSeparator();
   menu_debug_->addAction( DbgRun );
   //menu_debug_->addAction( DbgPause );
   menu_debug_->addSeparator();
   menu_debug_->addAction( DbgStepOver );
   menu_debug_->addAction( DbgStepInto );
   menu_debug_->addAction( DbgStepOut );
   menu_debug_->addSeparator();
   menu_debug_->addAction( DbgContinue );
   menu_debug_->addAction( DbgStop );
   //............................................
   menuBar()->addMenu( menu_documents_ );
   menu_documents_->addAction( Close );
   menu_documents_->addAction( CloseAll );
   menu_documents_->addSeparator();
   menu_documents_->addAction( Tile );
   menu_documents_->addAction( Cascade );
   menu_documents_->addAction( document_separator_ );
   //..............................................
   menuBar()->addMenu( menu_languages_ );
   menu_languages_->addAction( english_ );
   menu_languages_->addAction( polish_ );
   //..............................................
   menuBar()->addMenu( menu_help_ );
   menu_help_->addAction( d_help );
   menu_help_->addSeparator();
   menu_help_->addAction( d_about_program );
   menu_help_->addAction( d_about_qt );
}
// end of create_menu

//*******************************************************************
// create_toolbar                                            PRIVATE
//*******************************************************************
void MainWindow::create_toolbar()
{
   addToolBar( file_toolbar_ );
      file_toolbar_->addAction( exit_ );
      file_toolbar_->addSeparator();
      file_toolbar_->addAction( new_file_ );
      file_toolbar_->addAction( open_file_ );
      file_toolbar_->addAction( save_file_ );
      file_toolbar_->addAction( save_all_files_ );
      file_toolbar_->addSeparator();
      file_toolbar_->addAction( print_fast_ );

   addToolBar( edit_toolbar_ );
      edit_toolbar_->addAction( sidekick_->toggleViewAction() );
      edit_toolbar_->addAction( bottomkick_->toggleViewAction() );
      edit_toolbar_->addSeparator();
      edit_toolbar_->addAction( undo_ );
      edit_toolbar_->addAction( redo_ );
      edit_toolbar_->addAction( copy_ );
      edit_toolbar_->addAction( cut_ );
      edit_toolbar_->addAction( paste_ );

   addToolBar( search_toolbar_ );
      search_toolbar_->addAction( FindPrevious );
      search_toolbar_->addAction( Find );
      search_toolbar_->addAction( FindNext );
   
   addToolBar( bookmark_toolbar_ );
      bookmark_toolbar_->addAction( BookmarkAll );
      bookmark_toolbar_->addAction( BookmarkPrev );
      bookmark_toolbar_->addAction( BookmarkToggle );
      bookmark_toolbar_->addAction( BookmarkNext );
   
   addToolBar( project_toolbar_ );
      project_toolbar_->addAction( Run );
      project_toolbar_->addAction( Build );
      project_toolbar_->addAction( Rebuild );
      project_toolbar_->addSeparator();
      project_toolbar_->addAction( Break );

   addToolBar( Qt::RightToolBarArea, dbg_toolbar_ );
      //d_dbg_toolbar->addAction( DbgPause );
      //d_dbg_toolbar->addSeparator();
      dbg_toolbar_->addAction( DbgStepOver );
      dbg_toolbar_->addAction( DbgStepInto );
      dbg_toolbar_->addAction( DbgStepOut );
      dbg_toolbar_->addSeparator();
      dbg_toolbar_->addAction( DbgContinue );
      dbg_toolbar_->addAction( DbgStop );
}
// end of create_toolbar

//*******************************************************************
// init_status_bar                                           PRIVATE
//*******************************************************************
void MainWindow::init_status_bar()
{
   const QFontMetrics fm( statusBar()->font() );
   const int dx = fm.width( "9999" );

   ColPrompt->setIndent( 2 );
   statusBar()->addPermanentWidget( ColPrompt );
   d_col_number->setMinimumWidth( dx );
   d_col_number->setMaximumWidth( dx );
   d_col_number->setAlignment( Qt::AlignCenter );
   statusBar()->addPermanentWidget( d_col_number );

   RowPrompt->setIndent( 2 );
   statusBar()->addPermanentWidget( RowPrompt );
   d_row_number->setMinimumWidth( dx );
   d_row_number->setMaximumWidth( dx );
   d_row_number->setAlignment(Qt::AlignHCenter);
   statusBar()->addPermanentWidget( d_row_number );
}
// end of init_status_bar

//*******************************************************************
// translate                                                 PRIVATE
//*******************************************************************
void MainWindow::translate()
{
   sidekick_->retranslate();
   bottomkick_->retranslate();
   
   setWindowTitle( Shared::full_program_name() );
   
   menu_file_     ->setTitle( tr( MENU_FILE_LABEL      ) );
   menu_edit_     ->setTitle( tr( MENU_EDIT_LABEL      ) );
   menu_search_   ->setTitle( tr( MENU_SEARCH_LABEL    ) );
   menu_project_  ->setTitle( tr( MENU_PROJECT_LABEL   ) );
   menu_debug_    ->setTitle( tr( MENU_DEBUG_LABEL     ) );
   menu_documents_->setTitle( tr( MENU_DOCUMENTS_LABEL ) );
   menu_languages_->setTitle( tr( MENU_LANGUAGES       ) );
   menu_help_     ->setTitle( tr( MENU_HELP_LABEL      ) );

   // File ......................................
   open_file_->setText( tr( OPEN_FILE_LABEL) );
   open_file_->setShortcut( tr(OPEN_FILE_SHORTCUT) );
      new_file_->setText( tr( NEW_FILE_LABEL) );
      new_file_->setShortcut( tr(NEW_FILE_SHORTCUT) );
   save_file_->setText( tr( SAVE_FILE_LABEL) );
   save_file_->setShortcut( tr(SAVE_FILE_SHORTCUT) );
      save_file_as_->setText( tr( SAVE_FILE_AS_LABEL) );
      save_file_as_->setShortcut( tr(SAVE_FILE_AS_SHORTCUT) );
   save_all_files_->setText( tr(SAVE_ALL_FILES_LABEL) );
   save_all_files_->setShortcut( tr(SAVE_ALL_FILES_SHORTCUT) );
      print_->setText( tr( PRINT_LABEL ) );
      print_fast_->setText( tr( PRINT_LABEL ) );
      print_fast_->setShortcut( tr( PRINT_FAST_SHORTCUT ) );
   last_opened_files_->setTitle( tr(LAST_OPENED_FILES_LABEL) );
   last_opened_projects_->setTitle( tr(LAST_OPENED_PROJECTS_LABEL) );
      exit_->setText( tr(EXIT_LABEL) );
      exit_->setShortcut( tr(EXIT_SHORTCUT) );
   // Edit ......................................
   undo_->setText( tr(UNDO_LABEL) );
   undo_->setShortcut( tr(UNDO_SHORTCUT) );
      redo_->setText( tr(REDO_LABEL) );
      redo_->setShortcut( tr(REDO_SHORTCUT) );
   cut_->setText( tr(CUT_LABEL) );
   cut_->setShortcut( tr(CUT_SHORTCUT) );
      copy_->setText( tr(COPY_LABEL) );
      copy_->setShortcut( tr(COPY_SHORTCUT) );
   paste_->setText( tr(PASTE_LABEL) );
   paste_->setShortcut( tr(PASTE_SHORTCUT) );
      delete_->setText( tr(DELETE_LABEL) );
   select_all_->setText( tr(SELECT_ALL_LABEL) );
   select_all_->setShortcut( tr(SELECT_ALL_SHORTCUT) );
      properties_->setText( tr(PROPERTIES_LABEL) );
   // Search ....................................
   Find->setText( tr(FIND_LABEL) );
   Find->setShortcut( tr(FIND_SHORTCUT) );
      FindNext->setText( tr(FIND_NEXT_LABEL) );
      FindNext->setShortcut( tr(FIND_NEXT_SHORTCUT) );
   FindPrevious->setText( tr(FIND_PREVIOUS_LABEL) );
   FindPrevious->setShortcut( tr(FIND_PREVIOUS_SHORTCUT) );
      Replace->setText( tr(REPLACE_LABEL) );
      Replace->setShortcut( tr(REPLACE_SHORTCUT) );
   BookmarkNext->setText( tr( BOOKMARK_NEXT_LABEL ) );
   BookmarkNext->setShortcut( tr( BOOKMARK_NEXT_SHORTCUT ) );
      BookmarkPrev->setText( tr( BOOKMARK_PREV_LABEL ) );
      BookmarkPrev->setShortcut( tr( BOOKMARK_PREV_SHORTCUT ) );
   BookmarkToggle->setText( tr( BOOKMARK_TOGGLE_LABEL ) );
   BookmarkToggle->setShortcut( tr( BOOKMARK_TOGGLE_SHORTCUT ) );
      BookmarkAll->setText( tr( BOOKMARK_ALL_LABEL ) );
      BookmarkAll->setShortcut( tr( BOOKMARK_ALL_SHORTCUT ) );
   GotoLine->setText( tr(GOTO_LINE_LABEL) );
   GotoLine->setShortcut( tr(GOTO_LINE_SHORTCUT) );
      SwitchFiles->setText( tr(SWITCH_FILES_LABEL) );
      SwitchFiles->setShortcut( tr(SWITCH_FILES_SHORTCUT) );
   FindInProjectFiles->setText( tr(FIND_IN_PROJECT_FILES_LABEL) );
   // Project ...................................
   OpenProject->setText( tr(OPEN_PROJECT_LABEL) );
      CloseProject->setText( tr(CLOSE_PROJECT_LABEL) );
   NewProject->setText( tr(NEW_PROJECT_LABEL) );
      ProjectProperties->setText( tr( PROPERTIES_LABEL ) );
   AddFileToProject->setText( tr(ADD_FILE_TO_PROJECT_LABEL) );
      RemoveFileFromProject->setText( tr(REMOVE_FILE_FROM_PROJECT_LABEL) );
   Run->setText( tr(RUN_LABEL) );
   Run->setShortcut( tr(RUN_SHORTCUT) );
      Build->setText( tr(BUILD_LABEL) );
      Build->setShortcut( tr(BUILD_SHORTCUT) );
   Rebuild->setText( tr(REBUILD_LABEL) );
   Rebuild->setShortcut( tr(REBUILD_SHORTCUT) );
      Break->setText( tr(BREAK_LABEL) );
   // Debugger ....................................
   DbgBreakpoint->setText( tr( DBG_BREAKPOINT_LABEL ) );
   DbgBreakpoint->setShortcut( tr( DBG_BREAKPOINT_SHORTCUT ) );
      DbgBreakpointClear->setText( tr( DBG_BREAKPOINT_CLEAR_LABEL ) );
   DbgListBreakpoint->setText( tr( DBG_LIST_BREAKPOINT_LABEL ) );
   DbgListBreakpoint->setShortcut( tr( DBG_LIST_BREAKPOINT_SHORTCUT ) );
      DbgRun->setText( tr( DBG_RUN_LABEL ) );
      DbgRun->setShortcut( tr( DBG_RUN_SHORTCUT ) );
   DbgPause->setText( tr( DBG_PAUSE_LABEL ) );
      DbgStepOver->setText( tr( DBG_STEP_OVER_LABEL ) );
      DbgStepOver->setShortcut( tr( DBG_STEP_OVER_SHORTCUT ) );
   DbgStepInto->setText( tr( DBG_STEP_INTO_LABEL ) );
   DbgStepInto->setShortcut( tr( DBG_STEP_INTO_SHORTCUT ) );
      DbgStepOut->setText( tr( DBG_STEP_OUT_LABEL ) );
      DbgStepOut->setShortcut( tr( DBG_STEP_OUT_SHORTCUT ) );
   DbgContinue->setText( tr( DBG_CONTINUE_LABEL ) );
      DbgStop->setText( tr( DBG_STOP_LABEL ) );
   // Documents ...................................
   Close->setText( tr(CLOSE_LABEL) );
   Close->setShortcut( tr(CLOSE_SHORTCUT) );
      CloseAll->setText( tr(CLOSE_ALL_LABEL) );
      CloseAll->setShortcut( tr(CLOSE_ALL_SHORTCUT) );
   Tile->setText( tr(TILE_LABEL) );
      Cascade->setText( tr(CASCADE_LABEL) );
   // Languages ...................................
   english_->setText( tr( ENGLISH_LABEL ) );
   polish_->setText( tr( POLISH_LABEL ) );
   // Help ........................................
   d_help->setText( tr( HELP_LABEL) );
   d_help->setShortcut( tr( HELP_SHORTCUT) );
      d_about_program->setText( tr( ABOUT_PROGRAM_LABEL) );
   d_about_qt->setText( tr( ABOUT_QT_LABEL ) );
}
// end of translate

//*******************************************************************
// customEvent                                     PRIVATE inherited
//*******************************************************************
void MainWindow::customEvent( QEvent* const in_event )
{
   Event* const event = dynamic_cast< Event* >( in_event );
   const qint32 type = static_cast< qint32 >( in_event->type() );
   
   switch( type ) {
      case Event::FILE_OPENED:
         file_opened( event );
         break;
      case Event::FILE_CLOSED:
         file_closed( event );
         break;
      case Event::ALL_FILES_CLOSED:
         all_files_closed();
         break;
      case Event::FILE_RENAMED:
         file_renamed( event );
         break;
      case Event::ACTIVE_DOCUMENT_CHANGED:
         active_document_changed( event );
         break;
      case Event::CURSOR_POSITION_CHANGED:
         cursor_position_changed( event );
         break;
      case Event::PROJECT_OPENED:
         project_opened( event );
         break;
      case Event::PROJECT_CLOSED:
         project_closed( event );
         break;
      case Event::COPY_AVAILABLE:
         copy_->setEnabled( event->d_data.at( 0 ).toBool() );
         cut_->setEnabled( event->d_data.at( 0 ).toBool() );
         break;
      case Event::REDO_AVAILABLE:
         redo_->setEnabled( event->d_data.at( 0 ).toBool() );
         break;
      case Event::UNDO_AVAILABLE:
         undo_->setEnabled( event->d_data.at( 0 ).toBool() );
         break;
      case Event::PROCESS_STARTED:
         process_items( false );
         break;
      case Event::PROCESS_BROKEN:
         QApplication::beep();
         process_items( true );
         break;
      case Event::DBG_STARTED:
         active_debug_items( DBG_DISABLE_RUN );
         active_project_items( false );
         break;
      case Event::DBG_FINISHED:
         active_debug_items( DBG_ENABLE_RUN );
         active_project_items( true );
         d_debugging = false;
         dbg_toolbar_->hide();
         break;
      case Event::DBG_DISABLE_ALL_REQUEST:
         active_debug_items( DBG_DISABLE_ALL );
         break;
      case Event::DBG_DISABLE_RUN_REQUEST:
         active_debug_items( DBG_DISABLE_RUN );
         break;
      case Event::RUN_REQUEST_EXTERNAL:
         run_slot();
         break;         
      case Event::DBG_RUN_REQUEST_EXTERNAL:
         dbg_run_slot();
         break;
   }
}
// end of customEvent

//*******************************************************************
// file_opened                                               PRIVATE
//*******************************************************************
void MainWindow::file_opened( Event* const in_event )
{
   const qint32 window_count = in_event->d_data.at( 0 ).toInt();
   const QString file_path = in_event->d_data.at( 1 ).toString(); 
 
   if( window_count > 0 ) {
      active_file_items( true );
   }
   add_to_opened_documents( file_path );
   add_to_last_opened_files( file_path );
}
// end of file_opened

//*******************************************************************
// project_opened                                            PRIVATE
//*******************************************************************
void MainWindow::project_opened( Event* const in_event )
{
   const QString file_name = in_event->d_data.at( 0 ).toString();
   const QString file_dir  = in_event->d_data.at( 1 ).toString();

   QString file_path = file_dir;
   if( !file_path.endsWith( QBtParser::SLASH ) ) file_path += QBtParser::SLASH;
   file_path += file_name;
   
   active_project_items( true );
   process_items( true );
   active_debug_items( DBG_ENABLE_RUN );
   add_to_last_opened_projects( file_path );
}
// end of project_opened

//*******************************************************************
// cursor_position_changed                                   PRIVATE
//*******************************************************************
void MainWindow::cursor_position_changed( Event* const in_event )
{
   static int prv_row = -1;
   static int prv_col = -1;
   
   const int row = in_event->d_data.at( 0 ).toInt();
   const int col = in_event->d_data.at( 1 ).toInt();

   if( row != prv_row ) {
      prv_row = row;
      d_row_number->setText( ( -1 == row ) ? QString() : QString::number( row ) );
   }
   if( col != prv_col ) {
      prv_col = col;
      d_col_number->setText( ( -1 == col ) ? QString() : QString::number( col ) );
   }
}
// end of cursor_position_changed

//*******************************************************************
// add_to_opened_documents                                   PRIVATE
//*******************************************************************
void MainWindow::add_to_opened_documents( const QString& in_file_path )
{
   QAction* const new_action = new QAction( this );
   if( new_action ) {
      foreach( QAction* action, d_documents ) {
         menu_documents_->removeAction( action );
      }
      
      const QFileInfo fi( in_file_path );
      new_action->setText( fi.fileName() );
      new_action->setStatusTip( fi.filePath() );
      new_action->setActionGroup( d_document_group );
      new_action->setCheckable( true );
      new_action->setChecked( true );
      
      d_documents.prepend( new_action );
      if( d_documents.size() > MAX_NUMBER_OPENED_DOCUMENTS ) {
         delete d_last_files.takeLast();   
      }

      foreach( QAction* action, d_documents ) {
         menu_documents_->addAction( action );
      }

      if( !document_separator_->isVisible() ) {
         document_separator_->setVisible( true );
      }
      
      renum_documents();
   }
}
// end of add_to_opened_documents

//*******************************************************************
// renum_documents                                           PRIVATE
//*******************************************************************
void MainWindow::renum_documents()
{
   qint32 idx = 0;
   QFileInfo fi;
   QString text;
   
   foreach( QAction* action, d_documents ) {
      fi.setFile( action->statusTip() );
      text  = "&";
      text += QString::number( ++idx );
      text += " ";
      text += fi.fileName();
      action->setText( text );
   }
}
// end of renum_documents

//*******************************************************************
// add_to_last_opened_files                                  PRIVATE
//*******************************************************************
void MainWindow::add_to_last_opened_files( const QString& in_file_path )
{
   if( in_file_path.startsWith( "(" ) ) return;

   // najpierw trzeba sprawdzic czy plik juz jest na liscie,
   // jesli jest, to nalezy go usunac.
   foreach( QAction* action, d_last_files ) {
      if( action->statusTip() == in_file_path ) {
         d_last_files.removeAll( action );
         delete action;
         break;
      }
   }
   
   // tworzymy nowa pozycje w menu
   QAction* const new_action = new QAction( this );
   if( new_action ) {
      const QFileInfo fi( in_file_path );
      new_action->setText( fi.fileName() );
      new_action->setStatusTip( fi.filePath() );

      foreach( QAction* action, d_last_files ) {
         last_opened_files_->removeAction( action );
      }

      d_last_files.prepend( new_action );
      if( d_last_files.size() > MAX_NUMBER_LAST_OPENED_FILES ) {
         delete d_last_files.takeLast();   
      }

      foreach( QAction* action, d_last_files ) {
         action->setActionGroup( d_last_file_group );
         last_opened_files_->addAction( action );
      }

      if( !last_opened_files_->isEnabled() ) {
         last_opened_files_->setEnabled( true );
      }
   }
}
// end of add_to_last_opened_files

//*******************************************************************
// add_to_last_opened_projects                               PRIVATE
//*******************************************************************
void MainWindow::add_to_last_opened_projects( const QString& in_file_path )
{
   // najpierw trzeba sprawdzic czy projekt juz jest na liscie
   // jesli jest to nalezy go usunac.
   foreach( QAction* action, d_last_projects ) {
      if( action->statusTip() == in_file_path ) {
         d_last_projects.removeAll( action );
         delete action;
         break;
      }
   }
   
   // tworzymy nowa pozycje w menu
   QAction* const new_action = new QAction( this );
   if( new_action ) {
      const QFileInfo fi( in_file_path );
      new_action->setText( fi.fileName() );
      new_action->setStatusTip( fi.filePath() );

      foreach( QAction* action, d_last_projects ) {
         last_opened_projects_->removeAction( action );
      }

      d_last_projects.prepend( new_action );
      if( d_last_files.size() > MAX_NUMBER_LAST_OPENED_PROJECTS ) {
         delete d_last_projects.takeLast();   
      }

      foreach( QAction* action, d_last_projects ) {
         action->setActionGroup( d_last_project_group );
         last_opened_projects_->addAction( action );
      }

      if( !last_opened_projects_->isEnabled() ) {
         last_opened_projects_->setEnabled( true );
      }
   }
}
// end of add_to_last_opened_projects

//*******************************************************************
// file_closed                                               PRIVATE
//*******************************************************************
void MainWindow::file_closed( Event* const in_event )
{
   const qint32  window_count = in_event->d_data.at( 0 ).toInt();
   const QString file_path = in_event->d_data.at( 1 ).toString(); 
   
   if( 0 == window_count ) {
      active_file_items( false );
   }
   
   foreach( QAction* action, d_documents ) {
      if( action->statusTip() == file_path ) {
         menu_documents_->removeAction( action );
         d_document_group->removeAction( action );
         d_documents.removeAll( action );
         delete action;
         break;
      }
   }
   renum_documents();
   
   if( 0 == d_documents.size() ) {
      document_separator_->setVisible( false );
   }
}
// end of file_closed

//*******************************************************************
// project_closed                                            PRIVATE
//*******************************************************************
void MainWindow::project_closed( Event* const )
{
   active_project_items( false );
   active_debug_items( DBG_DISABLE_ALL );
}
// end of project_closed

//*******************************************************************
// all_files_closed                                          PRIVATE
//*******************************************************************
void MainWindow::all_files_closed()
{
   active_file_items( false );
}
// end of file_opened

//*******************************************************************
// file_renamed                                              PRIVATE
//*******************************************************************
void MainWindow::file_renamed( Event* const in_event )
{
   const QString old_file_path = in_event->d_data.at( 0 ).toString(); 
   const QString cur_file_path = in_event->d_data.at( 1 ).toString(); 
   
   foreach( QAction* action, d_documents ) {
      if( action->statusTip() == old_file_path ) {
         const QFileInfo fi( cur_file_path );
         action->setStatusTip( fi.filePath() );
         break;
      }
   }
   renum_documents();

   if( old_file_path.startsWith( "(" ) ) {
      add_to_last_opened_files( cur_file_path );
   }
   else {
      foreach( QAction* action, d_last_files ) {
         if( action->statusTip() == old_file_path ) {
            const QFileInfo fi( cur_file_path );
            action->setText( fi.fileName() );
            action->setStatusTip( fi.filePath() );
            break;
         }
      }
   }
}
// end of file_renamed

//*******************************************************************
// active_document_chaned                                    PRIVATE
//-------------------------------------------------------------------
// Gdy uzytkownik wybral jakis dokument musimy to odwzorowac na
// liscie dokumentow. Pozycja na liscie dokumentow zostajnie
// zaznaczona.
//*******************************************************************
void MainWindow::active_document_changed( Event* const in_event )
{
   const QString cur_file_path = in_event->d_data.at( 0 ).toString(); 
   foreach( QAction* action, d_documents ) {
      if( action->statusTip() == cur_file_path ) {
         action->setChecked( true );
         break;
      }
   }
}
// end of active_document


//###################################################################
//#                                                                 #
//#          F I L E   -   P R I V A T E   S L O T S                #
//#                                                                 #
//###################################################################


//*******************************************************************
// open_file_slot                                       PRIVATE slot
//*******************************************************************
void MainWindow::open_file_slot()
{
   EventsController::instance()->send_event( Event::OPEN_FILE_REQUEST );
}
// end of open_file_slot

//*******************************************************************
// new_file_slot                                        PRIVATE slot
//*******************************************************************
void MainWindow::new_file_slot()
{
   EventsController::instance()->send_event( Event::NEW_FILE_REQUEST );
}
// end of new_file_slot

//*******************************************************************
// save_file_slot                                       PRIVATE slot
//*******************************************************************
void MainWindow::save_file_slot()
{
   EventsController::instance()->send_event( Event::SAVE_FILE_REQUEST );
}
// end of save_file_slot

//*******************************************************************
// save_file_as_slot                                    PRIVATE slot
//*******************************************************************
void MainWindow::save_file_as_slot()
{
   EventsController::instance()->send_event( Event::SAVE_FILE_AS_REQUEST );
}
// end of save_file_as_slot

//*******************************************************************
// save_all_files_slot                                  PRIVATE slot
//*******************************************************************
void MainWindow::save_all_files_slot()
{
   EventsController::instance()->send_event( Event::SAVE_ALL_FILES_REQUEST );
}
// end of save_all_files_slot

//*******************************************************************
// print                                                PRIVATE slot
//*******************************************************************
void MainWindow::print()
{
   EventsController::instance()->send_event( Event::PRINT_REQUEST );
}
// end of print

//*******************************************************************
// print_fast                                           PRIVATE slot
//*******************************************************************
void MainWindow::print_fast()
{
   EventsController::instance()->send_event( Event::PRINT_FAST_REQUEST );
}
// end of print_fast

//###################################################################
//#                                                                 #
//#          E D I T   -   P R I V A T E   S L O T S                #
//#                                                                 #
//###################################################################


//*******************************************************************
// undo_slot                                            PRIVATE slot
//*******************************************************************
void MainWindow::undo_slot()
{
   EventsController::instance()->send_event( Event::UNDO_REQUEST );
}
// end of undo_slot

//*******************************************************************
// redo_slot                                            PRIVATE slot
//*******************************************************************
void MainWindow::redo_slot()
{
   EventsController::instance()->send_event( Event::REDO_REQUEST );
}
// end of redo_slot

//*******************************************************************
// cut_slot                                             PRIVATE slot
//*******************************************************************
void MainWindow::cut_slot()
{
   EventsController::instance()->send_event( Event::CUT_REQUEST );
}
// end of cut_slot

//*******************************************************************
// copy_slot                                            PRIVATE slot
//*******************************************************************
void MainWindow::copy_slot()
{
   EventsController::instance()->send_event( Event::COPY_REQUEST );
}
// end of copy_slot

//*******************************************************************
// paste_slot                                           PRIVATE slot
//*******************************************************************
void MainWindow::paste_slot()
{
   EventsController::instance()->send_event( Event::PASTE_REQUEST );
}
// end of paste_slot

//*******************************************************************
// delete_slot                                          PRIVATE slot
//*******************************************************************
void MainWindow::delete_slot()
{
   EventsController::instance()->send_event( Event::DELETE_REQUEST );
}
// end of delete_slot

//*******************************************************************
// select_all_slot                                      PRIVATE slot
//*******************************************************************
void MainWindow::select_all_slot()
{
   EventsController::instance()->send_event( Event::SELECT_ALL_REQUEST );
}
// end of select_all_slot

//*******************************************************************
// properties_slot                                      PRIVATE slot
//*******************************************************************
void MainWindow::properties_slot()
{
   EventsController::instance()->send_event( Event::PROPERTIES_REQUEST );
}
// end of properties_slot


//###################################################################
//#                                                                 #
//#        S E A R C H   -   P R I V A T E   S L O T S              #
//#                                                                 #
//###################################################################


//*******************************************************************
// find_slot                                            PRIVATE slot
//*******************************************************************
void MainWindow::find_slot()
{
   EventsController::instance()->send_event( Event::FIND_REQUEST );
}
// end of find_slot

//*******************************************************************
// find_next_slot                                       PRIVATE slot
//*******************************************************************
void MainWindow::find_next_slot()
{
   EventsController::instance()->send_event( Event::FIND_NEXT_REQUEST );
}
// end of find_next_slot

//*******************************************************************
// find_previous_slot                                   PRIVATE slot
//*******************************************************************
void MainWindow::find_previous_slot()
{
   EventsController::instance()->send_event( Event::FIND_PREVIOUS_REQUEST );
}
// end of find_previous_slot

//*******************************************************************
// replace_slot                                         PRIVATE slot
//*******************************************************************
void MainWindow::replace_slot()
{
   EventsController::instance()->send_event( Event::REPLACE_REQUEST );
}
// end of replace_slot

//*******************************************************************
// bookmark_next_slot                                   PRIVATE slot
//*******************************************************************
void MainWindow::bookmark_next_slot()
{
   EventsController::instance()->send_event( Event::BOOKMARK_NEXT_REQUEST );
}
// end of bookmark_next_slot

//*******************************************************************
// bookmark_prev_slot                                   PRIVATE slot
//*******************************************************************
void MainWindow::bookmark_prev_slot()
{
   EventsController::instance()->send_event( Event::BOOKMARK_PREV_REQUEST );
}
// end of bookmark_prev_slot

//*******************************************************************
// bookmark_toggle_slot                                 PRIVATE slot
//*******************************************************************
void MainWindow::bookmark_toggle_slot()
{
   EventsController::instance()->send_event( Event::BOOKMARK_TOGGLE_REQUEST );
}
// end of bookmark_toggle_slot

//*******************************************************************
// bookmark_all_slot                                    PRIVATE slot
//*******************************************************************
void MainWindow::bookmark_all_slot()
{
   EventsController::instance()->send_event( Event::BOOKMARK_ALL_REQUEST );
}
// end of bookmark_all_slot

//*******************************************************************
// goto_line_slot                                       PRIVATE slot
//*******************************************************************
void MainWindow::goto_line_slot()
{
   EventsController::instance()->send_event( Event::GOTO_LINE_REQUEST );
}
// end of goto_line_slot

//*******************************************************************
// switch_files_slot                                    PRIVATE slot
//*******************************************************************
void MainWindow::switch_files_slot()
{
   EventsController::instance()->send_event( Event::SWITCH_FILES_REQUEST );
}
// end of switch_files_slot

//*******************************************************************
// find_in_project_files_slot                           PRIVATE slot
//*******************************************************************
void MainWindow::find_in_project_files_slot()
{
   EventsController::instance()->send_event( Event::FIND_IN_PROJECT_FILES_REQUEST );
}
// end of find_in_project_files_slot


//###################################################################
//#                                                                 #
//#        P R O J E C T   -   P R I V A T E   S L O T S            #
//#                                                                 #
//###################################################################


//*******************************************************************
// open_project_slot                                    PRIVATE slot
//*******************************************************************
void MainWindow::open_project_slot()
{
   EventsController::instance()->send_event( Event::OPEN_PROJECT_REQUEST );
}
// end of open_project_slot

//*******************************************************************
// close_project_slot                                   PRIVATE slot
//*******************************************************************
void MainWindow::close_project_slot()
{
   EventsController::instance()->send_event( Event::CLOSE_PROJECT_REQUEST );
}
// end of close_project_slot

//*******************************************************************
// new_project_slot                                     PRIVATE slot
//*******************************************************************
void MainWindow::new_project_slot()
{
   EventsController::instance()->send_event( Event::NEW_PROJECT_REQUEST );
}
// end of new_project_slot

//*******************************************************************
// project_properties_slot                              PRIVATE slot
//*******************************************************************
void MainWindow::project_properties_slot()
{
   EventsController::instance()->send_event( Event::PROJECT_PROPERTIES_REQUEST );
}
// end of project_properties_slot

//*******************************************************************
// add_file_to_project_slot                             PRIVATE slot
//*******************************************************************
void MainWindow::add_file_to_project_slot()
{
   EventsController::instance()->send_event( Event::ADD_FILE_TO_PROJECT_REQUEST );
}
// end of add_file_to_project

//*******************************************************************
// remove_file_from_project_slot                        PRIVATE slot
//*******************************************************************
void MainWindow::remove_file_from_project_slot()
{
   EventsController::instance()->send_event( Event::REMOVE_FILE_FROM_PROJECT_REQUEST );
}
// end of remove_file_from_project

//*******************************************************************
// run_slot                                             PRIVATE slot
//*******************************************************************
void MainWindow::run_slot()
{
   EventsController::instance()->send_event( Event::SAVE_ALL_FILES_REQUEST );
   EventsController::instance()->send_event( Event::RUN_REQUEST );
}
// end of run_slot

//*******************************************************************
// build_slot                                           PRIVATE slot
//*******************************************************************
void MainWindow::build_slot()
{
   EventsController::instance()->send_event( Event::SAVE_ALL_FILES_REQUEST );
   EventsController::instance()->send_event( Event::BUILD_REQUEST );
}
// end of build_slot

//*******************************************************************
// rebuild_slot                                         PRIVATE slot
//*******************************************************************
void MainWindow::rebuild_slot()
{
   EventsController::instance()->send_event( Event::SAVE_ALL_FILES_REQUEST );
   EventsController::instance()->send_event( Event::REBUILD_REQUEST );
}
// end of rebuild_slot

//*******************************************************************
// break_slot                                           PRIVATE slot
//*******************************************************************
void MainWindow::break_slot()
{
   EventsController::instance()->send_event( Event::BREAK_REQUEST );
}
// end of break_slot


//###################################################################
//#                                                                 #
//#         D E B U G G E R   -   P R I V A T E   S L O T S         #
//#                                                                 #
//###################################################################


//*******************************************************************
// dbg_breakpoint_slot                                  PRIVATE slot
//*******************************************************************
void MainWindow::dbg_breakpoint_slot()
{
   EventsController::instance()->send_event( Event::DBG_BREAKPOINT_REQUEST );
}
// end of dbg_breakpoint_slot

//*******************************************************************
// dbg_breakpoint_clear_slot                            PRIVATE slot
//*******************************************************************
void MainWindow::dbg_breakpoint_clear_slot()
{
   Breakpoints::instance()->clear();
   EventsController::instance()->send_event( Event::DBG_BREAKPOINT_CLEAR_ALL_REQUEST );
}
// end of dbg_breakpoint_clear_slot

//*******************************************************************
// dbg_list_breakpoint_slot                             PRIVATE slot
//*******************************************************************
void MainWindow::dbg_list_breakpoint_slot()
{
   EventsController::instance()->send_event( Event::DBG_LIST_BREAKPOINT_REQUEST );
}
// end of dbg_list_breakpoint_slot

//*******************************************************************
// dbg_run_slot                                         PRIVATE slot
//*******************************************************************
void MainWindow::dbg_run_slot()
{
   d_debugging = true;
   dbg_toolbar_->show();
   EventsController::instance()->send_event( Event::DBG_RUN_REQUEST );
}
// end of dbg_run_slot

//*******************************************************************
// dbg_pause_slot                                       PRIVATE slot
//*******************************************************************
void MainWindow::dbg_pause_slot()
{
   EventsController::instance()->send_event( Event::DBG_PAUSE_REQUEST );
}
// end of dbg_pause_slot

//*******************************************************************
// dbg_step_over_slot                                   PRIVATE slot
//*******************************************************************
void MainWindow::dbg_step_over_slot()
{
   EventsController::instance()->send_event( Event::DBG_STEP_OVER_REQUEST );
}
// end of dbg_step_over_slot

//*******************************************************************
// dbg_step_into_slot                                   PRIVATE slot
//*******************************************************************
void MainWindow::dbg_step_into_slot()
{
   EventsController::instance()->send_event( Event::DBG_STEP_INTO_REQUEST );
}
// end of dbg_step_into_slot

//*******************************************************************
// dbg_step_out_slot                                    PRIVATE slot
//*******************************************************************
void MainWindow::dbg_step_out_slot()
{
   EventsController::instance()->send_event( Event::DBG_STEP_OUT_REQUEST );
}
// end of dbg_step_out_slot

//*******************************************************************
// dbg_continue_slot                                    PRIVATE slot
//*******************************************************************
void MainWindow::dbg_continue_slot()
{
   EventsController::instance()->send_event( Event::DBG_CONTINUE_REQUEST );
}
// end of dbg_continue_slot

//*******************************************************************
// dbg_stop_slot                                        PRIVATE slot
//*******************************************************************
void MainWindow::dbg_stop_slot()
{
   d_debugging = false;
   dbg_toolbar_->hide();
   EventsController::instance()->send_event( Event::DBG_STOP_REQUEST );
}
// end of dbg_stop


//###################################################################
//#                                                                 #
//#        D O C U M E N T S   -   P R I V A T E   S L O T S        #
//#                                                                 #
//###################################################################


//*******************************************************************
// close_slot                                           PRIVATE slot
//*******************************************************************
void MainWindow::close_slot()
{
   EventsController::instance()->send_event( Event::CLOSE_REQUEST );
}
// end of close_slot

//*******************************************************************
// close_all_slot                                       PRIVATE slot
//*******************************************************************
void MainWindow::close_all_slot()
{
   EventsController::instance()->send_event( Event::CLOSE_ALL_REQUEST );
}
// end of close_all_slot

//*******************************************************************
// tile_slot                                            PRIVATE slot
//*******************************************************************
void MainWindow::tile_slot()
{
   EventsController::instance()->send_event( Event::TILE_REQUEST );
}
// end of tile_slot

//*******************************************************************
// cascade_slot                                         PRIVATE slot
//*******************************************************************
void MainWindow::cascade_slot()
{
   EventsController::instance()->send_event( Event::CASCADE_REQUEST );
}    
// end of cascade_slot

//*******************************************************************
// language_slot                                        PRIVATE slot
//*******************************************************************
void MainWindow::language_slot( QAction* in_action )
{
   Translator::Language lang = Translator::ENGLISH;
   if( in_action == polish_ ) {
      lang = Translator::POLISH;
   }

  Translator::instance()->change_to( lang );
  translate();
}
// end of language_slot

//*******************************************************************
// help_slot                                            PRIVATE slot
//*******************************************************************
void MainWindow::help_slot()
{
}
// end of help_slot

//*******************************************************************
// about_program_slot                                   PRIVATE slot
//-------------------------------------------------------------------
// Wyswietlanie informacji o programie.
//*******************************************************************
void MainWindow::about_program_slot()
{
   const QString text =
      "<qt>"
      "<h3>%1</h3>"
      "<font color=blue>This is a platform independent IDE for programming with Qt.</font>"
      "<hr>"
      "Author: Piotr Pszczolkowski<br>"
      "Contact: piotr@beesoft.pl<br>"
      "Home page: www.beesoft.org"
      "</qt>";
   QMessageBox::about( this, "About Cobras", text.arg( Shared::full_program_name() ) );
}
// end of about_program_slot

//*******************************************************************
// about_qt_slot                                        PRIVATE slot
//-------------------------------------------------------------------
// Wyswietlenie informacji o Qt.
//*******************************************************************
void MainWindow::about_qt_slot()
{
   QMessageBox::aboutQt( this );
}
// end of about_qt_slot

//*******************************************************************
// document_group_triggered                             PRIVATE slot
//-------------------------------------------------------------------
// Ten slot jest wywolywany gdy uzytkownik wskaze ktorys z
// dokumentow znajdujacy sie na liscie otwartych dokumentow.
//*******************************************************************
void MainWindow::document_group_triggered( QAction* const in_action )
{
   EventsController::instance()->send_event( Event::DOCUMENT_SELECTED_REQUEST, in_action->statusTip() );
}
// end of document_group_triggered

//*******************************************************************
// last_documents_triggered                             PRIVATE slot
//*******************************************************************
void MainWindow::last_documents_triggered( QAction* const in_action )
{
   EventsController::instance()->send_event( Event::DOCUMENT_SELECTED_REQUEST, in_action->statusTip() );
}
// end of last_documents_triggered

//*******************************************************************
// last_projects_triggered                              PRIVATE slot
//*******************************************************************
void MainWindow::last_projects_triggered( QAction* const in_action )
{
   EventsController::instance()->send_event( Event::PROJECT_SELECTED_REQUEST, in_action->statusTip() );
}
// end of last_projects_triggered
