#ifndef INCLUDED_MainWindow_h
#define INCLUDED_MainWindow_h

/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : MainWindow.h
 * DATE   : 2.09.2006
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include <QMainWindow>

/*------- forward declarations:
-------------------------------------------------------------------*/
class QBtWorkspace;
class Event;
class QAction;
class QActionGroup;
class QMenu;
class QLabel;
class QSplitter;
class QStatusBar;
class QToolBar;
class QTranslator;
class QBtSidekick;
class QBtBottomkick;

/*------- class declaration:
-------------------------------------------------------------------*/
class MainWindow : public QMainWindow
{
   Q_OBJECT

// ******* TYPES *******
private:
   enum {
      MAX_NUMBER_LAST_OPENED_FILES = 9,
      MAX_NUMBER_LAST_OPENED_PROJECTS = 9,
      MAX_NUMBER_OPENED_DOCUMENTS = 9,
      DBG_DISABLE_ALL = 0,
      DBG_ENABLE_RUN = 1,
      DBG_DISABLE_RUN = 2,
   };

// ******* CONSTRUCTION/DESTRUCTION *******
public:
   MainWindow();
   ~MainWindow();
private:
   MainWindow( const MainWindow& );
   MainWindow& operator=( const MainWindow& );

// ******* CONSTANTS *******
private:
   // LABELS
   static const char* const MENU_FILE_LABEL;
   static const char* const MENU_EDIT_LABEL;
   static const char* const MENU_SEARCH_LABEL;
   static const char* const MENU_PROJECT_LABEL;
   static const char* const MENU_DEBUG_LABEL;
   static const char* const MENU_DOCUMENTS_LABEL;
   static const char* const MENU_LANGUAGES;
   static const char* const MENU_HELP_LABEL;
   // File ......................................
   static const char* const OPEN_FILE_LABEL;
   static const char* const NEW_FILE_LABEL;
   static const char* const SAVE_FILE_LABEL;
   static const char* const SAVE_FILE_AS_LABEL;
   static const char* const SAVE_ALL_FILES_LABEL;
   static const char* const PRINT_LABEL;
   static const char* const LAST_OPENED_FILES_LABEL;
   static const char* const LAST_OPENED_PROJECTS_LABEL;
   static const char* const EXIT_LABEL;
   // Edit ......................................
   static const char* const UNDO_LABEL;
   static const char* const REDO_LABEL;
   static const char* const CUT_LABEL;
   static const char* const COPY_LABEL;
   static const char* const PASTE_LABEL;
   static const char* const DELETE_LABEL;
   static const char* const SELECT_ALL_LABEL;
   static const char* const PROPERTIES_LABEL;
   // Search ....................................
   static const char* const FIND_LABEL;
   static const char* const FIND_NEXT_LABEL;
   static const char* const FIND_PREVIOUS_LABEL;
   static const char* const REPLACE_LABEL;
   static const char* const BOOKMARK_NEXT_LABEL;
   static const char* const BOOKMARK_PREV_LABEL;
   static const char* const BOOKMARK_TOGGLE_LABEL;
   static const char* const BOOKMARK_ALL_LABEL;
   static const char* const GOTO_LINE_LABEL;
   static const char* const SWITCH_FILES_LABEL;
   static const char* const FIND_IN_PROJECT_FILES_LABEL;
   // Project ...................................
   static const char* const OPEN_PROJECT_LABEL;
   static const char* const CLOSE_PROJECT_LABEL;
   static const char* const NEW_PROJECT_LABEL;
   static const char* const ADD_FILE_TO_PROJECT_LABEL;
   static const char* const REMOVE_FILE_FROM_PROJECT_LABEL;
   static const char* const RUN_LABEL;
   static const char* const BUILD_LABEL;
   static const char* const REBUILD_LABEL;
   static const char* const BREAK_LABEL;
   // Debugger ....................................
   static const char* const DBG_BREAKPOINT_LABEL;
   static const char* const DBG_BREAKPOINT_CLEAR_LABEL;
   static const char* const DBG_LIST_BREAKPOINT_LABEL;
   static const char* const DBG_RUN_LABEL;
   static const char* const DBG_STEP_OVER_LABEL;
   static const char* const DBG_STEP_INTO_LABEL;
   static const char* const DBG_STEP_OUT_LABEL;
   static const char* const DBG_PAUSE_LABEL;
   static const char* const DBG_CONTINUE_LABEL;
   static const char* const DBG_STOP_LABEL;
   // Documents .................................
   static const char* const CLOSE_LABEL;
   static const char* const CLOSE_ALL_LABEL;
   static const char* const TILE_LABEL;
   static const char* const CASCADE_LABEL;
   // Languages  ................................
   static const char* const ENGLISH_LABEL;
   static const char* const POLISH_LABEL;
   // Help ......................................
   static const char* const HELP_LABEL;
   static const char* const ABOUT_PROGRAM_LABEL;
   static const char* const ABOUT_QT_LABEL;
   
   // SHORTCUTS   
   // File ......................................
   static const char* const OPEN_FILE_SHORTCUT;
   static const char* const NEW_FILE_SHORTCUT;
   static const char* const SAVE_FILE_SHORTCUT;
   static const char* const SAVE_FILE_AS_SHORTCUT;
   static const char* const SAVE_ALL_FILES_SHORTCUT;
   static const char* const PRINT_FAST_SHORTCUT;
   static const char* const EXIT_SHORTCUT;
   // Edit ......................................
   static const char* const UNDO_SHORTCUT;
   static const char* const REDO_SHORTCUT;
   static const char* const CUT_SHORTCUT;
   static const char* const COPY_SHORTCUT;
   static const char* const PASTE_SHORTCUT;
   static const char* const SELECT_ALL_SHORTCUT;
   // Search ....................................
   static const char* const FIND_SHORTCUT;
   static const char* const FIND_NEXT_SHORTCUT;
   static const char* const FIND_PREVIOUS_SHORTCUT;
   static const char* const REPLACE_SHORTCUT;
   static const char* const BOOKMARK_NEXT_SHORTCUT;
   static const char* const BOOKMARK_PREV_SHORTCUT;
   static const char* const BOOKMARK_TOGGLE_SHORTCUT;
   static const char* const BOOKMARK_ALL_SHORTCUT;
   static const char* const GOTO_LINE_SHORTCUT;
   static const char* const SWITCH_FILES_SHORTCUT;
   // Project ...................................
   static const char* const RUN_SHORTCUT;
   static const char* const BUILD_SHORTCUT;
   static const char* const REBUILD_SHORTCUT;
   // Debugger ..................................
   static const char* const DBG_BREAKPOINT_SHORTCUT;
   static const char* const DBG_LIST_BREAKPOINT_SHORTCUT;
   static const char* const DBG_RUN_SHORTCUT;
   static const char* const DBG_STEP_OVER_SHORTCUT;
   static const char* const DBG_STEP_INTO_SHORTCUT;
   static const char* const DBG_STEP_OUT_SHORTCUT;
   // Documents .................................
   static const char* const CLOSE_SHORTCUT;
   static const char* const CLOSE_ALL_SHORTCUT;
   // Help ........................................
   static const char* const HELP_SHORTCUT;
   //............................................
   static const char* const COL_LABEL;
   static const char* const ROW_LABEL;
   static const char* const WINDOW_KEY;
   static const char* const WINDOW_POS_KEY;
   static const char* const WINDOW_SIZE_KEY;
   static const char* const HOR_SPLITTER_KEY;
   static const char* const VER_SPLITTER_KEY;
   static const char* const MENU_FILES_KEY;
   static const char* const MENU_PROJECTS_KEY;
      
// ******* MEMBERS *******
private:
   QMenu*      const menu_file_;
   QMenu*      const menu_edit_;
   QMenu*      const menu_search_;
   QMenu*      const menu_project_;
   QMenu*      const menu_debug_;
   QMenu*      const menu_documents_;
   QMenu*      const menu_languages_;
   QMenu*      const menu_help_;
   // Toolbars ..................................
   QToolBar*   const file_toolbar_;
   QToolBar*   const edit_toolbar_;
   QToolBar*   const search_toolbar_;
   QToolBar*   const bookmark_toolbar_;
   QToolBar*   const project_toolbar_;
   QToolBar*   const documents_toolbar_;
   QToolBar*   const dbg_toolbar_;
   // File ......................................
   QAction*    const open_file_;
   QAction*    const new_file_;
   QAction*    const save_file_;
   QAction*    const save_file_as_;
   QAction*    const save_all_files_;
   QAction*    const print_;
   QAction*    const print_fast_;
   QMenu*      const last_opened_files_;
   QMenu*      const last_opened_projects_;
   QAction*    const exit_;
   // Edit ......................................
   QAction*    const undo_;
   QAction*    const redo_;
   QAction*    const cut_;
   QAction*    const copy_;
   QAction*    const paste_;
   QAction*    const delete_;
   QAction*    const select_all_;
   QAction*    const properties_;
   // Search ....................................
   QAction*    const Find;
   QAction*    const FindNext;
   QAction*    const FindPrevious;
   QAction*    const Replace;
   QAction*    const BookmarkNext;
   QAction*    const BookmarkPrev;
   QAction*    const BookmarkToggle;
   QAction*    const BookmarkAll;
   QAction*    const GotoLine;
   QAction*    const SwitchFiles;
   QAction*    const FindInProjectFiles;
   // Project ...................................
   QAction*    const OpenProject;
   QAction*    const CloseProject;
   QAction*    const NewProject;
   QAction*    const ProjectProperties;
   QAction*    const AddFileToProject;
   QAction*    const RemoveFileFromProject;
   QAction*    const Run;
   QAction*    const Build;
   QAction*    const Rebuild;
   QAction*    const Break;
   // Debugger .................................
   QAction*    const DbgBreakpoint;
   QAction*    const DbgBreakpointClear;
   QAction*    const DbgListBreakpoint;
   QAction*    const DbgRun;
   QAction*    const DbgPause;
   QAction*    const DbgStepOver;
   QAction*    const DbgStepInto;
   QAction*    const DbgStepOut;
   QAction*    const DbgContinue;
   QAction*    const DbgStop;
   // Documents .................................
   QAction*    const Close;
   QAction*    const CloseAll;
   QAction*    const Tile;
   QAction*    const Cascade;
   QAction*    const document_separator_;
   // Languages .................................
   QAction*    const english_;
   QAction*    const polish_;
   // Help.......................................
   QAction*    const d_help;
   QAction*    const d_about_program;
   QAction*    const d_about_qt;

   QActionGroup* const d_document_group;
   QList<QAction*>     d_documents;
   QActionGroup* const d_last_project_group;
   QList<QAction*>     d_last_projects;
   QActionGroup* const d_last_file_group;
   QList<QAction*>     d_last_files;
   //............................................
   QBtWorkspace*  const workspace_;
   QBtSidekick*   const sidekick_;
   QBtBottomkick* const bottomkick_;
   QLabel*        const ColPrompt;
   QLabel*        const d_col_number;
   QLabel*        const RowPrompt;
   QLabel*        const d_row_number;
   bool                 d_first_time;
   bool                 d_debugging;

// ******* METHODS *******
private:
   void set_initial_lang_menu_item  ();
   void save_settings               ();
   void restore_settings            ();
   void showEvent                   ( QShowEvent* );
   void closeEvent                  ( QCloseEvent* );
   void customEvent                 ( QEvent* );
   void update_actions              ();
   void active_file_items           ( bool );
   void active_project_items        ( bool );
   void active_debug_items          ( qint32 );
   void process_items               ( bool );
   void create_menu                 ();
   void create_toolbar              ();
   void init_status_bar             ();
   void translate                   ();
   void file_opened                 ( Event* );
   void project_opened              ( Event* );
   void add_to_opened_documents     ( const QString& );
   void add_to_last_opened_files    ( const QString& );
   void add_to_last_opened_projects ( const QString& );
   void file_closed                 ( Event* );
   void project_closed              ( Event* );
   void all_files_closed            ();
   void file_renamed                ( Event* );
   void active_document_changed     ( Event* );
   void renum_documents             ();
   void cursor_position_changed     ( Event* );
   void project_from_command_line   ( QString& );
private slots:
   // File ......................................
   void open_file_slot              ();
   void new_file_slot               ();
   void save_file_slot              ();
   void save_file_as_slot           ();
   void save_all_files_slot         ();
   void print                       ();
   void print_fast                  ();
   // Edit ......................................
   void undo_slot                   ();
   void redo_slot                   ();
   void cut_slot                    ();
   void copy_slot                   ();
   void paste_slot                  ();
   void delete_slot                 ();
   void select_all_slot             ();
   void properties_slot             ();
   // Search ....................................
   void find_slot                   ();
   void find_next_slot              ();
   void find_previous_slot          ();
   void replace_slot                ();
   void bookmark_next_slot          ();
   void bookmark_prev_slot          ();
   void bookmark_toggle_slot        ();
   void bookmark_all_slot           ();
   void goto_line_slot              ();
   void switch_files_slot           ();
   void find_in_project_files_slot  ();
   // Project ...................................
   void open_project_slot            ();
   void close_project_slot           ();
   void new_project_slot             ();
   void project_properties_slot      ();
   void add_file_to_project_slot     ();
   void remove_file_from_project_slot();
   void run_slot                     ();
   void build_slot                   ();
   void rebuild_slot                 ();
   void break_slot                   ();
   // Debugger ....................................
   void dbg_breakpoint_slot         ();
   void dbg_breakpoint_clear_slot   ();
   void dbg_list_breakpoint_slot    ();
   void dbg_run_slot                ();
   void dbg_pause_slot              ();
   void dbg_step_over_slot          ();
   void dbg_step_into_slot          ();
   void dbg_step_out_slot           ();
   void dbg_continue_slot           ();
   void dbg_stop_slot               ();
   // Documents ...................................
   void close_slot                  ();
   void close_all_slot              ();
   void tile_slot                   ();
   void cascade_slot                ();
   // Languages..................................
   void language_slot               ( QAction* );
   // Help.......................................
   void help_slot                   ();
   void about_program_slot          ();
   void about_qt_slot               ();
   void document_group_triggered    ( QAction* );
   void last_documents_triggered    ( QAction* );
   void last_projects_triggered     ( QAction* );
};

#endif // INCLUDED_MainWindow_h
