/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : Project.cpp
 * DATE   : 28.09.2006
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "Project.h"
#include "ProjectProp.h"
#include "QtMake.h"
#include "QBtParser.h"
#include "Shared.h"
#include <QFileInfo>
#include <QFile>
#include <QDir>
#include <QTextStream>
#include <QMessageBox>
#include <QPointer>
#include <QtDebug>
using namespace std;

/*------- local constants:
-------------------------------------------------------------------*/
const char* const Project::CAPTION                = QT_TR_NOOP( "Qt Project" );
const char* const Project::CANT_OPEN_FILE         = QT_TR_NOOP( "Can't open file %1\n(%2)" );
const char* const Project::CANT_SAVE_FILE         = QT_TR_NOOP( "Can't save file %1\n(%2)" );
const char* const Project::PROJECT_FILE_NOT_FOUND = QT_TR_NOOP( "Project file not found.\n(%1)" );
const char* const Project::QT_CORE                = "core";
const char* const Project::QT_GUI                 = "gui";
const char* const Project::QT_NETWORK             = "network";
const char* const Project::QT_OPENGL              = "opengl";
const char* const Project::QT_SQL                 = "sql";
const char* const Project::QT_SVG                 = "svg";
const char* const Project::QT_XML                 = "xml";
const char* const Project::CFG_QT                 = "qt";
const char* const Project::CFG_X11                = "x11";
const char* const Project::CFG_THREAD             = "thread";
const char* const Project::CFG_RELEASE            = "release";
const char* const Project::CFG_DEBUG              = "debug";
const char* const Project::CFG_RELEASE_DEBUG      = "release_and_debug";
const char* const Project::CFG_BUILD_ALL          = "build_all";
const char* const Project::CFG_ORDERED            = "ordered";
const char* const Project::CFG_WARN_ON            = "warn_on";
const char* const Project::CFG_WARN_OFF           = "warn_off";


//*******************************************************************
// Project                                               CONSTRUCTOR
//*******************************************************************
Project::Project()
: QObject     ()
, d_is_opened ( false )
, d_idx       ( 0 )
{}
// end of Project

//*******************************************************************
// change_properties                                          PUBLIC
//*******************************************************************
bool Project::change_properties()
{
   bool retval = false;   // reopen
   
   if( d_is_opened ) {
      QPointer< ProjectProp > dlg = new ProjectProp();
      QString fpath;
      QString tmpl;
      QHash< QString, bool > qt;
      QHash< QString, bool > cfg;
      QStringList src_files;
      QStringList hdr_files;
      QStringList ts_files;
      QStringList ui_files;
      QStringList qrc_files;

      get_properties( fpath, tmpl, qt, cfg  );
      get_var_files( Shared::SRC, src_files );
      get_var_files( Shared::HDR, hdr_files );
      get_var_files( Shared::TS , ts_files  );
      get_var_files( Shared::UI , ui_files  );
      get_var_files( Shared::RSC, qrc_files );

      dlg->set_general_data( fpath, tmpl, qt, cfg );
      dlg->set_files( src_files, hdr_files, ts_files, ui_files, qrc_files );
      dlg->set_qt_make( d_qt_make );
      
      if( QDialog::Accepted == dlg->exec() ) {
         dlg->get_general_data( fpath, tmpl, qt, cfg );
         dlg->get_files( src_files, hdr_files, ts_files, ui_files, qrc_files );
         dlg->get_qt_make( d_qt_make );

         d_idx = 0;
         set_properties( fpath, tmpl, qt, cfg );
         if( dlg->changed_files() ) {
            set_var_files( Shared::SRC, src_files );
            set_var_files( Shared::HDR, hdr_files );
            set_var_files( Shared::TS , ts_files  );
            set_var_files( Shared::UI , ui_files  );
            set_var_files( Shared::RSC, qrc_files );
            retval = true;
         }
         set_qt_make();
      }
      if( !save() ) {
         if( !retval ) {
            retval = true;
         }
      }
   }
   return retval;
}
// end of change_properties

//*******************************************************************
// get_properties                                            PRIVATE
//-------------------------------------------------------------------
// Wpisanie aktualnych ustawien projektu do wskazanych zmiennych.
//*******************************************************************
void Project::get_properties(   QString& out_fpath,
                         QString& out_tmpl,
                        QHash< QString, bool >& out_qt,
                        QHash< QString, bool >& out_cfg )
{
   out_fpath = d_file_info.absoluteFilePath();

   get_tmpl( out_tmpl );
   get_qt( out_qt );
   get_cfg( out_cfg );   
}
// end of get_properties

//*******************************************************************
// get_tmpl                                                  PRIVATE
//*******************************************************************
void Project::get_tmpl( QString& out_tmpl )
{
   out_tmpl = "app";

   const QString varname = "TEMPLATE";
   if( d_data.contains( varname ) ) {
      const MarkValues& mark_values = d_data[ varname ];
      if( mark_values.contains( MARK_EQU ) ) {
         if( !mark_values[ MARK_EQU ].empty() ) {
            out_tmpl = mark_values[ MARK_EQU ].toList().first();
         }
      }
   }
}
// end of get_tmpl

//*******************************************************************
// get_qt                                                    PRIVATE
//*******************************************************************
void Project::get_qt( QHash< QString, bool >& out_qt )
{
   out_qt.clear();
   const QString varname = "QT";

   // WARTOSCI DOMYSLNE
   out_qt[ QT_CORE    ] = true;
   out_qt[ QT_GUI     ] = true;
   out_qt[ QT_NETWORK ] = false;
   out_qt[ QT_OPENGL  ] = false;
   out_qt[ QT_SQL     ] = false;
   out_qt[ QT_SVG     ] = false;
   out_qt[ QT_XML     ] = false;
   
   // WARTOSCI AKTUALNEGO PROJECTU.
   if( d_data.contains( varname ) ) {
      const MarkValues& mark_values = d_data[ varname ];
      if( mark_values.contains( MARK_PLUS ) ) {
         const QSet< QString >& values = mark_values[ MARK_PLUS ];
         QSet< QString >::const_iterator it = values.begin();
         const QSet< QString >::const_iterator end = values.end();
         while( it != end ) {
                 if( *it == QT_CORE    ) out_qt[ QT_CORE    ] = true;
            else if( *it == QT_GUI     ) out_qt[ QT_GUI     ] = true;
            else if( *it == QT_NETWORK ) out_qt[ QT_NETWORK ] = true;
            else if( *it == QT_OPENGL  ) out_qt[ QT_OPENGL  ] = true;
            else if( *it == QT_SQL     ) out_qt[ QT_SQL     ] = true;
            else if( *it == QT_SVG     ) out_qt[ QT_SVG     ] = true;
            else if( *it == QT_XML     ) out_qt[ QT_XML     ] = true;
            ++it;
         }
      }
      if( mark_values.contains( MARK_MINUS ) ) {
         const QSet< QString >& values = mark_values[ MARK_MINUS ];
         QSet< QString >::const_iterator it = values.begin();
         const QSet< QString >::const_iterator end = values.end();
         while( it != end ) {
                 if( *it == QT_CORE    ) out_qt[ QT_CORE    ] = false;
            else if( *it == QT_GUI     ) out_qt[ QT_GUI     ] = false;
            else if( *it == QT_NETWORK ) out_qt[ QT_NETWORK ] = false;
            else if( *it == QT_OPENGL  ) out_qt[ QT_OPENGL  ] = false;
            else if( *it == QT_SQL     ) out_qt[ QT_SQL     ] = false;
            else if( *it == QT_SVG     ) out_qt[ QT_SVG     ] = false;
            else if( *it == QT_XML     ) out_qt[ QT_XML     ] = false;
            ++it;
         }
      }
   }   
}
// end of get_qt

//*******************************************************************
// get_cfg                                                   PRIVATE
//*******************************************************************
void Project::get_cfg( QHash< QString, bool >& out_cfg )
{
   out_cfg.clear();
   const QString varname = "CONFIG";

   // WARTOSCI DOMYSLNE
   out_cfg[ CFG_QT            ] = false;
   out_cfg[ CFG_X11           ] = false;
   out_cfg[ CFG_THREAD        ] = false;
   out_cfg[ CFG_RELEASE       ] = false;
   out_cfg[ CFG_DEBUG         ] = false;
   out_cfg[ CFG_RELEASE_DEBUG ] = false;
   out_cfg[ CFG_BUILD_ALL     ] = false;
   out_cfg[ CFG_ORDERED       ] = false;
   out_cfg[ CFG_WARN_OFF      ] = false;
   out_cfg[ CFG_WARN_ON       ] = false;

   // WARTOSCI AKTUALNEGO PROJECTU.
   if( d_data.contains( varname ) ) {
      const MarkValues& mark_values = d_data[ varname ];
      if( mark_values.contains( MARK_PLUS ) ) {
         const QSet< QString >& values = mark_values[ MARK_PLUS ];
         QSet< QString >::const_iterator it = values.begin();
         const QSet< QString >::const_iterator end = values.end();
         while( it != end ) {
                 if( *it == CFG_QT            ) out_cfg[ CFG_QT            ] = true;
            else if( *it == CFG_X11           ) out_cfg[ CFG_X11           ] = true;
            else if( *it == CFG_THREAD        ) out_cfg[ CFG_THREAD        ] = true;
            else if( *it == CFG_RELEASE       ) out_cfg[ CFG_RELEASE       ] = true;
            else if( *it == CFG_DEBUG         ) out_cfg[ CFG_DEBUG         ] = true;
            else if( *it == CFG_RELEASE_DEBUG ) out_cfg[ CFG_RELEASE_DEBUG ] = true;
            else if( *it == CFG_BUILD_ALL     ) out_cfg[ CFG_BUILD_ALL     ] = true;
            else if( *it == CFG_ORDERED       ) out_cfg[ CFG_ORDERED       ] = true;
            else if( *it == CFG_WARN_OFF      ) out_cfg[ CFG_WARN_OFF      ] = true;
            else if( *it == CFG_WARN_ON       ) out_cfg[ CFG_WARN_ON       ] = true;
            ++it;
         }
      }
      else if( mark_values.contains( MARK_MINUS ) ) {
         const QSet< QString >& values = mark_values[ MARK_MINUS ];
         QSet< QString >::const_iterator it = values.begin();
         const QSet< QString >::const_iterator end = values.end();
         while( it != end ) {
                 if( *it == CFG_QT            ) out_cfg[ CFG_QT            ] = false;
            else if( *it == CFG_X11           ) out_cfg[ CFG_X11           ] = false;
            else if( *it == CFG_THREAD        ) out_cfg[ CFG_THREAD        ] = false;
            else if( *it == CFG_RELEASE       ) out_cfg[ CFG_RELEASE       ] = false;
            else if( *it == CFG_DEBUG         ) out_cfg[ CFG_DEBUG         ] = false;
            else if( *it == CFG_RELEASE_DEBUG ) out_cfg[ CFG_RELEASE_DEBUG ] = false;
            else if( *it == CFG_BUILD_ALL     ) out_cfg[ CFG_BUILD_ALL     ] = false;
            else if( *it == CFG_ORDERED       ) out_cfg[ CFG_ORDERED       ] = false;
            else if( *it == CFG_WARN_OFF      ) out_cfg[ CFG_WARN_OFF      ] = false;
            else if( *it == CFG_WARN_ON       ) out_cfg[ CFG_WARN_ON       ] = false;
            ++it;
         }
      }
   }
}
// end of get_cfg

//*******************************************************************
// create                                                     PUBLIC
//*******************************************************************
bool Project::create( QString& out_path )
{
   bool retval = false;
   out_path.clear();
   
   QPointer< ProjectProp > dlg = new ProjectProp();
   if( dlg ) {
      if( QDialog::Accepted == dlg->exec() ) {
         d_file_info.setFile( QString() );
         d_data.clear();
         d_vars.clear();
         d_is_opened = true;

         QString fpath;
         QString tmpl;
         QHash< QString, bool > qt;
         QHash< QString, bool > cfg;
         dlg->get_general_data( fpath, tmpl, qt, cfg );

         QStringList src_files;
         QStringList hdr_files;
         QStringList ts_files;
         QStringList ui_files;
         QStringList qrc_files;
         dlg->get_files( src_files, hdr_files, ts_files, ui_files, qrc_files );
         set_var_files( Shared::SRC, src_files );
         set_var_files( Shared::HDR, hdr_files );
         set_var_files( Shared::TS , ts_files  );
         set_var_files( Shared::UI, ui_files   );
         set_var_files( Shared::RSC, qrc_files );

         set_properties( fpath, tmpl, qt, cfg );
         retval = save();
         out_path = path();

         reset();
      }
   }
   return retval;
}
// end of create

//*******************************************************************
// set_properties                                            PRIVATE
//-------------------------------------------------------------------
// Zapamietanie ustalen konfiguracyjnych dotyczacych projektu.
//*******************************************************************
void Project::set_properties(   const QString& in_fpath,
                         const QString& in_tmpl,
                         const QHash< QString, bool > &in_qt,
                         const QHash< QString, bool > &in_cfg )
{
   QString varname;
         
   d_file_info.setFile( in_fpath );
         
   varname = "TEMPLATE";
   del_varname( varname );
   add_varname( varname );
   d_data[ varname ][ MARK_EQU ] << in_tmpl;
         
   varname = "QT";
   del_varname( varname );
   add_varname( varname );
   if( !in_qt[ QT_CORE   ] ) d_data[ varname ][ MARK_MINUS ] << QT_CORE;
   if( !in_qt[ QT_GUI    ] ) d_data[ varname ][ MARK_MINUS ] << QT_GUI;
   if( in_qt[ QT_NETWORK ] ) d_data[ varname ][ MARK_PLUS  ] << QT_NETWORK;
   if( in_qt[ QT_OPENGL  ] ) d_data[ varname ][ MARK_PLUS  ] << QT_OPENGL;
   if( in_qt[ QT_SQL     ] ) d_data[ varname ][ MARK_PLUS  ] << QT_SQL;
   if( in_qt[ QT_SVG     ] ) d_data[ varname ][ MARK_PLUS  ] << QT_SVG;
   if( in_qt[ QT_XML     ] ) d_data[ varname ][ MARK_PLUS  ] << QT_XML;
         
   varname = "CONFIG";
   del_varname( varname );
   add_varname( varname );
   if( in_cfg[ CFG_QT            ] ) d_data[ varname ][ MARK_PLUS ] << CFG_QT;
   if( in_cfg[ CFG_X11           ] ) d_data[ varname ][ MARK_PLUS ] << CFG_X11;
   if( in_cfg[ CFG_THREAD        ] ) d_data[ varname ][ MARK_PLUS ] << CFG_THREAD;
   if( in_cfg[ CFG_RELEASE       ] ) d_data[ varname ][ MARK_PLUS ] << CFG_RELEASE;
   if( in_cfg[ CFG_DEBUG         ] ) d_data[ varname ][ MARK_PLUS ] << CFG_DEBUG;
   if( in_cfg[ CFG_RELEASE_DEBUG ] ) d_data[ varname ][ MARK_PLUS ] << CFG_RELEASE_DEBUG;
   if( in_cfg[ CFG_BUILD_ALL     ] ) d_data[ varname ][ MARK_PLUS ] << CFG_BUILD_ALL;
   if( in_cfg[ CFG_ORDERED       ] ) d_data[ varname ][ MARK_PLUS ] << CFG_ORDERED;
   if( in_cfg[ CFG_WARN_ON       ] ) d_data[ varname ][ MARK_PLUS ] << CFG_WARN_ON;
   if( in_cfg[ CFG_WARN_OFF      ] ) d_data[ varname ][ MARK_PLUS ] << CFG_WARN_OFF;
}
// end of set_properties

//*******************************************************************
// set_qt_make                                               PRIVATE
//*******************************************************************
void Project::set_qt_make()
{
   QSet< QString >::const_iterator it = QtMake::instance()->begin();
   const QSet< QString >::const_iterator end = QtMake::instance()->end();
   while( it != end ) {
      del_varname( *it );
      if( d_qt_make.contains( *it ) ) {
         //qDebug() << *it;
         add_varname( *it );
      }
      ++it;
   }
}
// end of set_qt_make

//*******************************************************************
// set_var_files                                             PRIVATE
//-------------------------------------------------------------------
// Przypisanie do projektu podanych plikow.
//*******************************************************************
void Project::set_var_files( const QString& in_varname, const QStringList& in_files )
{
   del_varname( in_varname );

   QStringList::const_iterator it = in_files.begin();
   const QStringList::const_iterator end = in_files.end();
   const QString fpath = dir();
   while( it != end ) {
      d_data[ in_varname ][ MARK_EQU ] << *it;
      ++it;   
   }

   add_varname( in_varname );
}
// end of set_var_files

//*******************************************************************
// add_varname                                               PRIVATE
//-------------------------------------------------------------------
// UWAGA: czesciowo ingeruje w kolenosc zmiennych.
//*******************************************************************
void Project::add_varname( const QString& in_varname )
{
   if( !d_vars.contains( in_varname ) ) {
      qint32 idx = -1;
           if( "TEMPLATE"               == in_varname ) idx = d_idx = 0;
      else if( "QT"                     == in_varname ) idx = d_idx = 1;
      else if( "CONFIG"                 == in_varname ) idx = d_idx = 2;

      if( d_qt_make.contains( in_varname ) ) {
         idx = ++d_idx;
      }
      
      if(( idx > -1 ) && ( idx < d_vars.size() )) {
         d_vars.insert( idx, 1, in_varname );
      }
      else {
         d_vars << in_varname;
      }
   }
}
// end of add_varname

//*******************************************************************
// del_varname                                               PRIVATE
//*******************************************************************
void Project::del_varname( const QString& in_varname )
{
   const qint32 idx = d_vars.indexOf( in_varname );
   if( idx != -1 ) {
      d_vars.remove( idx );
   }

   d_data.remove( in_varname );
}
// end of del_varname

//*******************************************************************
// open                                                       PUBLIC
//*******************************************************************
bool Project::open( const QString& in_path )
{
   bool retval = false;

   if( !d_is_opened )   {
      const QFileInfo file_info( in_path );
      if( file_info.exists() ) {
         retval = ( d_is_opened = parse_project( file_info.absoluteFilePath() ) );
         if( d_is_opened ) {
            d_file_info = file_info;
         }
      }
      else {
         QMessageBox::critical( 0, tr( CAPTION ), tr( PROJECT_FILE_NOT_FOUND ).arg( in_path ) );
      }

      if( empty( Shared::SRC ) ) {
         close();
         retval = false;
      }
   }
   return retval;
}
// end of open

//*******************************************************************
// close                                                      PUBLIC
//*******************************************************************
void Project::close()
{
   if( d_is_opened ) {
      save();
      reset();
   }
}
// end of close

//*******************************************************************
// reset                                                     PRIVATE
//*******************************************************************
void Project::reset()
{
   d_file_info.setFile( QString() );
   d_data.clear();
   d_vars.clear();
   d_is_opened = false;
}
// end of reset

//*******************************************************************
// file_name                                                  PUBLIC
//-------------------------------------------------------------------
// Zwraca nazwe pliku projektu razem z rozszerzeniem.
//*******************************************************************
const QString& Project::file_name()
{
   d_buffer = ( d_is_opened ) ? d_file_info.fileName() : "";
   return d_buffer;
}
// end of file_name

//*******************************************************************
// name                                                       PUBLIC
//-------------------------------------------------------------------
// Zwraca nazwe programu jaka bedzie mial program wygenerowany z
// projektu.
//*******************************************************************
const QString& Project::name()
{
   d_buffer.clear();
   if( d_is_opened ) {
      d_buffer = d_file_info.fileName();
      if( d_buffer.endsWith( ".pro" ) ) {
         d_buffer.resize( d_buffer.length() - 4 );
      }
   }
   return d_buffer;
}
// end of name

//*******************************************************************
// dir                                                        PUBLIC
//*******************************************************************
const QString& Project::dir()
{
   d_buffer.clear();
   if( d_is_opened ) {
      d_buffer = d_file_info.absolutePath();
      Shared::complete_dir( d_buffer );   // <<< !!!
   }
   return d_buffer;
}
// end of dir

//*******************************************************************
// path                                                       PUBLIC
//*******************************************************************
const QString& Project::path()
{
   d_buffer.clear();
   if( d_is_opened ) {
      d_buffer = d_file_info.absoluteFilePath();
   }
   return d_buffer;
}
// end of path

//*******************************************************************
// src                                                        PUBLIC
//-------------------------------------------------------------------
// Zwaraca liste plikow zrodlowych.
//*******************************************************************
bool Project::src( QStringList& out_src_list )
{
   get_var_files( Shared::SRC, out_src_list );
   return ( false == out_src_list.empty() );
}
// end of src

//*******************************************************************
// hdr                                                        PUBLIC
//-------------------------------------------------------------------
// Zwraca liste plikow naglowkowych.
//*******************************************************************
bool Project::hdr( QStringList& out_hdr_list )
{
   get_var_files( Shared::HDR, out_hdr_list );
   return ( false == out_hdr_list.empty() );
}
// end of hdr

//*******************************************************************
// ui                                                         PUBLIC
//-------------------------------------------------------------------
// Zwraca liste plikow formatek.
//*******************************************************************
bool Project::ui( QStringList& out_ui_list )
{
   get_var_files( Shared::UI, out_ui_list );
   return ( false == out_ui_list.isEmpty() );
}
// end of ui

//*******************************************************************
// qrc                                                        PUBLIC
//-------------------------------------------------------------------
// Zwaraca liste plikow zawierajacych zasoby.
//*******************************************************************
bool Project::qrc( QStringList& out_qrc_list )
{
   get_var_files( Shared::RSC, out_qrc_list );
   return ( false == out_qrc_list.empty() );
}
// end of qrc

//*******************************************************************
// get_var_files                                             PRIVATE
//-------------------------------------------------------------------
// Zwraca pliki przyporzadkowane wskazanej zmiennej.
//*******************************************************************
void Project::get_var_files( const QString& in_var_name, QStringList& out_list )
{
   out_list.clear();

   if( d_data.contains( in_var_name ) ) {
      MarkValues& mark_values = d_data[ in_var_name ];
      if( mark_values.contains( MARK_EQU ) ) {
         out_list << mark_values[ MARK_EQU ].toList();
      }
      if( mark_values.contains( MARK_PLUS ) ) {
         out_list << mark_values[ MARK_PLUS ].toList();
      }
   }
}
// end of get_files

//*******************************************************************
// dest_dir                                                   PUBLIC
//*******************************************************************
const QString& Project::dest_dir()
{
   d_buffer.clear();

   if( d_data.contains( "DESTDIR" ) ) {
      MarkValues& mark_values = d_data[ "DESTDIR" ];

      if( mark_values.contains( MARK_EQU ) ) {
         const Values& values = mark_values[ MARK_EQU ];
         if( !values.empty() ) {
            d_buffer = *values.begin();
         }
      }
      else if( mark_values.contains( MARK_PLUS ) ) {
         const Values& values = mark_values[ MARK_PLUS ];
         if( !values.empty() ) {
            d_buffer = *values.begin();
         }
      }
   }

   return d_buffer;
}
// end of dest_dir

//*******************************************************************
// prg_fpath                                                  PUBLIC
//*******************************************************************
const QString& Project::prg_fpath()
{
   // katalog projektu
   QString fpath = dir();
   // katalog pliku wykonywalnego okreslonego przez DEST_DIR         
   const QString dst = dest_dir();
   
   if( !dst.isEmpty() ) {
      // Nalezy rozroznic dwa przypadki.
      // 1. DESTDIR zawiera abolutna sciezke gdzie jest program wynikowy,
      // 2. DESTDIR zawiera sciezke wzgledna - odnosi sie ona do sciezki projektu
      if( dst.startsWith( "/" ) ) {
         fpath = dst;
      }
      else {
         const QDir dir( fpath + dst );
         fpath = dir.absolutePath();
      }
      Shared::complete_dir( fpath );
   }

   d_buffer = ( fpath += name() );
   return d_buffer;
}
// end of prg_fpath

//*******************************************************************
// add_file                                                   PUBLIC
//*******************************************************************
const QString& Project::add_file( const QString& in_fpath )
{
   // Na podstawie nazwy pliku znajdujemy w jakiej zmiennej
   // powinnien sie znalezc.
   const QString varname = var_name( in_fpath );

   if( !varname.isEmpty() ) {
      if( d_data.contains( varname ) ) {
         bool found = false;
      
         MarkValues& mark_values  = d_data[ varname ];
         const bool eq_contains   = mark_values.contains( MARK_EQU );
         const bool plus_contains = mark_values.contains( MARK_PLUS );

         // Nie wolno duplikowac plikow.
         // Dlatego najpierw sprawdzamy czy plik juz gdziec wpisano.
         if( eq_contains) {
            found = mark_values[ MARK_EQU ].contains( in_fpath );
         }
         if( !found && plus_contains ) {
            found = mark_values[ MARK_PLUS ].contains( in_fpath );
         }

         // Mozna dodac. Pliku jeszcze nigdzie nie ma.
         if( !found ) {
                 if( eq_contains   ) mark_values[ MARK_EQU  ] << in_fpath;
            else if( plus_contains ) mark_values[ MARK_PLUS ] << in_fpath;
            else                     mark_values[ MARK_EQU  ] << in_fpath;
         }
      }
      // Dopisanie nazwy pliku wraz z utworzeniem zmiennej.
      else {
         d_data[ varname ][ MARK_EQU ] << in_fpath;
         add_varname( varname );
      }
      d_buffer = varname;
   }
   return d_buffer;
}
// end of add_file

//*******************************************************************
// rename_file                                                PUBLIC
//*******************************************************************
const QString& Project::rename_file( const QString& in_old_fpath, const QString& in_new_fpath )
{
   bool found = false;
   const QString varname = var_name( in_old_fpath );
   
   if( d_data.contains( varname ) ) {
      MarkValues& mark_values = d_data[ varname ];

      if( mark_values.contains( MARK_EQU ) ) {
         Values& values = mark_values[ MARK_EQU ];
         if( values.remove( in_old_fpath ) ) {
            values << in_new_fpath;
            found = true;
         }
      }
      if( !found && mark_values.contains( MARK_PLUS ) ) {
         Values& values = mark_values[ MARK_PLUS ];
         if( values.remove( in_old_fpath ) ) {
            values << in_new_fpath;
            found = true;
         }
      }
   }

   d_buffer = ( found ) ? varname : "";
   return d_buffer;
}
// end of rename_file

//*******************************************************************
// remove_file                                                PUBLIC
//*******************************************************************
const QString& Project::remove_file( const QString& in_fpath )
{
   bool found = false;
   QString varname = var_name( in_fpath );
   
   if( d_data.contains( varname ) ) {
      MarkValues& mark_values = d_data[ varname ];
      
      if( mark_values.contains( MARK_EQU ) ) {
         found = mark_values[ MARK_EQU ].remove( in_fpath );
      }
      if( !found && mark_values.contains( MARK_PLUS ) ) {
         found = mark_values[ MARK_PLUS ].remove( in_fpath );
      }
   }

   d_buffer = ( found ) ? varname : "";
   return d_buffer;
}
// end of remove_file


//###################################################################
//#                                                                 #
//#                        P R I V A T E                            #
//#                                                                 #
//###################################################################


//*******************************************************************
// parse_project                                             PRIVATE
//*******************************************************************
bool Project::parse_project( const QString& in_fpath )
{
   QFileInfo file_info( in_fpath );
   QString project_dir = file_info.absolutePath();
   Shared::complete_dir( project_dir );
   QStringList lines;

   // Najpierw odczytujemy plik projektu z dysku.   
   const bool retval = read_project_file( in_fpath, lines );

   if( retval ) {
      // Teraz przystepujemy do analizy linii,
      QString varname,value;
      qint32 mark;
      QStringList value_list;

      QStringList::const_iterator lines_it = lines.begin();
      const QStringList::const_iterator lines_end = lines.end();
      while( lines_it != lines_end ) {
         // Dzielimy linie na trzy wartosci:
         // nazwa zmiennej, znak, wartosci zmiennej.
         clip_line( *lines_it, varname, mark, value );

         if( QtMake::instance()->contains( varname ) ) {
            d_qt_make[ varname ] = value;
         }
         else {
            // Wyznaczamy wszystkie wartosci.
            value_list.clear();
            if( !value.isEmpty() ) {
               value_list = value.split( " ", QString::SkipEmptyParts );
               if( is_source_file( varname ) ) {
                  // Dla powyzej wymienionych zmiennych uzupelniamy
                  // nazwy pliko o absolutna sciezke.
                  QStringList::iterator value_list_it = value_list.begin();
                  const QStringList::const_iterator value_list_end = value_list.end();
                  while( value_list_it != value_list_end ) {
                     if( value_list_it->indexOf( '/' ) != 0 ) {
                        file_info.setFile( project_dir + *value_list_it );
                        *value_list_it = file_info.absoluteFilePath();
                     }
                     ++value_list_it;
                  }
               }
            }

            Values& values = d_data[ varname ][ mark ];
            QStringList::const_iterator value_list_it = value_list.begin();
            const QStringList::const_iterator value_list_end = value_list.end();
            while( value_list_it != value_list_end ) {
               values << *value_list_it;
               ++value_list_it;
            }
         }
         // Zapamietujemy kolejnosc zmiennych.
         // (Potrzebna nam pozniej do zapisu projektu na dysk.)
         add_varname( varname );
         ++lines_it;
      }
   }
   return retval;
}
// end of parse_project

//*******************************************************************
// read_project_line                                         PRIVATE
//*******************************************************************
bool Project::read_project_file( const QString& in_fpath, QStringList& out_list )
{
   out_list.clear();

   QString line;
   QString tmp;
   QFile file( in_fpath );
   
   const bool retval = file.open( QIODevice::ReadOnly );

   if( retval ) {
      QTextStream in( &file );
      while( !in.atEnd() ) {
         if( normalize_line( tmp = in.readLine() ) ) {
            line = tmp;
            while( line.endsWith( '\\' ) && !in.atEnd() ) {
               line.resize( line.length() - 1 );
               if( normalize_line( tmp = in.readLine() ) ) {
                  line += " ";
                  line += tmp;
               }
            }
            out_list << line;
         }
      }
      file.close();
   }
   return retval;
}
// end of read_project_file

//*******************************************************************
// clip_line                                                 PRIVATE
//-------------------------------------------------------------------
// Dzieli linie opisujaca zmienna projektu na czesci.
//*******************************************************************
void Project::clip_line( const QString& in_line, QString& out_varname, qint32& out_mark, QString& out_values ) const
{
   out_varname.clear();
   out_mark = MARK_UNKNOWN;
   out_values.clear();
   
   const qint32 idx = in_line.indexOf( '=' );
   if( idx > 0 ) {
      const QChar& prefix = in_line[ idx - 1 ];
      if( '+' == prefix ) {
         out_mark = MARK_PLUS;
         out_varname = in_line.left( idx - 1 ).trimmed();
      }
      else if( '-' == prefix ) {
         out_mark = MARK_MINUS;
         out_varname = in_line.left( idx - 1 ).trimmed();
      }
      else {
         out_mark = MARK_EQU;
         out_varname = in_line.left( idx ).trimmed();
      }
      out_values  = in_line.mid( idx + 1 ).trimmed();
   }
}
// end of clip_line

//*******************************************************************
// normalize_line                                            PRIVATE
//-------------------------------------------------------------------
// Wycina z linii komentarze (fragmenty rozpoczynajace sie znakiem #).
// Obcina spacje z poczatku i konca linii.
// Zastepuje sekwencje bialych znakow pojedyczna spacja.
//*******************************************************************
bool Project::normalize_line( QString& inout_line )
{
   if( !inout_line.isEmpty() ) {
      const qint32 idx = inout_line.indexOf( '#' );
      if( idx != -1 ) {
         inout_line = inout_line.left( idx );
      }
      if( !inout_line.isEmpty() ) inout_line = inout_line.trimmed();
      if( !inout_line.isEmpty() ) inout_line = inout_line.simplified();
   }
   return ( false == inout_line.isEmpty() );
}
// end of normalize_line

//*******************************************************************
// save                                                      PRIVATE
//*******************************************************************
bool Project::save()
{
   const QString project_path = d_file_info.absoluteFilePath();
   const QString project_bak  = project_path + ".bak";
   
   QFile file( project_path );
   QFile::remove( project_bak );
   QFile::rename( project_path, project_bak );
   
   const bool retval = file.open( QIODevice::WriteOnly | QIODevice::Truncate );
   if( retval ) {
      QTextStream out( &file );
      Vars::const_iterator it = d_vars.begin();
      const Vars::const_iterator end = d_vars.end();
      while( it != end ) {
         if( d_qt_make.contains( *it ) ) {
            out << *it << " = " << d_qt_make[ *it ] << "\n";
         }
         else if( d_data.contains( *it ) ) {
            save_var( out, *it );
         }
         ++it;
      }
      file.close();
   }
   else {
      QMessageBox::critical( 0, tr( CAPTION ), tr( CANT_SAVE_FILE ).arg( file.fileName() ).arg( file.errorString() ) );
   }
   return retval;
}
// end of save

//*******************************************************************
// save_var                                                  PRIVATE
//*******************************************************************
void Project::save_var( QTextStream& out_stream, const QString& in_varname )
{
   QString item_separator;
   QString value;
   QString mark_as_string;
   QStringList lista;
   MarkValues& values = d_data[ in_varname ];

   QVector<qint32> marks;
   marks << MARK_EQU << MARK_PLUS << MARK_MINUS;
   
   QVector<qint32>::const_iterator mit = marks.begin();
   const QVector<qint32>::const_iterator mit_end = marks.end();
   while( mit != mit_end ) {
      if( values.contains( *mit ) ) {
         
              if( MARK_PLUS  == *mit ) mark_as_string = "+=";
         else if( MARK_MINUS == *mit ) mark_as_string = "-=";
         else                          mark_as_string = "=";
         
         out_stream << in_varname << " " << mark_as_string << " ";
         
         lista = values[ *mit ].toList();
         lista.sort();
         item_separator = ( lista.size() > 5 ) ? " \\\n\t" : " ";
         QStringList::const_iterator it = lista.begin();
         const QStringList::const_iterator end = lista.end();
         while( it != end ) {
            value = *it;
            if( is_source_file( in_varname ) ) {
               value = Shared::relative_path( dir(), value );
            }
            out_stream << item_separator << value;
            ++it;
         }
         out_stream << "\n";
      }
      ++mit;
   }
}
// end of save_var

//*******************************************************************
// is_source_file                                            PRIVATE
//*******************************************************************
bool Project::is_source_file( const QString& in_varname ) const
{
   return ( ( Shared::SRC == in_varname ) ||
            ( Shared::HDR == in_varname ) ||
            ( Shared::UI  == in_varname ) ||
            ( Shared::TS  == in_varname ) ||
            ( Shared::RSC == in_varname ) );
}
// end of is_source_file

//*******************************************************************
// empty                                                     PRIVATE
//-------------------------------------------------------------------
// Sprawdza czy hash zawiera pozycje ze wskazanym kluczem.
// Jesli taki klucz istnieje, sprawdza czy zawiera on jakies dane.
// Jesli nie zawiera to jest empty.
//*******************************************************************
bool Project::empty( const QString& in_varname )
{
   return d_data.contains( in_varname ) ? d_data[ in_varname ].empty() : true;
}
// end of empty

//*******************************************************************
// var_name                                                 PRIVATE
//-------------------------------------------------------------------
// Na podstawie nazwy pliku okresla rodzaj zmiennej (SRC,HDR,UI,RSC).
//*******************************************************************
const QString& Project::var_name( const QString& in_fname )
{
   d_buffer.clear();

   const QString fext = Shared::extension( in_fname );

        if( Shared::is_src_ext( fext ) ) d_buffer = Shared::SRC;
   else if( Shared::is_hdr_ext( fext ) ) d_buffer = Shared::HDR;
   else if( Shared::is_ui_ext ( fext ) ) d_buffer = Shared::UI;
   else if( Shared::is_qrc_ext( fext ) ) d_buffer = Shared::RSC;
   
   return d_buffer;
}
// end of var_name
