/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : ProjectCppFilesWidget.cpp
 * DATE   : 14.12.2006
 *******************************************************************/
 
/*------- include files:
-------------------------------------------------------------------*/
#include "ProjectCppFilesWidget.h"
#include "Shared.h"
#include <QMessageBox>

/*------- local constants:
-------------------------------------------------------------------*/
const char* const ProjectCppFilesWidget::CAPTION    = QT_TR_NOOP( "Add source files" );


//*******************************************************************
// ProjectCppFilesWidget                                 CONSTRUCTOR
//*******************************************************************
ProjectCppFilesWidget::ProjectCppFilesWidget( ProjectProp* const in_project_prop, QWidget* const in_parent )
: ProjectFilesWidget( in_project_prop, in_parent )
{
   d_filters << "All C/C++ files (*.cpp *.c *.cxx *.cc)";
}
// end of ProjectCppFilesWidget

//*******************************************************************
// fname_is_valid                                  PRIVATE inherited
//*******************************************************************
bool ProjectCppFilesWidget::fname_is_valid( QStringList& inout_flist )
{
   bool retval = true;
   QString fext;

   QStringList::iterator it = inout_flist.begin();
   const QStringList::iterator end = inout_flist.end();
   while( it != end ) {
      fext = Shared::extension( *it );
      if( fext.isEmpty() ) {
         *it += ".cpp";
      }
      else {
         if( !Shared::is_src_ext( fext ) ) {
            retval = false;
            break;
         }
      }
      ++it;
   }

   if( !retval ) {
      const QString msg = ( inout_flist.size() > 1 ) ? tr( MSG_MULTI ) : tr( MSG_SINGLE ).arg( inout_flist[ 0 ] );
      QMessageBox::warning( 0, tr( CAPTION ), msg );
   }
   return retval;
}
// end of fname_is_valid

//*******************************************************************
// accept                                                     PUBLIC
//*******************************************************************
bool ProjectCppFilesWidget::accept() const
{
   return !empty();
}
// end of accept
