/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : ProjectProp.cpp
 * DATE   : 12.12.2006
 *-------------------------------------------------------------------
 * Klasa opisujaca dialog, w ktorym uzytkownik okrelsa elementy
 * wchodzace w sklad nowego obiektu. Czyli parametry projektu i
 * pliki, ktore go tworza.
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "ProjectProp.h"
#include "ProjectTools.h"
#include "ProjectPropGeneral.h"
#include "ProjectCppFilesWidget.h"
#include "ProjectHppFilesWidget.h"
#include "ProjectTsFilesWidget.h"
#include "ProjectUiFilesWidget.h"
#include "ProjectQrcFilesWidget.h"
#include "ProjectQtMake.h"
#include <QGridLayout>
#include <QTabWidget>
#include <QPushButton>
#include <QMessageBox>
#include <QtDebug>

/*------- constants:
-------------------------------------------------------------------*/
const char* const ProjectProp::CAPTION      = QT_TR_NOOP( "Project properties" );
const char* const ProjectProp::TOOLS        = QT_TR_NOOP( "&Tools" );
const char* const ProjectProp::GENERAL      = QT_TR_NOOP( "&General" );
const char* const ProjectProp::SOURCES      = QT_TR_NOOP( "&Sources" );
const char* const ProjectProp::HEADERS      = QT_TR_NOOP( "&Headers" );
const char* const ProjectProp::TRANSLATIONS = QT_TR_NOOP( "&Translations" );
const char* const ProjectProp::FORMS        = QT_TR_NOOP( "&Forms" );
const char* const ProjectProp::RESOURCES    = QT_TR_NOOP( "&Resources" );
const char* const ProjectProp::FLAGS        = QT_TR_NOOP( "&QtMake" );
const char* const ProjectProp::APPLY        = QT_TR_NOOP( "&Apply" );
const char* const ProjectProp::CANCEL       = QT_TR_NOOP( "&Cancel" );
const char* const ProjectProp::INFO         = QT_TR_NOOP( "When you would like to create new project,\nyou must add to project minimum one source file" );

//*******************************************************************
// ProjectProp                                            CONSTUCTOR
//*******************************************************************
ProjectProp::ProjectProp( QWidget* const in_parent )
: QDialog       ( in_parent                 )
, d_tab_widget  ( new QTabWidget            )
, tools_        ( new ProjectTools          )
, d_prop_general( new ProjectPropGeneral    )
, d_cpp_files   ( new ProjectCppFilesWidget ( this ) )
, d_hpp_files   ( new ProjectHppFilesWidget ( this ) )
, d_ts_files    ( new ProjectTsFilesWidget  ( this ) )
, d_ui_files    ( new ProjectUiFilesWidget  ( this ) )
, d_qrc_files   ( new ProjectQrcFilesWidget ( this ) )
, d_qtmake      ( new ProjectQtMake         ( this ) )
{
   setWindowTitle( tr( CAPTION ) );
   
   QGridLayout* const main_glay = new QGridLayout( this );

   main_glay->addWidget( d_tab_widget, 0, 0 );
   d_tab_widget->insertTab( 0, d_prop_general, tr( GENERAL      ) );
   d_tab_widget->insertTab( 1, d_cpp_files   , tr( SOURCES      ) );
   d_tab_widget->insertTab( 2, d_hpp_files   , tr( HEADERS      ) );
   d_tab_widget->insertTab( 3, d_ts_files    , tr( TRANSLATIONS ) );
   d_tab_widget->insertTab( 4, d_ui_files    , tr( FORMS        ) );
   d_tab_widget->insertTab( 5, d_qrc_files   , tr( RESOURCES    ) );
   d_tab_widget->insertTab( 6, d_qtmake      , tr( FLAGS        ) );
   d_tab_widget->insertTab( 7, tools_        , tr( TOOLS        ) );
   
   // 2. PRZYCISKI
   QVBoxLayout* const btn_vlay = new QVBoxLayout;
   main_glay->addLayout( btn_vlay, 0, 1 );
   QPushButton* const btn_ok = new QPushButton( tr( APPLY ), this );
   QPushButton* const btn_cancel = new QPushButton( tr( CANCEL ), this );
   btn_vlay->addStretch( 10 );
   btn_vlay->addWidget( btn_ok );
   btn_vlay->addWidget( btn_cancel );
   
   connect( btn_ok        , SIGNAL( clicked() )        , this, SLOT( accept_slot()          ) );
   connect( btn_cancel    , SIGNAL( clicked() )        , this, SLOT( reject()               ) );
   connect( d_prop_general, SIGNAL( enable_tabs(bool) ), this, SLOT( enable_tabs_slot(bool) ) );

   enable_tabs_slot( false );
}
// end of ProjectProp

//*******************************************************************
// accept_slot                                          PRIVATE slot
//*******************************************************************
void ProjectProp::accept_slot()
{
   Q_CHECK_PTR( d_prop_general );

   if( tools_->accept() && d_prop_general->accept() ) {
      if( d_cpp_files->accept() ) {
         accept();
      }
      else {
         QMessageBox::information( 0, tr( CAPTION ), tr( INFO ) );
         d_tab_widget->setCurrentIndex( 1 );
      }
   }
}
// end of accept_slot

//*******************************************************************
// enable_tabs_slot                                     PRIVATE slot
//-------------------------------------------------------------------
// Wlaczanie/Wylaczanie zakladek, w ktorych uzytkownik moze okreslic
// pliki wchodzace w sklad projektu.
//*******************************************************************
void ProjectProp::enable_tabs_slot( const bool in_state )
{
   for( qint32 i = 1; i < d_tab_widget->count(); ++i ) {
      d_tab_widget->setTabEnabled( i, in_state );
   }
}
// end of enable_tabs

//*******************************************************************
// get_general_data                                           PUBLIC
//-------------------------------------------------------------------
// Funkcja zwracajaca informacje o ustawieniach projektu.
//*******************************************************************
void ProjectProp::get_general_data( QString& out_fpath,
                                    QString& out_tmpl,
                                    QHash< QString, bool >& out_qt,
                                    QHash< QString, bool >& out_cfg )
{
   d_prop_general->get_data( out_fpath, out_tmpl, out_qt, out_cfg );
}
// end of get_general_data

//*******************************************************************
// set_general_data                                           PUBLIC
//*******************************************************************
void ProjectProp::set_general_data( const QString& in_fpath,
                                    const QString& in_tmpl,
                                    const QHash< QString, bool >& in_qt,
                                    const QHash< QString, bool >& in_cfg )
{
   d_prop_general->set_data( in_fpath, in_tmpl, in_qt, in_cfg );
}
// end of set_general_data

//*******************************************************************
// set_files                                                  PUBLIC
//*******************************************************************
void ProjectProp::set_files(  const QStringList& in_src_files,
                              const QStringList& in_hdr_files,
                              const QStringList& in_ts_files,
                              const QStringList& in_ui_files,
                              const QStringList& in_qrc_files )
{
   d_cpp_files->set_data( in_src_files );
   d_hpp_files->set_data( in_hdr_files );
   d_ts_files->set_data ( in_ts_files  );
   d_ui_files->set_data ( in_ui_files  );
   d_qrc_files->set_data( in_qrc_files );
}
// end of set_files

//*******************************************************************
// get_files                                                  PUBLIC
//*******************************************************************
void ProjectProp::get_files(  QStringList& out_src_files,
                              QStringList& out_hdr_files,
                              QStringList& out_ts_files,
                              QStringList& out_ui_files,
                              QStringList& out_qrc_files )
{
   d_cpp_files->get_data( out_src_files );
   d_hpp_files->get_data( out_hdr_files );
   d_ts_files->get_data ( out_ts_files  );
   d_ui_files->get_data ( out_ui_files  );
   d_qrc_files->get_data( out_qrc_files );
}
// end of get_files

//*******************************************************************
// set_qt_make                                                PUBLIC
//*******************************************************************
void ProjectProp::set_qt_make( const QHash< QString, QString >& in_data )
{
   d_qtmake->set_qt_make( in_data );
}
// end of set_qt_make

//*******************************************************************
// get_qt_make                                                PUBLIC
//*******************************************************************
void ProjectProp::get_qt_make( QHash< QString, QString >& out_data )
{
   d_qtmake->get_qt_make( out_data );
}
// end of get_qt_make

//*******************************************************************
// changed_files                                              PUBLIC
//*******************************************************************
bool ProjectProp::changed_files() const
{
   const bool retval = (   d_cpp_files->changed() ||
                           d_hpp_files->changed() ||
                           d_ts_files->changed()  ||
                           d_ui_files->changed()  ||
                           d_qrc_files->changed() );
   return retval;
}
// end of changed_files

//*******************************************************************
// dir                                                        PUBLIC
//-------------------------------------------------------------------
// Funkcja zwraca katalog, w ktorym znajduje sie plik projektu.
// Zakladki pozwalajace wybierac pliki do projektu korzystaja z
// tego katalogu jako katalogu startowego w dialogu wyboru plikow.
//*******************************************************************
const QString& ProjectProp::dir() const
{
   return d_prop_general->dir();
}
// end of dir
