/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : ProjectProp.cpp
 * DATE   : 12.12.2006
 *******************************************************************/
#ifndef INCLUDED_ProjectProp_h
#define INCLUDED_ProjectProp_h

/*------- include files:
-------------------------------------------------------------------*/
#include <QDialog>
#include <QPointer>

/*------- forward declarations:
-------------------------------------------------------------------*/
class QTabWidget;
class ProjectTools;
class ProjectPropGeneral;
class ProjectCppFilesWidget;
class ProjectHppFilesWidget;
class ProjectTsFilesWidget;
class ProjectUiFilesWidget;
class ProjectQrcFilesWidget;
class ProjectQtMake;

/*------- class declaration:
-------------------------------------------------------------------*/
class ProjectProp : public QDialog
{
   Q_OBJECT
public:
   ProjectProp( QWidget* in_parent = 0 );
private:
   ProjectProp( const ProjectProp& );
   ProjectProp& operator=( const ProjectProp& );

// ******* CONSTANTS *******
private:
   static const char* const CAPTION;
   static const char* const TOOLS;
   static const char* const GENERAL;
   static const char* const SOURCES;
   static const char* const HEADERS;
   static const char* const TRANSLATIONS;
   static const char* const FORMS;
   static const char* const RESOURCES;
   static const char* const FLAGS;
   static const char* const APPLY;
   static const char* const CANCEL;
   static const char* const INFO;

// ******* MEMBERS *******
public:
   void get_general_data(  QString& out_fpath, QString& out_tmpl,
                     QHash< QString, bool >& out_qt,
                     QHash< QString, bool >& out_cfg );
   void set_general_data(  const QString& in_fpath,
                     const QString& in_tmpl,
                     const QHash< QString, bool >& in_qt,
                     const QHash< QString, bool >& in_cfg ); 
   void set_files (  const QStringList&,
                     const QStringList&,
                     const QStringList&,
                     const QStringList&,
                     const QStringList& );
   void get_files (  QStringList&,
                     QStringList&,
                     QStringList&,
                     QStringList&,
                     QStringList& );
   void     set_qt_make (  const QHash< QString, QString >& );
   void     get_qt_make (  QHash< QString, QString >& );   
   bool changed_files   () const;
private:
   QTabWidget*            const d_tab_widget;
   ProjectTools*          const tools_;
   ProjectPropGeneral*    const d_prop_general;
   ProjectCppFilesWidget* const d_cpp_files;
   ProjectHppFilesWidget* const d_hpp_files;
   ProjectTsFilesWidget*  const d_ts_files;
   ProjectUiFilesWidget*  const d_ui_files;
   ProjectQrcFilesWidget* const d_qrc_files;
   ProjectQtMake*         const d_qtmake;

// ******* METHODS *******
public:
   const QString& dir() const;
private slots:
   void enable_tabs_slot( bool in_state );
   void accept_slot();
};

#endif // INCLUDED_ProjectProp_h
