/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : ProjectPropGeneral.cpp
 * DATE   : 12.12.2006
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "ProjectPropGeneral.h"
#include <QDir>
#include <QFileInfo>
#include <QLayout>
#include <QLabel>
#include <QComboBox>
#include <QStringList>
#include <QGroupBox>
#include <QGridLayout>
#include <QCheckBox>
#include <QRadioButton>
#include <QButtonGroup>
#include <QLineEdit>
#include <QPushButton>
#include <QFileDialog>
#include <QMessageBox>
#include <QFileInfo>
#include <QtDebug>

/*------- local constants:
-------------------------------------------------------------------*/
const char* const ProjectPropGeneral::TEMPLATE           = QT_TR_NOOP( "&Template: " );
const char* const ProjectPropGeneral::PATH               = QT_TR_NOOP( "&Project path: " );
const char* const ProjectPropGeneral::CAPTION            = QT_TR_NOOP( "Select a project location" );
const char* const ProjectPropGeneral::FILTER             = QT_TR_NOOP( "Project files (*.pro)" );
const char* const ProjectPropGeneral::FILE_EXIST_CAPTION = QT_TR_NOOP( "Project file" );
const char* const ProjectPropGeneral::FILE_EXIST_MSG     = QT_TR_NOOP( "Selected project file already exists.\nWould you like to create new file?\(%1)" );

//*******************************************************************
// ProjectPropGeneral                                    CONSTRUCTOR
//*******************************************************************
ProjectPropGeneral::ProjectPropGeneral( QWidget* const in_parent )
: QWidget              ( in_parent )
, d_project_path_le    ( new QLineEdit )
, d_project_select_pbtn( 0 )
, d_template_cbx( 0 )
, d_core_cbx( 0 )
, d_gui_cbx( 0 )
, d_network_cbx( 0 )
, d_opengl_cbx( 0 )
, d_sql_cbx( 0 )
, d_svg_cbx( 0 )
, d_xml_cbx( 0 )
, d_qt_cbx( 0 )
, d_x11_cbx( 0 )
, d_thread_cbx( 0 )
, d_release_rbtn( 0 )
, d_debug_rbtn( 0 )
, d_release_and_debug_rbtn( 0 )
, d_build_all_cbx( 0 )
, d_ordered_cbx( 0 )
, d_warn_on_rbtn( 0 )
, d_warn_off_rbtn( 0 )
, d_fpath( "" )
, d_tmpl( "" )
{
	d_qt.clear();
	d_cfg.clear();

	QVBoxLayout* const main_lay = new QVBoxLayout( this );
	
	{	// 1. TEMPLATE
		QGridLayout* const general_lay = new QGridLayout;
		QLabel* const project_path_lbl = new QLabel( tr( PATH ), this );
		general_lay->addWidget( project_path_lbl, 0, 0 );
		general_lay->addWidget( d_project_path_le, 0, 1 );
		general_lay->addWidget( d_project_select_pbtn = new QPushButton( "..." ), 0, 2 );
		project_path_lbl->setBuddy( d_project_path_le );
		d_project_select_pbtn->setMaximumWidth( d_project_select_pbtn->fontMetrics().width( "XXX" ) );

		QLabel* const template_lbl = new QLabel( tr( TEMPLATE ), this );
		general_lay->addWidget( template_lbl, 1, 0 );
		general_lay->addWidget( d_template_cbx = new QComboBox( this ), 1, 1 );
		template_lbl->setBuddy( d_template_cbx );
		general_lay->setColumnStretch( 1, 10 );
		QStringList template_items;
		template_items << "app" << "lib" << "subdirs" << "vcapp" << "vclib";
		d_template_cbx->addItems( template_items );
		main_lay->addLayout( general_lay );	
		
		connect( d_template_cbx, SIGNAL( currentIndexChanged( const QString& ) ), this, SLOT( template_changed( const QString& ) ) );
		connect( d_project_select_pbtn, SIGNAL( clicked() ), this, SLOT( get_path_slot() ) );
	}

	QHBoxLayout* const box_lay = new QHBoxLayout;
	{	// 2. QT
		QGroupBox* const qt_gbox = new QGroupBox( "QT", this );
		QVBoxLayout* const qt_lay = new QVBoxLayout( qt_gbox );
		qt_lay->addWidget( d_core_cbx    = new QCheckBox( Project::QT_CORE   , qt_gbox ) );
		qt_lay->addWidget( d_gui_cbx     = new QCheckBox( Project::QT_GUI    , qt_gbox ) );
		qt_lay->addWidget( d_network_cbx = new QCheckBox( Project::QT_NETWORK, qt_gbox ) );
		qt_lay->addWidget( d_opengl_cbx  = new QCheckBox( Project::QT_OPENGL , qt_gbox ) );
		qt_lay->addWidget( d_sql_cbx     = new QCheckBox( Project::QT_SQL    , qt_gbox ) );
		qt_lay->addWidget( d_svg_cbx     = new QCheckBox( Project::QT_SVG    , qt_gbox ) );
		qt_lay->addWidget( d_xml_cbx     = new QCheckBox( Project::QT_XML    , qt_gbox ) );
		box_lay->addWidget( qt_gbox );
	}
	{	// 3. CONFIG
		QGroupBox* const config_gbox = new QGroupBox( "CONFIG", this );
		box_lay->addWidget( config_gbox );
		QVBoxLayout* const config_lay                   = new QVBoxLayout ( config_gbox );
		config_lay->addWidget( d_qt_cbx                 = new QCheckBox   ( Project::CFG_QT, config_gbox ) );
		config_lay->addWidget( d_x11_cbx                = new QCheckBox   ( Project::CFG_X11, config_gbox ) );
		config_lay->addWidget( d_thread_cbx             = new QCheckBox   ( Project::CFG_THREAD, config_gbox ) );
		config_lay->addWidget( d_release_rbtn           = new QRadioButton( Project::CFG_RELEASE, config_gbox ) );
		config_lay->addWidget( d_debug_rbtn             = new QRadioButton( Project::CFG_DEBUG, config_gbox ) );
		config_lay->addWidget( d_release_and_debug_rbtn = new QRadioButton( Project::CFG_RELEASE_DEBUG, config_gbox ) );
		config_lay->addWidget( d_build_all_cbx          = new QCheckBox   ( Project::CFG_BUILD_ALL, config_gbox ) );
		config_lay->addWidget( d_ordered_cbx            = new QCheckBox   ( Project::CFG_ORDERED, config_gbox ) );
		config_lay->addWidget( d_warn_on_rbtn           = new QRadioButton( Project::CFG_WARN_ON, config_gbox ) );
		config_lay->addWidget( d_warn_off_rbtn          = new QRadioButton( Project::CFG_WARN_OFF, config_gbox ) );
		
		QButtonGroup* const release_group = new QButtonGroup( config_gbox );
		release_group->addButton( d_release_rbtn );
		release_group->addButton( d_debug_rbtn );
		release_group->addButton( d_release_and_debug_rbtn );
		QButtonGroup* const warn_group = new QButtonGroup( config_gbox );
		warn_group->addButton( d_warn_on_rbtn );
		warn_group->addButton( d_warn_off_rbtn );

		connect( release_group, SIGNAL( buttonClicked( QAbstractButton* ) ), this, SLOT( release_and_debug_slot( QAbstractButton* ) ) );
	}
	main_lay->addLayout( box_lay );

	set_defaults();
	connect( d_project_path_le, SIGNAL( textChanged( const QString& ) ), this, SLOT( text_changed_slot( const QString& ) ) );
}
// end of ProjectPropGeneral

//*******************************************************************
// set_defaults                                              PRIVATE
//*******************************************************************
void ProjectPropGeneral::set_defaults()
{
	d_core_cbx->setChecked( true );
	d_gui_cbx->setChecked( true );

	d_qt_cbx->setChecked( true );
	d_x11_cbx->setChecked( true );
	d_build_all_cbx->setEnabled( false );
	d_ordered_cbx->setEnabled( false );
	d_release_rbtn->setChecked( true );
	d_warn_off_rbtn->setChecked( true );
}
// end of set_defaults

//*******************************************************************
// template_changed                                     PRIVATE slot
//*******************************************************************
void ProjectPropGeneral::template_changed( const QString& in_text )
{
	if( "subdirs" == in_text ) {
		d_ordered_cbx->setEnabled( true );
	}
	else {
		d_ordered_cbx->setChecked( false );
		d_ordered_cbx->setEnabled( false );
	}
}
// end of template_changed

//*******************************************************************
// release_and_debug_slot                               PRIVATE slot
//*******************************************************************
void ProjectPropGeneral::release_and_debug_slot( QAbstractButton* const in_button )
{
	if( in_button == d_release_and_debug_rbtn ) {
		d_build_all_cbx->setEnabled( true );
	}
	else {
		d_build_all_cbx->setChecked( false );
		d_build_all_cbx->setEnabled( false );
	}
}
// end of release_and_debug_slot

//*******************************************************************
// get_path_slot                                        PRIVATE slot
//-------------------------------------------------------------------
// Uzytkownik wywolal dialog, w ktorym chce wskazac lokalizacje
// pliku nowego projektu.
// 1. Jesli plik nie ma rozszerzenia 'pro', to zostanie ono dodane,
// 2. Jesli taki plik juz istnieje, uzytkownik musi sie zdecydowac
//    czy chce tworzyc nowy plik.
// 3. Jesli nowy plik zostal utworzony jego sciezka zostaje
//    wpisana do pola edycyjnego 'd_project_path_le'.
//*******************************************************************
void ProjectPropGeneral::get_path_slot()
{
	QFileDialog* dlg = new QFileDialog( 0, tr( CAPTION ), QDir::homePath(), tr( FILTER ) );
	if( dlg ) {
		if( QDialog::Accepted == dlg->exec() ) {
			const QStringList flist = dlg->selectedFiles();
			if( !flist.empty() ) {
				QString fpath = flist.first();
				const QFileInfo finfo( fpath );

				if( !fpath.endsWith( ".pro" ) ) {
					fpath += ".pro";
				}
				if( create_project_file( fpath ) ) {
					d_project_path_le->setText( fpath );
				}
			}
		}
		delete dlg;
		dlg = 0;
	}
}
// end of get_path_slot

//*******************************************************************
// accept                                                     PUBLIC
//*******************************************************************
bool ProjectPropGeneral::accept()
{
	bool retval = false;
	
	const QString fname = d_project_path_le->text();
	if( !fname.isEmpty() ) {
		const QFileInfo finfo( fname );
		if( finfo.exists() ) {
			retval = true;

			d_fpath = d_project_path_le->text();
			d_tmpl  = d_template_cbx->currentText();

			d_qt[ Project::QT_CORE            ] = d_core_cbx->isChecked();
			d_qt[ Project::QT_GUI             ] = d_gui_cbx->isChecked();
			d_qt[ Project::QT_NETWORK         ] = d_network_cbx->isChecked();
			d_qt[ Project::QT_OPENGL          ] = d_opengl_cbx->isChecked();
			d_qt[ Project::QT_SQL             ] = d_sql_cbx->isChecked();
			d_qt[ Project::QT_SVG             ] = d_svg_cbx->isChecked();
			d_qt[ Project::QT_XML             ] = d_xml_cbx->isChecked();

			d_cfg[ Project::CFG_QT            ] = d_qt_cbx->isChecked();	
			d_cfg[ Project::CFG_X11           ] = d_x11_cbx->isChecked();
			d_cfg[ Project::CFG_THREAD        ] = d_thread_cbx->isChecked();
			d_cfg[ Project::CFG_RELEASE       ] = d_release_rbtn->isChecked();
			d_cfg[ Project::CFG_DEBUG         ] = d_debug_rbtn->isChecked();
			d_cfg[ Project::CFG_RELEASE_DEBUG ] = d_release_and_debug_rbtn->isChecked();
			d_cfg[ Project::CFG_BUILD_ALL     ] = d_build_all_cbx->isChecked();
			d_cfg[ Project::CFG_ORDERED       ] = d_ordered_cbx->isChecked();
			d_cfg[ Project::CFG_WARN_ON       ] = d_warn_on_rbtn->isChecked();
			d_cfg[ Project::CFG_WARN_OFF      ] = d_warn_off_rbtn->isChecked();
		}
	}
	
	return retval;
}
// end of accept

//*******************************************************************
// get_data                                                   PUBLIC
//-------------------------------------------------------------------
// Funkcja zewnetrzna pobiera zapamietane ustawienia.
//*******************************************************************
void ProjectPropGeneral::get_data ( QString& out_fpath,
									QString& out_tmpl,
									QHash< QString, bool >& out_qt,
									QHash< QString, bool >& out_cfg )
{
	out_fpath = d_fpath;
	out_tmpl = d_tmpl;
	out_qt = d_qt;
	out_cfg = d_cfg;	
}	
// end of data

//*******************************************************************
// set_data                                                   PUBLIC
//-------------------------------------------------------------------
// Funkcja aktualizuje w dialogu te ustawienia, ktore zostaly
// przyslane. Funkcja jest stosowana przy zmianie ustawien w
// aktualnie otwartym projekcie.
//*******************************************************************
void ProjectPropGeneral::set_data ( const QString& in_fpath,
									const QString& in_tmpl,
									const QHash< QString, bool > &in_qt,
									const QHash< QString, bool > &in_cfg )
{
	d_project_path_le->setText( in_fpath );

	const qint32 idx = d_template_cbx->findText( in_tmpl );
	if( idx != -1 ) d_template_cbx->setCurrentIndex( idx );
	

	d_core_cbx->setChecked     ( in_qt[ Project::QT_CORE            ] );
	d_gui_cbx->setChecked      ( in_qt[ Project::QT_GUI             ] );
	d_network_cbx->setChecked  ( in_qt[ Project::QT_NETWORK         ] );
	d_opengl_cbx->setChecked   ( in_qt[ Project::QT_OPENGL          ] );
	d_sql_cbx->setChecked      ( in_qt[ Project::QT_SQL             ] );
	d_svg_cbx->setChecked      ( in_qt[ Project::QT_SVG             ] );
	d_xml_cbx->setChecked      ( in_qt[ Project::QT_XML             ] );

	d_qt_cbx->setChecked       ( in_cfg[ Project::CFG_QT            ] );	
	d_x11_cbx->setChecked      ( in_cfg[ Project::CFG_X11           ] );
	d_thread_cbx->setChecked   ( in_cfg[ Project::CFG_THREAD        ] );
	d_release_rbtn->setChecked ( in_cfg[ Project::CFG_RELEASE       ] );
	d_debug_rbtn->setChecked   ( in_cfg[ Project::CFG_DEBUG         ] );
	d_release_and_debug_rbtn->setChecked( in_cfg[ Project::CFG_RELEASE_DEBUG ] );
	d_build_all_cbx->setChecked( in_cfg[ Project::CFG_BUILD_ALL     ] );
	d_ordered_cbx->setChecked  ( in_cfg[ Project::CFG_ORDERED       ] );
	d_warn_on_rbtn->setChecked ( in_cfg[ Project::CFG_WARN_ON       ] );
	d_warn_off_rbtn->setChecked( in_cfg[ Project::CFG_WARN_OFF      ] );

	// Poniewaz plik projektu juz istnieje nalezy uniemozliwic
	// zmiane sciezki do niego.
	d_project_path_le->setEnabled( false );
}
// end of set_data

//*******************************************************************
// text_changed_slot                                    PRIVATE slot
//-------------------------------------------------------------------
// Podano nazwe sciezki do projektu.
// Nalezy odblokowac zakladki pozwalajace wskazac pliki wchodzace
// w sklad projektu.
//*******************************************************************
void ProjectPropGeneral::text_changed_slot( const QString& in_text )
{
	emit enable_tabs( QFile::exists( in_text ) );
}
// end of editing_finished_slot

//*******************************************************************
// dir                                                        PUBLIC
//*******************************************************************
const QString& ProjectPropGeneral::dir()
{
	const QFileInfo finfo( d_project_path_le->text() );
	return ( d_fpath = finfo.exists() ? finfo.absolutePath() : "" );
}
// end of dir

//*******************************************************************
// create_project_file                                       PRIVATE
//-------------------------------------------------------------------
// Proba utworzenia nowego pliku projektu.
// Jesli plik projektu o podanej nazwie juz istnieje, uzytkownik
// ma mozliwosc potwierdzenia, ze chce utworzyc nowy plik projektu,
// lub rezygnacje z jego tworzenia.
//*******************************************************************
bool ProjectPropGeneral::create_project_file( const QString& in_fpath )
{
	bool retval = true;

	const QFileInfo finfo( in_fpath );
	if( finfo.exists() ) {
		const int ret = QMessageBox::question( 0,
							tr( FILE_EXIST_CAPTION ),
							tr( FILE_EXIST_MSG ).arg( in_fpath ),
							QMessageBox::Yes,
							QMessageBox::No );
		if( QMessageBox::Yes == ret ) {
			// Uzytkownik zgodzil sie na usuniecie istniejacego pliku.
			retval = QFile::remove( in_fpath );
		}
		else {
			// Uzytkownik nie chce nadpisywac pliku.
			retval = false;
		}
	}

	if( retval ) {
		// Mozna utworzyc nowy, pusty plik projecktu
		retval = Shared::create_empty_file( in_fpath );
	}
	return retval;
}
// end of create_project_file
