/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : ProjectQtMake.cpp
 * DATE   : 07.03.2007
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "ProjectQtMake.h"
#include "QtMake.h"
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QFont>
#include <QPushButton>
#include <QtDebug>

/*------- local constants:
-------------------------------------------------------------------*/
const char* const ProjectQtMake::PROPOSITION = QT_TR_NOOP( "&Proposition" );
const char* const ProjectQtMake::CLEAR_ALL   = QT_TR_NOOP( "&Clear all" );
const QString     ProjectQtMake::PROP_DBG    = "-O0 -g";
const QString     ProjectQtMake::PROP_REL    = "-O3";

//*******************************************************************
// ProjectQtMake                                         CONSTRUCTOR
//*******************************************************************
ProjectQtMake::ProjectQtMake( QWidget* const in_parent )
: QWidget ( in_parent             )
, d_table ( new DataTable( this ) )
{
	QVBoxLayout* const mlay = new QVBoxLayout( this );
	mlay->addWidget( d_table );

	QHBoxLayout* const blay = new QHBoxLayout;
	QPushButton* const btn_proposition = new QPushButton( tr( PROPOSITION ) );
	QPushButton* const btn_clear_all   = new QPushButton( tr( CLEAR_ALL ) );
	blay->addStretch( 100 );
	blay->addWidget( btn_proposition );
	blay->addWidget( btn_clear_all );
	mlay->addLayout( blay );

	QStringList lista = QtMake::instance()->values();
	lista.sort();

	qint32 row = 0;
	QStringList::const_iterator it = lista.begin();
	const QStringList::const_iterator end = lista.end();
	while( it != end ) {
		d_table->add_row( row );
		d_table->set_item( row++, 0, *it );
		++it;
	}

	connect( btn_proposition, SIGNAL( clicked() ), this, SLOT( proposition() ) );
	connect( btn_clear_all  , SIGNAL( clicked() ), this, SLOT( clear_all  () ) );
}
// end of ProjectFlags

//*******************************************************************
// proposition                                          PRIVATE slot
//*******************************************************************
void ProjectQtMake::proposition()
{
	QString           name  = "";
	QString           value = "";
	const qint32      n     = d_table->rowCount();

	for( qint32 i = 0; i < n; ++i ) {
		value.clear();
		d_table->get_item( i, 0, name );

             if( "QMAKE_CXXFLAGS_DEBUG"   == name ) value = PROP_DBG;
		else if( "QMAKE_CXXFLAGS_RELEASE" == name ) value = PROP_REL;
		d_table->set_item( i, 1, value );
	}
}
// end of proposition

//*******************************************************************
// clear_all                                            PRIVATE slot
//*******************************************************************
void ProjectQtMake::clear_all()
{
	const qint32 n = d_table->rowCount();
	for( qint32 i = 0; i < n; ++i ) {
		d_table->set_item( i, 1, "" );
	}
}
// end of clear_all

//*******************************************************************
// set_qt_make                                                PUBLIC
//*******************************************************************
void ProjectQtMake::set_qt_make( const QHash< QString, QString >& in_data )
{
	QString name = "";
	QString value = "";
	const qint32 n = d_table->rowCount();
	for( qint32 i = 0; i < n; ++i ) {
		value.clear();
		d_table->get_item( i, 0, name );
		if( in_data.contains( name ) ) {
			value = in_data[ name ];
		}
		d_table->set_item( i, 1, value );
	}
}
// end of set_qt_make

//*******************************************************************
// get_qt_make                                                PUBLIC
//*******************************************************************
void ProjectQtMake::get_qt_make( QHash< QString, QString >& out_data )
{
	out_data.clear();

	QString name = "";
	QString value = "";
	const qint32 n = d_table->rowCount();
	for( qint32 i = 0; i < n; ++i ) {
		d_table->get_item( i, 1, value );
		if( !value.isEmpty() ) {
			d_table->get_item( i, 0, name );
			out_data[ name ] = value;
		}
	}

	out_data.squeeze();
}
// end of  get_flags
