/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : ProjectTools.cpp
 * DATE   : 21.05.2007
 *******************************************************************/


/*------- include files:
-------------------------------------------------------------------*/
#include "ProjectTools.h"
#include "ToolsTable.h"
#include "Config.h"
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QLineEdit>
#include <QPushButton>
#include <QGroupBox>
#include <QFileDialog>
#include <QtDebug>

/*------- local constants:
-------------------------------------------------------------------*/
const char* const ProjectTools::QT_MAKE      = QT_TR_NOOP( "&QMake" );
const char* const ProjectTools::BEFORE_QMAKE = QT_TR_NOOP( "Commands befor qmake" );
const char* const ProjectTools::AFTER_QMAKE  = QT_TR_NOOP( "Commands after qmake" );
const char* const ProjectTools::UP_ITEM      = QT_TR_NOOP( "Up" );
const char* const ProjectTools::DOWN_ITEM    = QT_TR_NOOP( "Down" );
const char* const ProjectTools::ADD_ITEM     = QT_TR_NOOP( "Add" );
const char* const ProjectTools::INS_ITEM     = QT_TR_NOOP( "Insert" );
const char* const ProjectTools::DEL_ITEM     = QT_TR_NOOP( "Remove" );
const char* const ProjectTools::CAPTION      = QT_TR_NOOP( "Select qmake location" );
const char* const ProjectTools::FILTER       = QT_TR_NOOP( "All files (*)" );

//*******************************************************************
// ProjectTools                                          CONSTRUCTOR
//*******************************************************************
ProjectTools::ProjectTools( QWidget* const in_parent )
: QWidget      ( in_parent )
, qmake_edit_  ( new QLineEdit )
, before_list_ ( new ToolsTable )
, after_list_  ( new ToolsTable )
{
	QVBoxLayout* const main_layout = new QVBoxLayout( this );

	// QMake
	QHBoxLayout* const qmake_layout = new QHBoxLayout;
	QPushButton* const qmake_button = new QPushButton( "..." );
	QLabel* const qmake_label = new QLabel( tr( QT_MAKE ), this );
	qmake_label->setBuddy( qmake_edit_ );
	qmake_button->setMaximumWidth( 5 * fontMetrics().width( "X" ) );
	qmake_layout->addWidget( qmake_label );
	qmake_layout->addWidget( qmake_edit_ );
	qmake_layout->addWidget( qmake_button );
	
	// Befor qmake
	QGroupBox* const before_grp = new QGroupBox( tr( BEFORE_QMAKE ) );
	QHBoxLayout* const before_layout = new QHBoxLayout( before_grp );
	before_layout->addWidget( before_list_ );
	QVBoxLayout* const before_btn_layout  = new QVBoxLayout;
	QPushButton* const before_up_button   = new QPushButton( tr( UP_ITEM   ) );
	QPushButton* const before_down_button = new QPushButton( tr( DOWN_ITEM ) );
	QPushButton* const before_add_button  = new QPushButton( tr( ADD_ITEM  ) );
	QPushButton* const before_ins_button  = new QPushButton( tr( INS_ITEM  ) );
	QPushButton* const before_del_button  = new QPushButton( tr( DEL_ITEM  ) );
	before_btn_layout->addStretch();
	before_btn_layout->addWidget( before_up_button );
	before_btn_layout->addWidget( before_down_button );
	before_btn_layout->addStretch();
	before_btn_layout->addWidget( before_add_button );
	before_btn_layout->addWidget( before_ins_button );
	before_btn_layout->addWidget( before_del_button );
	before_layout->addLayout( before_btn_layout );
	
	// After qmake
	QGroupBox* const after_grp = new QGroupBox( tr( AFTER_QMAKE ) );
	QHBoxLayout* const after_layout = new QHBoxLayout( after_grp );
	after_layout->addWidget( after_list_ );
	QVBoxLayout* const after_btn_layout  = new QVBoxLayout;
	QPushButton* const after_up_button   = new QPushButton( tr( UP_ITEM   ) );
	QPushButton* const after_down_button = new QPushButton( tr( DOWN_ITEM ) );
	QPushButton* const after_add_button  = new QPushButton( tr( ADD_ITEM  ) );
	QPushButton* const after_ins_button  = new QPushButton( tr( INS_ITEM  ) );
	QPushButton* const after_del_button  = new QPushButton( tr( DEL_ITEM  ) );
	after_btn_layout->addStretch();
	after_btn_layout->addWidget( after_up_button );
	after_btn_layout->addWidget( after_down_button );
	after_btn_layout->addStretch();
	after_btn_layout->addWidget( after_add_button );
	after_btn_layout->addWidget( after_ins_button );
	after_btn_layout->addWidget( after_del_button );
	after_layout->addLayout( after_btn_layout );
	
	// Finish
	main_layout->addLayout( qmake_layout );
	main_layout->addWidget( before_grp );
	main_layout->addWidget( after_grp );

	connect( qmake_button      , SIGNAL( clicked() ), this        , SLOT( qmake_dir_slot () ) );
	connect( before_up_button  , SIGNAL( clicked() ), before_list_, SLOT( up             () ) );
	connect( before_down_button, SIGNAL( clicked() ), before_list_, SLOT( down           () ) );
	connect( before_add_button , SIGNAL( clicked() ), before_list_, SLOT( append         () ) );
	connect( before_ins_button , SIGNAL( clicked() ), before_list_, SLOT( insert         () ) );
	connect( before_del_button , SIGNAL( clicked() ), before_list_, SLOT( remove         () ) );
	connect( after_up_button   , SIGNAL( clicked() ), after_list_ , SLOT( up             () ) );
	connect( after_down_button , SIGNAL( clicked() ), after_list_ , SLOT( down           () ) );
	connect( after_add_button  , SIGNAL( clicked() ), after_list_ , SLOT( append         () ) );
	connect( after_ins_button  , SIGNAL( clicked() ), after_list_ , SLOT( insert         () ) );
	connect( after_del_button  , SIGNAL( clicked() ), after_list_ , SLOT( remove         () ) );

	read_data();
}
// end of ProjectTools

//*******************************************************************
// ~ProjectTools                                          DESTRUCTOR
//*******************************************************************
ProjectTools::~ProjectTools()
{
}
// end of ~ProjectTools

//*******************************************************************
// accept                                                     PUBLIC
//*******************************************************************
bool ProjectTools::accept()
{
	bool retval = false;

	const QString qmake_path = qmake_edit_->text();
	if( !qmake_path.isEmpty() ) {
		retval = true;
		save_data( qmake_path );
	}

	return retval;
}
// end of accept

//*******************************************************************
// read_data                                                 PRIVATE
//*******************************************************************
void ProjectTools::read_data()
{
	qmake_edit_ ->setText ( Config::instance()->qmake_path  () );
	before_list_->set_data( Config::instance()->qmake_before() );
	after_list_ ->set_data( Config::instance()->qmake_after () );
}
// end of read_data

//*******************************************************************
// save_data                                                 PRIVATE
//*******************************************************************
void ProjectTools::save_data( const QString& in_qmake_path )
{
	QStringList strings;

	Config::instance()->qmake_path( in_qmake_path );

	before_list_->get_data( strings );
	Config::instance()->qmake_before( strings );

	after_list_->get_data( strings );
	Config::instance()->qmake_after( strings );

	Config::instance()->save();
}
// end of save_data

//*******************************************************************
// qmake_dir_slot                                       PRIVATE slot
//*******************************************************************
void ProjectTools::qmake_dir_slot()
{
	QString fpath = QFileDialog::getOpenFileName( this, tr( CAPTION ), "/", tr( FILTER ) );

	if( fpath != QString::null ) {
		fpath = fpath.trimmed();
		if( !fpath.isEmpty() ) {
			qmake_edit_->setText( fpath );
			Config::instance()->save();
		}
	}
}
// end of qmake_dir_slot
