/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : PropertiesEdit.cpp
 * DATE   : 22.10.2006
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "PropertiesEdit.h"
#include "EventsController.h"
#include "PropertiesNumbers.h"
#include "PropertiesRMargin.h"
#include "Config.h"
#include "Shared.h"
#include "Settings.h"
#include <QLayout>
#include <QLabel>
#include <QCheckBox>
#include <QPushButton>
#include <QGroupBox>
#include <QGridLayout>
#include <QListWidget>
#include <QSpinBox>
#include <QLineEdit>
#include <QColorDialog>
#include <QFontDialog>
#include <QtDebug>

/*------- local constants:
-------------------------------------------------------------------*/
const char* const PropertiesEdit::DEFAULT         = QT_TR_NOOP( "Use &Default Configuration" );
const char* const PropertiesEdit::HILI_LINE       = QT_TR_NOOP( "&Highlight current line" );
const char* const PropertiesEdit::TAB_WIDTH       = QT_TR_NOOP( "&Tab width" );
const char* const PropertiesEdit::AUTO_INDENT     = QT_TR_NOOP( "Enable &automatic Indentation" );
const char* const PropertiesEdit::TAB2SPC         = QT_TR_NOOP( "Tabs as &Spaces" );
const char* const PropertiesEdit::NUMBERS         = QT_TR_NOOP( "Line &numbers" );
const char* const PropertiesEdit::RMARGIN         = QT_TR_NOOP( "Right &margin" );
const char* const PropertiesEdit::EDITING_GRP     = QT_TR_NOOP( "Editing" );
const char* const PropertiesEdit::BACKGROUNDS     = QT_TR_NOOP( "Backgrounds" );
const char* const PropertiesEdit::CPP_BACKGROUND  = QT_TR_NOOP( "cpp/c" );
const char* const PropertiesEdit::HPP_BACKGROUND  = QT_TR_NOOP( "h/hpp" );
const char* const PropertiesEdit::LINE_BACKGROUND = QT_TR_NOOP( "current line");
const char* const PropertiesEdit::SYNTAX_GRP      = QT_TR_NOOP( "Syntax Highlighting" );
const char* const PropertiesEdit::SYNTAX_CBX      = QT_TR_NOOP( "&Enable syntax highlighting" );
const char* const PropertiesEdit::EXAMPLE_LBL     = QT_TR_NOOP( "Example text for presentation" );
const char* const PropertiesEdit::FONT            = QT_TR_NOOP( "&Font" );
const char* const PropertiesEdit::COLOR           = QT_TR_NOOP( "&Color" );
const char* const PropertiesEdit::ITALIC          = QT_TR_NOOP( "&Italic" );
const char* const PropertiesEdit::BOLD            = QT_TR_NOOP( "&Bold" );
const char* const PropertiesEdit::LIST_ITEMS []   = {
   QT_TR_NOOP( "Text" ),
   QT_TR_NOOP( "Comments" ),
   QT_TR_NOOP( "Strings" ),
   QT_TR_NOOP( "Characters" ),
   QT_TR_NOOP( "Decimal numbers" ),
   QT_TR_NOOP( "Hex numbers" ),
   QT_TR_NOOP( "Keywords" ),
   QT_TR_NOOP( "Native types" ),
   QT_TR_NOOP( "Pragma directive" ),
   QT_TR_NOOP( "Class name" ),
   QT_TR_NOOP( "Qt keywords" ),
   QT_TR_NOOP( "STL keywords" )
};


//*******************************************************************
// PropertiesEdit                                        CONSTRUCTOR
//*******************************************************************
PropertiesEdit::PropertiesEdit( QWidget* const in_parent )
: QWidget          ( in_parent )
, d_main_layout    ( new QVBoxLayout( this ) )
, d_cbx_default    ( new QCheckBox( tr( DEFAULT ), this ) )
, d_gbox1          ( new QGroupBox( tr( EDITING_GRP ), this ) )
, d_grid           ( new QGridLayout( d_gbox1 ) )
, d_hili_line_lbl  ( new QLabel( tr( HILI_LINE ) ) )
, d_hili_line_cbx  ( new QCheckBox( d_gbox1 ) )
, d_tab_width_lbl  ( new QLabel( tr( TAB_WIDTH ) ) )
, d_tab_width_spx  ( new QSpinBox( d_gbox1 ) )
, d_auto_indent_lbl( new QLabel( tr( AUTO_INDENT ) ) )
, d_auto_indent_cbx( new QCheckBox( d_gbox1 ) )
, tab2spc_lbl_     ( new QLabel( tr( TAB2SPC ) ) )
, tab2spc_cbx_     ( new QCheckBox( d_gbox1 ) )
, numbers_btn_     ( new QPushButton( tr( NUMBERS ), d_gbox1 ) )
, rmargin_btn_     ( new QPushButton( tr( RMARGIN ), d_gbox1 ) )
, d_gbox2          ( new QGroupBox( tr( BACKGROUNDS ), this ) )
, d_bkglay         ( new QGridLayout( d_gbox2 ) )
, d_cpp_view       ( new QLineEdit( d_gbox2 ) )
, d_hpp_view       ( new QLineEdit( d_gbox2 ) )
, d_currline_view  ( new QLineEdit( d_gbox2 ) )
, d_cpp_bkg_btn    ( new QPushButton( QIcon( ":/img/colors.png" ), tr( CPP_BACKGROUND ), d_gbox2 ) )
, d_hpp_bkg_btn    ( new QPushButton( QIcon( ":/img/colors.png" ), tr( HPP_BACKGROUND ), d_gbox2 ) )
, d_currline_btn   ( new QPushButton( QIcon( ":/img/colors.png" ), tr( LINE_BACKGROUND ), d_gbox2 ) )
, d_cbx_syntax     ( new QCheckBox( tr( SYNTAX_CBX ), this ))
, d_gbox3          ( new QGroupBox( tr( SYNTAX_GRP ), this ) )
, d_syntax_layout  ( new QHBoxLayout( d_gbox3 ) )
, d_lstlay         ( new QVBoxLayout )
, d_list           ( new QListWidget( d_gbox3 ) )
, d_example_lbl    ( new QLabel( tr( EXAMPLE_LBL ), d_gbox3 ) )
, d_sblay          ( new QVBoxLayout )
, d_btn_font       ( new QPushButton(  QIcon( ":/img/fonts.png" ), tr( FONT ), d_gbox3 ) )
, d_btn_color      ( new QPushButton(  QIcon( ":/img/colors.png" ), tr( COLOR ), d_gbox3 ) )
, d_cbx_italic     ( new QCheckBox( tr( ITALIC ), d_gbox3 ) )
, d_cbx_bold       ( new QCheckBox( tr( BOLD ), d_gbox3 ) )
, d_showed         ( false )
{
   d_cpp_view->setMaximumHeight( 12 );
   d_hpp_view->setMaximumHeight( 12 );
   d_currline_view->setMaximumHeight( 12 );
   d_cpp_view->setFocusPolicy( Qt::NoFocus );
   d_hpp_view->setFocusPolicy( Qt::NoFocus );
   d_currline_view->setFocusPolicy( Qt::NoFocus );
   d_cpp_view->setReadOnly( true );
   d_hpp_view->setReadOnly( true );
   d_currline_view->setReadOnly( true );
   d_cpp_bkg_btn->setMaximumHeight( d_cpp_bkg_btn->iconSize().height() + 2 );
   d_hpp_bkg_btn->setMaximumHeight( d_hpp_bkg_btn->iconSize().height() + 2 );
   d_currline_btn->setMaximumHeight( d_currline_btn->iconSize().height() + 2 );
   d_bkglay->addWidget( d_cpp_view     , 0, 0 );
   d_bkglay->addWidget( d_hpp_view     , 0, 1 );
   d_bkglay->addWidget( d_cpp_bkg_btn  , 1, 0 );
   d_bkglay->addWidget( d_hpp_bkg_btn  , 1, 1 );
   d_bkglay->addWidget( d_currline_view, 2, 0 );
   d_bkglay->addWidget( d_currline_btn , 3, 0 );
   
   d_hili_line_lbl->setBuddy( d_hili_line_cbx );
   d_tab_width_lbl->setBuddy( d_tab_width_spx );
   d_auto_indent_lbl->setBuddy( d_auto_indent_cbx );
   tab2spc_lbl_->setBuddy( tab2spc_cbx_ );
   d_tab_width_spx->setMaximumHeight( fontMetrics().height() + 4 );
   d_grid->addWidget( d_hili_line_lbl  , 0, 0 );
   d_grid->addWidget( d_tab_width_lbl  , 1, 0 );
   d_grid->addWidget( d_auto_indent_lbl, 2, 0 );
   d_grid->addWidget( tab2spc_lbl_     , 3, 0 );
   d_grid->addWidget( numbers_btn_     , 4, 0 );
   d_grid->addWidget( rmargin_btn_     , 5, 0 );
   d_grid->addWidget( d_hili_line_cbx  , 0, 1 );
   d_grid->addWidget( d_tab_width_spx  , 1, 1 );
   d_grid->addWidget( d_auto_indent_cbx, 2, 1 );
   d_grid->addWidget( tab2spc_cbx_     , 3, 1 );
   d_grid->setColumnStretch( 2, 10 );

   qint32 maxlen = 0;
   for( qint32 i = 0; i < Config::EndMarker; ++i ) {
      const QString txt( tr( LIST_ITEMS[ i ] ) );
      d_list->insertItem( i, txt );
      if( txt.length() > maxlen ) {
         maxlen = txt.length();
      }
   }
   d_list->setMaximumWidth( ( maxlen + 2 ) * d_list->fontMetrics().width( "X" ) );
   d_list->setMaximumHeight( 8 * d_list->fontMetrics().height() );
   d_list->setCurrentRow( 0 );
   
   d_lstlay->addWidget( d_list );
   d_lstlay->addWidget( d_example_lbl );

   d_btn_font->setMaximumHeight( d_btn_font->iconSize().height() + 2 );
   d_btn_color->setMaximumHeight( d_btn_color->iconSize().height() + 2 );
   d_sblay->addWidget( d_btn_font );
   d_sblay->addWidget( d_btn_color );
   d_sblay->addWidget( d_cbx_bold );
   d_sblay->addWidget( d_cbx_italic );
   d_sblay->addStretch();

   d_syntax_layout->addLayout( d_lstlay );
   d_syntax_layout->addLayout( d_sblay );

   d_main_layout->addWidget( d_cbx_default );
   d_main_layout->addWidget( d_gbox1 );
   d_main_layout->addWidget( d_gbox2 );
   d_main_layout->addWidget( d_cbx_syntax );
   d_main_layout->addWidget( d_gbox3 );
   
   // Zmiana pozycji na liscie (text, comment, itd ).
   connect( d_list, SIGNAL( currentRowChanged( int ) ), this, SLOT( row_changed( int ) ) );
   // Przyciski i checkbox'y
   connect( d_cbx_default , SIGNAL( clicked() ), this, SLOT( default_clicked() ) );
   connect( d_cpp_bkg_btn , SIGNAL( clicked() ), this, SLOT( cpp_bkg_color()   ) );
   connect( d_hpp_bkg_btn , SIGNAL( clicked() ), this, SLOT( hpp_bkg_color()   ) );
   connect( d_currline_btn, SIGNAL( clicked() ), this, SLOT( currline_color()  ) );
   connect( d_btn_font    , SIGNAL( clicked() ), this, SLOT( font_clicked()    ) );
   connect( d_btn_color   , SIGNAL( clicked() ), this, SLOT( color_clicked()   ) );
   connect( d_cbx_italic  , SIGNAL( clicked() ), this, SLOT( italic_clicked()  ) );
   connect( d_cbx_bold    , SIGNAL( clicked() ), this, SLOT( bold_clicked()    ) );
   
   connect( numbers_btn_  , SIGNAL( clicked         () ),
            this          , SLOT  ( numbers_selected() ) );
   connect( rmargin_btn_  , SIGNAL( clicked         () ),
            this          , SLOT  ( rmargin_selected() ) );
}
// end of PropertiesEdit

//*******************************************************************
// showEvent                                       PRIVATE inherited
//*******************************************************************
void PropertiesEdit::showEvent( QShowEvent* const in_event )
{
   if( !d_showed ) {
      d_cbx_default->setChecked( Config::instance()->default_edit_config() );
      d_hili_line_cbx->setChecked( Config::instance()->line_highlight() );
      d_tab_width_spx->setValue( Config::instance()->tab_width() );
      d_auto_indent_cbx->setChecked( Config::instance()->auto_indent() );
      tab2spc_cbx_->setChecked( Config::instance()->tab2spc() );
      set_background( d_cpp_view, d_cpp_background = Config::instance()->cpp_background() );
      set_background( d_hpp_view, d_hpp_background = Config::instance()->hpp_background() );
      set_background( d_currline_view, d_currline_background = Config::instance()->currline_background() );
      d_cbx_syntax->setChecked( Config::instance()->syntax_highlight() );
      d_syntax = Config::instance()->syntax();

      default_clicked();
      d_showed = true;
   }
   QWidget::showEvent( in_event );
}
// end of showEvent

//*******************************************************************
// set_example_lbl                                           PRIVATE
//*******************************************************************
void PropertiesEdit::set_example_lbl( const QFont& in_font, const QRgb in_rgb )
{
   QPalette palette;
   palette.setColor( d_example_lbl->foregroundRole(), QColor::fromRgba( in_rgb ) );

   d_example_lbl->setPalette( palette );
   d_example_lbl->setFont( in_font );
   
   QString txt = in_font.family();
   txt += ", ";
   txt += QString::number( in_font.pointSize() );
   txt += " pt.";
   d_example_lbl->setText( txt );
   
}
// end of set_example_lbl

//*******************************************************************
// row_changed                                          PRIVATE slot
//*******************************************************************
void PropertiesEdit::row_changed( const int in_row )
{
   d_font.fromString( d_syntax[ in_row ][ 0 ].toString() );

   if( 0 == in_row ) {
      d_btn_font->setEnabled( true );
      d_cbx_italic->setEnabled( false );
      d_cbx_bold->setEnabled( false );
      d_cbx_italic->setChecked( d_font.italic() );
      d_cbx_bold->setChecked( d_font.bold() );
   }
   else {
      d_btn_font->setEnabled( false );
      d_cbx_italic->setEnabled( true );
      d_cbx_bold->setEnabled( true );
      d_cbx_italic->setChecked( d_font.italic() );
      d_cbx_bold->setChecked( d_font.bold() );
   }
   set_example_lbl( d_font, d_syntax[ in_row ][ 1 ].toUInt() );
}
// end of row_changed

//*******************************************************************
// default_clicked                                      PRIVATE slot
//*******************************************************************
void PropertiesEdit::default_clicked()
{
   const bool is_default = d_cbx_default->isChecked();
   d_gbox1->setDisabled( is_default );
   d_gbox2->setDisabled( is_default );
   d_cbx_syntax->setDisabled( is_default );
   d_gbox3->setDisabled( is_default );
}
// end of default_clicked

//*******************************************************************
// set_background                                            PRIVATE
//-------------------------------------------------------------------
// Nadanie koloru wskazanemu polu edycyjnemu (jako demostracja).
//*******************************************************************
void PropertiesEdit::set_background( QLineEdit* in_led, const QRgb in_rgb )
{
   QPalette palette;
   QColor color;
   color.setRgba( in_rgb );
   palette.setColor( in_led->backgroundRole(), color );
   in_led->setPalette( palette );
}
// end of set_background

//*******************************************************************
// cpp_bkg_color                                        PRIVATE slot
//-------------------------------------------------------------------
// Wybor koloru tla dla plikow zrodlowych.
//*******************************************************************
void PropertiesEdit::cpp_bkg_color()
{
   bool ok;
   const QRgb qrgb = QColorDialog::getRgba( d_cpp_background, &ok, this );
   if( ok ) {
      set_background( d_cpp_view, d_cpp_background = qrgb );
   }
}
// end of cpp_bkg_color

//*******************************************************************
// hpp_bkg_color                                        PRIVATE slot
//-------------------------------------------------------------------
// Wybor koloru tla dla plikow naglowkowych.
//*******************************************************************
void PropertiesEdit::hpp_bkg_color()
{
   bool ok;
   const QRgb qrgb = QColorDialog::getRgba( d_hpp_background, &ok, this );
   if( ok ) {
      set_background( d_hpp_view, d_hpp_background = qrgb );
   }
}
// end of hpp_bkg_color

//*******************************************************************
// currline_color                                       PRIVATE slot
//-------------------------------------------------------------------
// Wybor koloru tla dla aktualnej linii.
//*******************************************************************
void PropertiesEdit::currline_color()
{
   bool ok;
   const QRgb qrgb = QColorDialog::getRgba( d_currline_background, &ok, this );
   if( ok ) {
      set_background( d_currline_view, d_currline_background = qrgb );
   }
}
// end of currline_color

//*******************************************************************
// font_clicked                                         PRIVATE slot
//-------------------------------------------------------------------
// Uzytkownik chce wybrac font.
// Taka mozliwosc istnienie tylko dla pozycji 'Text'.
// Wynika to z tego, ze zakladamy ze wszystkie elementu tekstu
// sa pisane tym samym fontem. Oznacza to, ze wybrany font nalezy
// wpisac do wszystkich elementow listy.
//*******************************************************************
void PropertiesEdit::font_clicked()
{
   const qint32 row = d_list->currentRow();

   if( ( row >= Config::Text ) && ( row < Config::EndMarker ) ) {
      bool ok;
      d_font.fromString( d_syntax[ row ][ 0 ].toString() );
      const QFont fnt = QFontDialog::getFont( &ok, d_font, this );
      if( ok ) {
         const QString fnt_txt = fnt.toString();
         for( qint32 i = 0; i < Config::EndMarker; ++i ) {
            d_syntax[ i ][ 0 ] = fnt_txt;
         }
         d_font.fromString( d_syntax[ row ][ 0 ].toString() );
         set_example_lbl( d_font, d_syntax[ row ][ 1 ].toUInt() );
      }
   }
}
// end of font_clicked

//*******************************************************************
// color_clicked                                        PRIVATE slot
//*******************************************************************
void PropertiesEdit::color_clicked()
{
   const qint32 row = d_list->currentRow();

   if( ( row >= Config::Text ) && ( row < Config::EndMarker ) ) {
      bool ok;
      const QRgb rgb = QColorDialog::getRgba( d_syntax[ row ][ 1 ].toUInt(), &ok, this );
      if( ok ) {
         d_syntax[ row ][ 1 ] = rgb;
         set_example_lbl( d_font, rgb );
      }
   }
}
// end of italic_clicked

//*******************************************************************
// italic_clicked                                       PRIVATE slot
//*******************************************************************
void PropertiesEdit::italic_clicked()
{
   const qint32 row = d_list->currentRow();

   if( ( row >= Config::Text ) && ( row < Config::EndMarker ) ) {
      const bool is_italic = d_cbx_italic->isChecked();
      d_font.fromString( d_syntax[ row ][ 0 ].toString() );
      d_font.setItalic( is_italic );
      d_syntax[ row ][ 0 ] = d_font.toString();
      set_example_lbl( d_font, d_syntax[ row ][ 1 ].toUInt() );
   }
}
// end of italic_clicked

//*******************************************************************
// bold_clicked                                         PRIVATE slot
//*******************************************************************
void PropertiesEdit::bold_clicked()
{
   const qint32 row = d_list->currentRow();

   if( ( row >= Config::Text ) && ( row < Config::EndMarker ) ) {
      const bool is_bold = d_cbx_bold->isChecked();
      d_font.fromString( d_syntax[ row ][ 0 ].toString() );
      d_font.setBold( is_bold );
      d_syntax[ row ][ 0 ] = d_font.toString();
      set_example_lbl( d_font, d_syntax[ row ][ 1 ].toUInt() );
   }
}
// end of bold_clicked

//*******************************************************************
// numbers_selected                                     PRIVATE slot
//*******************************************************************
void PropertiesEdit::numbers_selected()
{
   PropertiesNumbers* dialog = new PropertiesNumbers( this );
   if( dialog ) {
      dialog->exec();
      delete dialog;
   }
}
// end of numbers_selected

//*******************************************************************
// rmargin_selected                                     PRIVATE slot
//*******************************************************************
void PropertiesEdit::rmargin_selected()
{
   PropertiesRMargin* dialog = new PropertiesRMargin( this );
   if( dialog ) {
      dialog->exec();
      delete dialog;
   }
}
// end of rmargin_selected

//*******************************************************************
// save_data                                            PRIVATE slot
//*******************************************************************
void PropertiesEdit::save_data()
{
   if( d_showed ) {
      const bool is_unchanged  = 
            ( d_cbx_default->isChecked()     == Config::instance()->default_edit_config() )
         &&   ( d_hili_line_cbx->isChecked()   == Config::instance()->line_highlight() )
         && ( d_tab_width_spx->value()       == Config::instance()->tab_width() )
         && ( d_auto_indent_cbx->isChecked() == Config::instance()->auto_indent() )
         && ( tab2spc_cbx_->isChecked()      == Config::instance()->tab2spc() )
         && ( d_cbx_syntax->isChecked()      == Config::instance()->syntax_highlight() )
         && ( d_syntax                       == Config::instance()->syntax() )
         && ( d_cpp_background               == Config::instance()->cpp_background() )
         && ( d_hpp_background               == Config::instance()->hpp_background() )
         && ( d_currline_background          == Config::instance()->currline_background() );

      if( !is_unchanged ) {
         Config::instance()->default_edit_config( d_cbx_default->isChecked() );
         Config::instance()->line_highlight( d_hili_line_cbx->isChecked() );
         Config::instance()->tab_width( d_tab_width_spx->value() );
         Config::instance()->auto_indent( d_auto_indent_cbx->isChecked() );
         Config::instance()->tab2spc( tab2spc_cbx_->isChecked() );
         Config::instance()->cpp_background( d_cpp_background );
         Config::instance()->hpp_background( d_hpp_background );
         Config::instance()->currline_background( d_currline_background );
         Config::instance()->syntax_highlight( d_cbx_syntax->isChecked() );
         Config::instance()->syntax( d_syntax );
         
         Config::instance()->save();
      
         Config::instance()->read();
         EventsController::instance()->send_event( Event::FONT_CHANGED );
      }   
   }
}
// end of save_data
