/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : PropertiesGeneral.cpp
 * DATE   : 22.10.2006
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "PropertiesGeneral.h"
#include "Config.h"
#include <QLabel>
#include <QLayout>
#include <QGroupBox>
#include <QCheckBox>
#include <QLineEdit>
#include <QGridLayout>
#include <QPushButton>
#include <QFileDialog>
#include <QRadioButton>
#include <QButtonGroup>
#include <QtDebug>

/*------- CONSTANTS:
-------------------------------------------------------------------*/
const char* const PropertiesGeneral::GBOX1_TITLE         = QT_TR_NOOP( "Load At Start" );
const char* const PropertiesGeneral::GBOX2_TITLE         = QT_TR_NOOP( "Qmake" );
const char* const PropertiesGeneral::LOAD_PROJECT        = QT_TR_NOOP( "Load the last &project at start" );
const char* const PropertiesGeneral::LOAD_FILES          = QT_TR_NOOP( "Load at start last opened &files" );
const char* const PropertiesGeneral::QMAKE_PATH          = QT_TR_NOOP( "Location of a &qmake" );
const char* const PropertiesGeneral::CAPTION             = QT_TR_NOOP( "Select a qmake location" );
const char* const PropertiesGeneral::FILTER              = QT_TR_NOOP( "All files (*)" );
const char* const PropertiesGeneral::CLASS_PARSING_TITLE = QT_TR_NOOP( "Class Reparsing" );
const char* const PropertiesGeneral::PARSE_ON_SAVE       = QT_TR_NOOP( "Reparse classes on file save" );
const char* const PropertiesGeneral::PARSE_ON_LINE       = QT_TR_NOOP( "Reparse classes online" );

//*******************************************************************
// PropertiesGeneral                                     CONSTRUCTOR
//*******************************************************************
PropertiesGeneral::PropertiesGeneral( QWidget* const in_parent )
: QWidget       ( in_parent )
, d_main_layout ( new QVBoxLayout( this ) )
, d_gbox1       ( new QGroupBox( tr( GBOX1_TITLE ), this ) )
, d_glay1       ( new QGridLayout( d_gbox1 ) )
, d_pload_lbl   ( new QLabel( tr( LOAD_PROJECT ) ) )
, d_pload_cbx   ( new QCheckBox( d_gbox1 ) )
, d_fload_lbl   ( new QLabel( tr( LOAD_FILES ) ) )
, d_fload_cbx   ( new QCheckBox( d_gbox1 ) )
, d_gbox2       ( new QGroupBox( tr( GBOX2_TITLE ), this ) )
, d_glay2       ( new QGridLayout( d_gbox2 ) )
, d_qmake_lbl   ( new QLabel( tr( QMAKE_PATH ) ) )
, d_qmake_ted   ( new QLineEdit( d_gbox1 ) )
, d_qmake_btn   ( new QPushButton( "...", d_gbox2 ) )
, d_gbox3       ( new QGroupBox( tr( CLASS_PARSING_TITLE ), this ) )
, d_glay3       ( new QVBoxLayout( d_gbox3 ) )
, d_parse_bgrp  ( new QButtonGroup( d_gbox3 ) )
, d_onsave_rbx  ( new QRadioButton( tr( PARSE_ON_SAVE ), d_gbox3 ) )
, d_online_rbx  ( new QRadioButton( tr( PARSE_ON_LINE ), d_gbox3 ) )
, d_showed      ( false )
{
	d_qmake_btn->setMaximumWidth( 3 * d_qmake_btn->fontMetrics().width( "X" ) );
	
	d_pload_lbl->setBuddy( d_pload_cbx );
	d_fload_lbl->setBuddy( d_fload_cbx );
	d_qmake_lbl->setBuddy( d_qmake_ted );
	
	d_glay1->addWidget( d_pload_lbl, 0, 0 );
	d_glay1->addWidget( d_pload_cbx, 0, 1 );
	d_glay1->addWidget( d_fload_lbl, 1, 0 );
	d_glay1->addWidget( d_fload_cbx, 1, 1 );
	d_main_layout->addWidget( d_gbox1 );
	
	d_glay2->addWidget( d_qmake_lbl, 0, 0 );
	d_glay2->addWidget( d_qmake_ted, 1, 0 );
	d_glay2->addWidget( d_qmake_btn, 1, 1 );
	d_main_layout->addWidget( d_gbox2 );

	d_glay3->addWidget( d_onsave_rbx );
	d_glay3->addWidget( d_online_rbx );
	d_parse_bgrp->addButton( d_onsave_rbx );
	d_parse_bgrp->addButton( d_online_rbx );
	d_main_layout->addWidget( d_gbox3 );

	d_main_layout->addStretch( 1 );
		
	connect( d_qmake_btn, SIGNAL( clicked() ), this, SLOT( get_path() ) );
}
// end of PropertiesGeneral

//*******************************************************************
// showEvent                                       PRIVATE inherited
//*******************************************************************
void PropertiesGeneral::showEvent( QShowEvent* const in_event )
{
	d_showed = true;
	
	d_pload_cbx->setChecked( Config::instance()->load_project() );
	d_fload_cbx->setChecked( Config::instance()->load_files() );
	d_qmake_ted->setText( Config::instance()->qmake_path() );
	d_onsave_rbx->setChecked( Config::instance()->parse_class_onsave() );
	d_online_rbx->setChecked( Config::instance()->parse_class_online() );
	
	QWidget::showEvent( in_event );
}
// end of showEvent

//*******************************************************************
// save_data                                            PRIVATE slot
//*******************************************************************
void PropertiesGeneral::save_data()
{
	if( d_showed ) {
		const bool is_unchanged =
				( d_pload_cbx->isChecked()  == Config::instance()->load_project() )
			&&	( d_fload_cbx->isChecked()  == Config::instance()->load_files() )
			&&	( d_qmake_ted->text()       == Config::instance()->qmake_path() )
			&&  ( d_onsave_rbx->isChecked() == Config::instance()->parse_class_onsave() );
			
		if( !is_unchanged ) {
			Config::instance()->load_project( d_pload_cbx->isChecked() );
			Config::instance()->load_files( d_fload_cbx->isChecked() );
			Config::instance()->qmake_path( d_qmake_ted->text() );
			Config::instance()->parse_class_onsave( d_onsave_rbx->isChecked() );
			Config::instance()->save();
		}
	}
}
// end of save_data

//*******************************************************************
// get_path                                             PRIVATE slot
//*******************************************************************
void PropertiesGeneral::get_path()
{
	QString fpath = QFileDialog::getOpenFileName( this, tr( CAPTION ), "/", tr( FILTER ) );

	if( fpath != QString::null ) {
		fpath = fpath.trimmed();
		if( !fpath.isEmpty() ) {
			d_qmake_ted->setText( fpath );
		}
	}
}
// end of get_path
