/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : PropertiesMsgTab.cpp
 * DATE   : 22.10.2006
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "PropertiesMsgTab.h"
#include "Config.h"
#include "EventsController.h"
#include <QLayout>
#include <QLabel>
#include <QCheckBox>
#include <QGroupBox>
#include <QLineEdit>
#include <QGridLayout>
#include <QPushButton>
#include <QFontDialog>
#include <QColorDialog>
#include <QtDebug>

//*******************************************************************
// PropertiesMsgTab                                      CONSTRUCTOR
//*******************************************************************
PropertiesMsgTab::PropertiesMsgTab( QWidget* const in_parent )
: PropertiesWidget( in_parent )
, d_showed        ( false )
{}
// end of PropertiesMsgTab

//*******************************************************************
// showEvent                                       PRIVATE inherited
//*******************************************************************
void PropertiesMsgTab::showEvent( QShowEvent* const in_event )
{
	if( !d_showed ) {
		d_defaults_cbx->setChecked( Config::instance()->msg_tab_use_defaults() );
		d_altbkg_cbx->setChecked( Config::instance()->msg_tab_use_altbkg() );
		set_background( d_bkg_view, d_bkg_rgb = Config::instance()->msg_tab_base_bkg() );
		set_background( d_altbkg_view, d_altbkg_rgb = Config::instance()->msg_tab_altbase_bkg() );
		d_font_as_string = Config::instance()->msg_tab_font();
		d_font_color = Config::instance()->msg_tab_font_color();
		
		set_example_lbl( d_font_as_string, d_font_color );
		colors();
		d_showed = true;
	}
	PropertiesWidget::showEvent( in_event );
}
// end of showEvent

//*******************************************************************
// save_data                                            PRIVATE slot
//*******************************************************************
void PropertiesMsgTab::save_data()
{
	if( d_showed ) {
		const bool is_unchanged = 
				( d_defaults_cbx->isChecked() == Config::instance()->msg_tab_use_defaults() )
			&&	( d_altbkg_cbx->isChecked()   == Config::instance()->msg_tab_use_altbkg() )
			&&	( d_bkg_rgb                   == Config::instance()->msg_tab_base_bkg() )
			&&	( d_altbkg_rgb                == Config::instance()->msg_tab_altbase_bkg() )
			&&	( d_font_as_string            == Config::instance()->msg_tab_font() )
			&&	( d_font_color                == Config::instance()->msg_tab_font_color() );


		if( !is_unchanged ) {
			Config::instance()->msg_tab_use_defaults( d_defaults_cbx->isChecked() );
			Config::instance()->msg_tab_use_altbkg( d_altbkg_cbx->isChecked() );
			Config::instance()->msg_tab_base_bkg( d_bkg_rgb );
			Config::instance()->msg_tab_altbase_bkg( d_altbkg_rgb );
			Config::instance()->msg_tab_font( d_font_as_string );
			Config::instance()->msg_tab_font_color( d_font_color );
			
			Config::instance()->save();
			Config::instance()->read();
			EventsController::instance()->send_event( Event::MSG_TAB_PROP_CHANGED );
		}
	}
}
// end of save_data
