/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : PropertiesWidget.h
 * DATE   : 30.10.2006
 *******************************************************************/
#ifndef INCLUDED_PropertiesWidget_h
#define INCLUDED_PropertiesWidget_h

/*------- include files:
-------------------------------------------------------------------*/
#ifndef INCLUDED_QWidget
#define INCLUDED_QWidget
#include <QWidget>
#endif // INCLUDED_QWidget

/*------- forward declarations:
-------------------------------------------------------------------*/
class QLabel;
class QCheckBox;
class QGroupBox;
class QLineEdit;
class QVBoxLayout;
class QGridLayout;
class QPushButton;

/*------- class declaration:
-------------------------------------------------------------------*/
class PropertiesWidget : public QWidget
{
	Q_OBJECT

// ******* CONSTRUCTION / DESTRUCTION ********
public:
	PropertiesWidget( QWidget* );
private:
	PropertiesWidget( const PropertiesWidget& );
	PropertiesWidget& operator=( const PropertiesWidget& );

// ******* CONSTANTS *******
private:
	static const char* const USE_DEFAULTS;
	static const char* const USE_ALTBKG;
	static const char* const BACKGROUNDS;
	static const char* const NORMAL_BKG;
	static const char* const ALTERNATE_BKG;
	static const char* const FONT_TITLE;
	static const char* const FONT;
	static const char* const FONT_COLOR;
	static const char* const EXAMPLE_LBL;

// ******* MEMBERS *******
protected:
	QVBoxLayout* const d_mlay;
	QCheckBox*   const d_defaults_cbx;
	QCheckBox*   const d_altbkg_cbx;
	QGroupBox*   const d_gbox1;
	QGridLayout* const d_glay1;
	QLineEdit*   const d_bkg_view;
	QLineEdit*   const d_altbkg_view;
	QPushButton* const d_bkg_btn;
	QPushButton* const d_altbkg_btn;
	QGroupBox*   const d_gbox2;
	QGridLayout* const d_glay2;
	QPushButton* const d_font_btn;
	QPushButton* const d_font_color_btn;
	QLabel*      const d_example_lbl;
	QRgb               d_bkg_rgb;
	QRgb               d_altbkg_rgb;
	QString            d_font_as_string;
	QRgb               d_font_color;

// ******* METHODS *******
protected:
	void    set_background ( QLineEdit*, QRgb );
	void   set_example_lbl ( const QString&, QRgb );
private slots:
	void        select_bkg ();
	void     select_altbkg ();
	void       select_font ();
	void select_font_color ();
protected slots:
	void            colors ();
	virtual void save_data () = 0;
};

#endif // INCLUDED_PropertiesWidget_h
