/********************************************************************
 * Copyright (C) 2006-2007 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : QBTFunctionsBox.cpp
 * DATE   : 14.06.2007
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "QBTFunctionsBox.h"
#include "QBTParserFunDef.h"
#include <QHBoxLayout>
#include <QListWidget>
#include <QKeyEvent>
#include <QtDebug>
#include <vector>
using namespace std;

/*------- local constants:
-------------------------------------------------------------------*/
const char* const QBTFunctionsBox::CAPTION = QT_TR_NOOP( "Function list" );

//*******************************************************************
// FunctionsCBox                                         CONSTRUCTOR
//*******************************************************************
QBTFunctionsBox::QBTFunctionsBox( const QString& in_text, QWidget* const in_parent )
: QDialog    ( in_parent, Qt::Tool )
, list_      ( new QListWidget )
, parser_    ( new QBTParserFunDef )
, position_  ( -1 )
{
   setWindowTitle( tr( CAPTION ) );
   list_->setAlternatingRowColors( true );
   
   QHBoxLayout* const mlay = new QHBoxLayout( this );
   mlay->setMargin( 0 );
   mlay->addWidget( list_ );
   
   connect( parser_, SIGNAL( finished() ),
            this   , SLOT  ( finished() ) );
   connect( list_  , SIGNAL( itemActivated  ( QListWidgetItem* ) ),
            this   , SLOT  ( item_activated ( QListWidgetItem* ) ) );
   vector<QBTCodeBlockInfo> tmp;
   parser_->parse_it( in_text, tmp );
}
// end of FunctionsCBox

//*******************************************************************
// ~FunctionsCBox                                         DESTRUCTOR
//*******************************************************************
QBTFunctionsBox::~QBTFunctionsBox()
{
   if( parser_->isRunning() ) {
      parser_->terminate();
      parser_->wait();
   }
   delete parser_;
}
// end of ~FunctionsCBox

//*******************************************************************
// showEvent                                       PRIVATE inherited
//*******************************************************************
void QBTFunctionsBox::showEvent( QShowEvent* const in_event )
{
   QSize s = size();
   s.setHeight( s.height() << 1 );
   resize( s );
   QDialog::showEvent( in_event );
}
// end of showEvent

//*******************************************************************
// finished                                             PRIVATE slot
//*******************************************************************
void QBTFunctionsBox::finished()
{
   vector<QBTCodeBlockInfo> data;
   parser_->data( data );

   vector<QBTCodeBlockInfo>::const_iterator it = data.begin();
   const vector<QBTCodeBlockInfo>::const_iterator end = data.end();
   while( it != end ) {
      QListWidgetItem* const item = new QListWidgetItem( it->header().simplified() );
      item->setData( POSITION, it->pos_idx() );
      list_->addItem( item );
      ++it;
   }
   list_->sortItems();
}
// end of finished

//*******************************************************************
// keyPressEvent                                   PRIVATE inherited
//*******************************************************************
void QBTFunctionsBox::keyPressEvent( QKeyEvent* const in_event )
{
   QDialog::keyPressEvent( in_event );
   
   switch( in_event->key() ) {
      case Qt::Key_Escape:
         in_event->accept();
         reject();
         break;
   }
}
// end of keyPressEvent

//*******************************************************************
// item_activated                                       PRIVATE slot
//*******************************************************************
void QBTFunctionsBox::item_activated( QListWidgetItem* const in_item )
{
   if( in_item ) {
      position_ = in_item->data( POSITION ).toInt();
      accept();
   }
}
// end of item_activated
