/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : QBtFind.h
 * DATE   : 23.09.2006
 *******************************************************************/
#ifndef INCLUDED_QBtFind_h
#define INCLUDED_QBtFind_h

/*------- include files:
-------------------------------------------------------------------*/
#ifndef INCLUDED_QDialog
#define INCLUDED_QDialog
#include <QDialog>
#endif // INCLUDED_QDialog

#ifndef INCLUDED_QComboBox
#define INCLUDED_QComboBox
#include <QComboBox>
#endif // INCLUDED_QComboBox

#ifndef INCLUDED_QCheckBox
#define INCLUDED_QCheckBox
#include <QCheckBox>
#endif // INCLUDED_QCheckBox

/*------- forward declarations:
-------------------------------------------------------------------*/
class QVBoxLayout;
class QHBoxLayout;
class QGridLayout;
class QPushButton;
class QShowEvent;
class QCheckBox;
class QLabel;

/*------- class declaration:
-------------------------------------------------------------------*/
class QBtFind : public QDialog
{
   Q_OBJECT

// ******* CONSTRUCTION / DESTRUCTION *******
public:
   explicit QBtFind( QWidget*, const QString&, bool = true );
   ~QBtFind();
private:
   QBtFind( const QBtFind& );
   QBtFind& operator=( const QBtFind& );

// ******* CONSTANTS *******
private:
   static const char* const FIND_CAPTION;
   static const char* const REPLACE_CAPTION;
   static const char* const SEARCH_FOR;
   static const char* const REPLACE_WITH;
   static const char* const SEARCH_BACKWARD;
   static const char* const WHOLE_DOCUMENT;
   static const char* const SELECTED_TEXT;
   static const char* const WHOLE_WORDS;
   static const char* const CASE_SENSITIVE;
   static const char* const FIND;
   static const char* const CANCEL;

// ******* MEMBERS *******
private:
   QLabel*      const input_label_;
   QComboBox*   const input_;
   QLabel*      const replace_label_;
   QComboBox*   const replace_;
   QCheckBox*   const backward_cbx_;
   QCheckBox*   const whole_document_cbx_;
   QCheckBox*   const selected_text_cbx_;
   QCheckBox*   const whole_cbx_;
   QCheckBox*   const sensitive_cbx_;
   QPushButton* const go_;
   QPushButton* const cancel_;
   const QString      initial_text_;
   bool               selected_text_active_;

// ******* TYPES *******
private:
   enum {
      MAX_ITEMS_NUMBER = 10
   };
// ******* METHODS *******
private:
   void showEvent( QShowEvent* );
   void save_data();
   void restore_data();
   bool update_combo( QComboBox* );
private slots:
   void accept();
public:
   void find_in_project();
   void input_text( QString& out_text ) const { out_text = input_->currentText(); }
   void replace_text( QString& out_text ) const { out_text = replace_->currentText(); }
   bool is_backward() const { return backward_cbx_->isChecked(); }
   bool is_whole_document() const { return whole_document_cbx_->isChecked(); }
   bool is_selected_text() const { return selected_text_cbx_->isChecked(); }
   bool is_whole() const { return whole_cbx_->isChecked(); }
   bool is_sensitive() const { return sensitive_cbx_->isChecked(); }
   void in_selected_text_active() { selected_text_active_ = true; }
};

inline void QBtFind::find_in_project() {
   backward_cbx_->setVisible( false );
}

#endif // INCLUDED_QBtFind_h
