/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : QBtParser.h
 * DATE   : 20.09.2006
 *******************************************************************/
#ifndef INCLUDED_QBtParser_H
#define INCLUDED_QBtParser_H


/*------- include files:
-------------------------------------------------------------------*/
#include <QString>
#include <vector>

/*------- forward declarations:
-------------------------------------------------------------------*/
class QBTCodeBlockInfo;


/*------- class declaration:
-------------------------------------------------------------------*/
class QBtParser
{
// ******* MEMBERS *******
public:
   static const QChar   UNDER_SCORE;
   static const QChar   BACK_SLASH;
   static const QChar   SLASH;
   static const QChar   STERN;
   static const QChar   HASH;
   static const QChar   SINGLE_QUOTE;
   static const QChar   DOUBLE_QUOTE;
   static const QChar   COLON;
   static const QChar   SEMICOLON;
   static const QChar   BLOCK_BEGIN;
   static const QChar   BLOCK_END;
   static const QChar   NEW_LINE;
   static const QString CPP_COMMENT;
   static const QString C_COMMENT_BEGIN;
   static const QString C_COMMENT_END;
private:
   static const QString WHITE_CHARS;
   static const QString DEC_DIGITS;
   static const QString HEX_DIGITS;

// ******* METHODS *******
public:
   static void   reverse              ( QString& );
   static bool   is_white_char        ( const QChar& );
   static bool   is_white_string      ( const QString& );
   static bool   is_dec_digit         ( const QChar& );
   static bool   is_hex_digit         ( const QChar& );
   static bool   is_valid_char        ( const QChar& );
   static bool   is_acceptable        ( const QString&, const int );
   static void   remove_tokens        ( QString&, const QChar&, const QChar& );
   static qint32 search_char          ( const QString&, const QChar&, qint32 );
   static int    first_after_white_chr( const QString&, int = 0 );
   static qint32 first_white_chr      ( const QString&, qint32 );
   static bool   is_hex_number        ( const QString& );
   static bool   is_dec_number        ( const QString& );
   static bool   is_number            ( const QString& );
   static bool   search_word          ( const QString&, int&, int&, const bool );
   static bool   is_word_begin        ( const QString&, int );
   static int    search_word_end      ( const QString&, int, bool = false );
   static qint32 search_next_item     ( const QString&, qint32 );
   static bool   search_item          ( const QString&, int&, int& );
   static qint32 search_chars         ( const QString&, qint32, const QString& );
   static bool   starts_with          ( const QString&, qint32, const QString& );

   static qint32 search_muster        ( const QString&, const QString&, qint32 = 0 );
   static qint32 search_marker_end    ( const QString&, qint32, const QChar&, const QChar&, qint32* = 0 );
   static bool   search_code_block    ( const QString&, qint32, QBTCodeBlockInfo& );
   static qint32 clean_function_head  ( QString& );
   static qint32 validate_header      ( QString& );
   static bool   get_function         ( QString& );
};

#endif // INCLUDED_QBtParser_H
