/********************************************************************
 * Copyright (C) Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : QBtProjectTab.cpp
 * DATE   : 3.09.2006
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "QBtProjectTab.h"
#include "FilesView.h"
#include "ClassesView.h"
//...........................
#include <QLayout>
#include <QTabBar>
#include <QStackedWidget>

/*------- local constants:
-------------------------------------------------------------------*/
const char* const QBtProjectTab::FILES_TAB_LABEL   = QT_TR_NOOP( "Files" );
const char* const QBtProjectTab::CLASSES_TAB_LABEL = QT_TR_NOOP( "Classes" );


//*******************************************************************
// QBtProjectTab                                            CONSTRUCTOR
//*******************************************************************
QBtProjectTab::QBtProjectTab( QWidget* const in_parent ) : QWidget( in_parent )
, tab_bar_         ( new QTabBar )
, widget_stack_    ( new QStackedWidget )
, files_tab_idx_   ( tab_bar_->addTab( tr( FILES_TAB_LABEL   ) ) )
, classes_tab_idx_ ( tab_bar_->addTab( tr( CLASSES_TAB_LABEL ) ) )
{
   widget_stack_->insertWidget( files_tab_idx_  , new FilesView  ( widget_stack_ ) );
   widget_stack_->insertWidget( classes_tab_idx_, new ClassesView( widget_stack_ ) );

   QVBoxLayout* const  main_layout = new QVBoxLayout( this );
   main_layout->addWidget( tab_bar_ );
   main_layout->addWidget( widget_stack_ );

   connect( tab_bar_, SIGNAL( currentChanged( int ) ), this, SLOT( tab_changed( int ) ));
}
// end of QBtProjectTab

//*******************************************************************
// tab_changed                                          PRIVATE slot
//*******************************************************************
void QBtProjectTab::tab_changed( const int in_tab_idx )
{
   widget_stack_->setCurrentIndex( in_tab_idx );
}
// end of tab_changed

//*******************************************************************
// retranslate                                                PUBLIC
//*******************************************************************
void QBtProjectTab::retranslate()
{
   tab_bar_->setTabText( files_tab_idx_  , tr( FILES_TAB_LABEL   ) );
   tab_bar_->setTabText( classes_tab_idx_, tr( CLASSES_TAB_LABEL ) );
}
// end of retranslate
