/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : QBtWorkspace.cpp
 * DATE   : 3.09.2006
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "QBtWorkspace.h"
#include "MainWindow.h"
#include "EventsController.h"
#include "Classer.h"
#include "Editor.h"
#include "Config.h"
#include "Properties.h"
#include "QBtParser.h"
#include "BookmarksList.h"
#include "Settings.h"
#include <QDir>
#include <QMainWindow>
#include <QPointer>
#include <QtDebug>

//*******************************************************************
// QBtWorkspace                                              CONSTRUCTOR
//*******************************************************************
QBtWorkspace::QBtWorkspace( QWidget* const in_parent ) : QWorkspace( in_parent )
, d_last_save_dir ( QDir::homePath()              )
, d_last_open_dir ( d_last_save_dir               )
, d_project_ctrl  ( new ProjectController( this ) )
{
   // File ......................................
   EventsController::instance()->append( this, Event::OPEN_FILE_REQUEST );
   EventsController::instance()->append( this, Event::NEW_FILE_REQUEST );
   EventsController::instance()->append( this, Event::SAVE_FILE_REQUEST );
   EventsController::instance()->append( this, Event::SAVE_FILE_AS_REQUEST );
   EventsController::instance()->append( this, Event::SAVE_ALL_FILES_REQUEST );
   EventsController::instance()->append( this, Event::PRINT_REQUEST );
   EventsController::instance()->append( this, Event::PRINT_FAST_REQUEST );
   EventsController::instance()->append( this, Event::SWITCH_FILES_REQUEST );
   EventsController::instance()->append( this, Event::PROPERTIES_REQUEST );

   EventsController::instance()->append( this, Event::NEXT_WINDOW_FOCUS );
   EventsController::instance()->append( this, Event::CLOSE_REQUEST );
   EventsController::instance()->append( this, Event::CLOSE_ALL_REQUEST );
   EventsController::instance()->append( this, Event::TILE_REQUEST );
   EventsController::instance()->append( this, Event::CASCADE_REQUEST );

   EventsController::instance()->append( this, Event::FILE_SELF_CLOSED );
   EventsController::instance()->append( this, Event::DOCUMENT_SELECTED_REQUEST );
   EventsController::instance()->append( this, Event::SELECTED_FILE_AND_ROW );
   EventsController::instance()->append( this, Event::REMOVE_FILE_FROM_PROJECT_REQUEST );
   
   EventsController::instance()->append( this, Event::CLASS_SELECTED );
   EventsController::instance()->append( this, Event::MEMBER_SELECTED );
   EventsController::instance()->append( this, Event::METHOD_SELECTED );
   EventsController::instance()->append( this, Event::BOOKMARK_ALL_REQUEST );
}
// end of QBtWorkspace

//*******************************************************************
// ~QBtWorkspace                                              DESTRUCTOR
//*******************************************************************
QBtWorkspace::~QBtWorkspace()
{
   EventsController::instance()->remove( this );

   if( d_project_ctrl ) {
      delete d_project_ctrl;
   }
}
// end of ~QBtWorkspace

//*******************************************************************
// save_info_about_windows                                    PUBLIC
//-------------------------------------------------------------------
// Zachowanie informacji o wszystkich otwartych oknach.
//*******************************************************************
void QBtWorkspace::save_info_about_windows()
{
   QList<QVariant> path_list;
   QList<QVariant> curpos_list;
   
   QWidgetList window_list = windowList( StackingOrder );
   if( !window_list.isEmpty() ) {
      QWidgetList::const_iterator it = window_list.begin();
      const QWidgetList::const_iterator end = window_list.end();
      while( it != end ) {
         const Editor* const editor = dynamic_cast<Editor*>( *it );
         if( editor ) {
            path_list.append( editor->file_path() );
            curpos_list.append( editor->cursor_position() );
         }
         ++it;
      }
   }
   Settings stt;
   stt.save( Config::MAIN_WINDOW_GROUP + Config::LAST_OPENED_WINDOWS_KEY1, path_list );
   stt.save( Config::MAIN_WINDOW_GROUP + Config::LAST_OPENED_WINDOWS_KEY2, curpos_list );
}
// end of save_info_about_windows

//*******************************************************************
// restore_all_windows                                        PUBLIC
//-------------------------------------------------------------------
// Otwiera wszystkie okna, jakie byly otwarte w czasie ostatniego
// zakonczenia programu.
// Oczywiscie tylko wowczas jesli tak sa ustawione odpowiednie
// opcje.
//*******************************************************************
void QBtWorkspace::restore_all_windows()
{
   QVariant data;
   QList<QVariant> path_list;
   QList<QVariant> curpos_list;

   Settings stt;
   if( stt.read( Config::MAIN_WINDOW_GROUP + Config::LAST_OPENED_WINDOWS_KEY1, data ) ) {
      path_list = data.toList();
   }
   if( stt.read( Config::MAIN_WINDOW_GROUP + Config::LAST_OPENED_WINDOWS_KEY2, data ) ) {
      curpos_list = data.toList();
   }
   
   for( qint32 i = 0; i < path_list.size(); ++i ) {
      Editor* const edt = document_selected( path_list.at( i ).toString() );
      if( edt ) {
         edt->set_cursor_position( curpos_list.at( i ).toInt() );
      }
   }
   cascade();
}
// end of restore_all_windows

//*******************************************************************
// customEvent                                     PRIVATE inherited
//*******************************************************************
void QBtWorkspace::customEvent( QEvent* const in_event )
{
   Event* const event = dynamic_cast<Event*>( in_event );
   const qint32 type = static_cast<qint32>( event->type() );
   
   switch( type ) {
      case Event::OPEN_FILE_REQUEST:
         open_file();
         break;
      case Event::NEW_FILE_REQUEST:
         new_file();
         break;
      case Event::SAVE_FILE_REQUEST:
         save_file();
         break;
      case Event::SAVE_FILE_AS_REQUEST:
         save_file_as();
         break;
      case Event::SAVE_ALL_FILES_REQUEST:
         save_all_files();
         break;
      case Event::PRINT_REQUEST:
         print( false );
         break;
      case Event::PRINT_FAST_REQUEST:
         print( true );
         break;
      case Event::SWITCH_FILES_REQUEST:
         switch_files();
         break;
      case Event::NEXT_WINDOW_FOCUS:
         next_window_focus();
         break;
      case Event::CLOSE_REQUEST:
         close();
         break;
      case Event::CLOSE_ALL_REQUEST:
         close_all();
         break;
      case Event::PROPERTIES_REQUEST:
         properties();
         break;
      case Event::TILE_REQUEST:
         tile();
         break;
      case Event::CASCADE_REQUEST:
         cascade();
         break;
      case Event::FILE_SELF_CLOSED:
         file_self_closed( event );
         break;
      case Event::DOCUMENT_SELECTED_REQUEST:
         document_selected( event->d_data.at( 0 ).toString() );
         break;
      case Event::SELECTED_FILE_AND_ROW:
         selected_file_and_row( event );
         break;
      case Event::REMOVE_FILE_FROM_PROJECT_REQUEST:
         remove_file_from_project();
         break;
      case Event::CLASS_SELECTED:
         class_selected( event );
         break;
      case Event::MEMBER_SELECTED:
         member_selected( event );
         break;
      case Event::METHOD_SELECTED:
         method_selected( event );
         break;
      case Event::BOOKMARK_ALL_REQUEST:
         bookmark_all();
         break;
    }
}
// end of customEvent

//*******************************************************************
// properties                                                PRIVATE
//*******************************************************************
void QBtWorkspace::properties()
{
   Properties* const dialog = new Properties( this );
   if( dialog ) {
      dialog->exec();
      delete dialog;
   }
}
// end of properties

//*******************************************************************
// bookmark_all                                              PRIVATE
//-------------------------------------------------------------------
// Zadanie wyswietlenia dialogu te wszystkimi znaczikami bookmark.
//*******************************************************************
void QBtWorkspace::bookmark_all()
{
   QPointer< BookmarksList > dialog = new BookmarksList( this );
   if( QDialog::Accepted == dialog->exec() ) {
      QString fpath;
      qint32 line;
      if( dialog->get_selected_bookmark( fpath, line ) ) {
         EventsController::instance()->send_event( Event::SELECTED_FILE_AND_ROW, fpath, line + 1 );
      }
   }
}
// end of bookmark_all

//*******************************************************************
// open_file                                                 PRIVATE
//*******************************************************************
void QBtWorkspace::open_file()
{
   Editor* const editor = new Editor( this );
   if( editor ) {
      if( editor->open_file_dialog( d_last_open_dir ) ) {
         Editor* const document = search_document( editor->file_path() );
         if( document ) {
            editor->block_events( TRUE );
            editor->close();
            document->setWindowState( Qt::WindowActive );
         }
         else {
            editor->show();
            EventsController::instance()->send_event( Event::FILE_OPENED,
                                                      windowList().size(),
                                                      editor->file_path() );
         }
      }
      else {
         editor->close();
      }
   }
}
// end of open_file

//*******************************************************************
// new_file                                                  PRIVATE
//*******************************************************************
void QBtWorkspace::new_file()
{
   Editor* const editor = new Editor( this );
   if( editor ) {
      editor->new_file();
      editor->show();
      EventsController::instance()->send_event( Event::FILE_OPENED,
                                                windowList().size(),
                                                editor->file_path() );
   }
}
// end of new_file

//*******************************************************************
// save_file                                                 PRIVATE
//-------------------------------------------------------------------
// Zapisanie (Save) na dysk zawartosci aktualnego edytora.
//*******************************************************************
void QBtWorkspace::save_file()
{
   Editor* const editor = dynamic_cast<Editor*>( activeWindow() );
   if( editor ) {
      editor->save();
   }
}
// end of save_file

//*******************************************************************
// save_file_as                                              PRIVATE
//-------------------------------------------------------------------
// Zapisanie (Save As) na dysk zawartosci aktualnego edytora
// pod inna nazwa.
//*******************************************************************
void QBtWorkspace::save_file_as()
{
   Editor* const editor = dynamic_cast<Editor*>( activeWindow() );
   if( editor ) {
      editor->save_as();
   }
}
// end of save_file

//*******************************************************************
// save_all_files                                            PRIVATE
//-------------------------------------------------------------------
// Zapisanie na dysk zawrtosci wszystkich otwartych edytorow.
// Zapisywane sa tylko te edytory, ktorych zawartosc od chwili
// odczytu (lub ostatniego zapisu) ulegla zmianie.
//*******************************************************************
void QBtWorkspace::save_all_files()
{
   QWidgetList windows = windowList( CreationOrder );
   if( !windows.isEmpty() ) {
      QWidgetList::const_iterator it = windows.begin();
      const QWidgetList::const_iterator end = windows.end();
      while( it != end ) {
         Editor* const editor = dynamic_cast<Editor*>( *it );
         if( editor ) {
            if( editor->document()->isModified() ) {
               editor->save();
            }
         }
         ++it;
      }
   }
}
// end of save_file

//*******************************************************************
// print                                                     PRIVATE
//-------------------------------------------------------------------
// Wydrukuj aktualny dokument.
//*******************************************************************
void QBtWorkspace::print( const bool in_fast ) const
{
   const Editor* const editor = dynamic_cast< Editor* >( activeWindow() );
   if( editor ) {
      if( editor->windowState() != Qt::WindowMinimized ) {
         editor->print( in_fast );
      }
   }
}
// end of print

//*******************************************************************
// next_window_focus                                         PRIVATE
//-------------------------------------------------------------------
// Jezeli okno majace fokus zostalo pomniejszone nalezy przekazac
// fokus do nastepnego otwartego okna.
//*******************************************************************
void QBtWorkspace::next_window_focus()
{
   // Gdyby sie mialo okazac, ze wszystkie okna sa zminimalizowane
   // aktywnym oknem robimy workspace - po to aby nie bylo tak,
   // ze ostatnie zminimalizowane okno ma nadal fokus.
   setActiveWindow( this );
 
   QWidgetList windows = windowList( StackingOrder );
   if( !windows.empty() ) {
      Editor* last_opened = 0;
      QWidgetList::const_iterator it = windows.begin();
      const QWidgetList::const_iterator end = windows.end();
      while( it != end ) {
         Editor* const w = dynamic_cast< Editor* >( *it );
         if( w->windowState() != Qt::WindowMinimized ) {
            last_opened = w;
         }
         ++it;
      }
      if( last_opened ) {
         last_opened->setFocus( Qt::OtherFocusReason );
      }
   }
}
// end of next_window_focus

//*******************************************************************
// close                                                     PRIVATE
//-------------------------------------------------------------------
// Okno edytora zamkniete poprzez wybor pozycji w menu.
//*******************************************************************
void QBtWorkspace::close()
{
   Editor* const editor = dynamic_cast<Editor*>( activeWindow() );
   if( editor ) {
      const QString file_path = editor->file_path();
      editor->close();
      EventsController::instance()->send_event( Event::FILE_CLOSED, windowList().size(), file_path );
   }
}
// end of close

//*******************************************************************
// file_self_closed                                           PRIVATE
//-------------------------------------------------------------------
// Okno edytora zostalo zamkniete przez klikniecie mysza na krzyzyk
// na belce tytulowej okna.
//*******************************************************************
void QBtWorkspace::file_self_closed( Event* const in_event )
{
   const QString file_path = in_event->d_data.at( 0 ).toString();
   EventsController::instance()->send_event( Event::FILE_CLOSED, windowList().size(), file_path );
}
// end of file_self_closed

//*******************************************************************
// close_all                                                 PRIVATE
//*******************************************************************
void QBtWorkspace::close_all()
{
   closeAllWindows();
   QWidgetList const windows = windowList( CreationOrder );
   if( windows.isEmpty() ) {
      EventsController::instance()->send_event( Event::ALL_FILES_CLOSED );
   }
}
// end of close_all

//*******************************************************************
// class_selected                                            PRIVATE
//*******************************************************************
void QBtWorkspace::class_selected( Event* const in_event )
{
   const QString class_name = in_event->d_data.at( 0 ).toString();
   const QString file = Classer::instance()->data()[ class_name ][ 0 ].file();

   if( !class_name.isEmpty() && !file.isEmpty() ) {
      Editor* const editor = document_selected( file );
      if( editor ) {
         editor->search_class( class_name );
      }
   }
}
// end of class_selected

//*******************************************************************
// member_selected                                           PRIVATE
//*******************************************************************
void QBtWorkspace::member_selected( Event* const in_event )
{
   const QString class_name = in_event->d_data.at( 0 ).toString();
   const QString text = in_event->d_data.at( 1 ).toString();

   QString file = "";
   qint32  pos  = -1;
   
   const QVector< ParsedItem >& items = Classer::instance()->data()[ class_name ];
   QVector< ParsedItem >::const_iterator it = items.begin();
   const QVector< ParsedItem >::const_iterator end = items.end();
   while( it != end ) {
      if( ( ParsedItem::MEMBER == it->type() ) && ( text == it->text() ) ) {
         file = it->file();
         pos = it->pos();
         break;
      }
      ++it;
   }

   if( pos != -1 ) {
      Editor* const edt = document_selected( file );
      if( edt ) {
         edt->set_cursor_position( pos );
      }
   }
}
// end of member_selected

//*******************************************************************
// method_selected                                           PRIVATE
//-------------------------------------------------------------------
// Proba znalezienia deklaracji funkcji w pliku zrodlowym.
// UWAGA: plik zrodlowy moze nie istniec.
//        Nalezy wowczas szukac w pliku naglowkowym.
//*******************************************************************
void QBtWorkspace::method_selected( Event* const in_event )
{
   const QString class_name = in_event->d_data.at( 0 ).toString();
   const QString text = in_event->d_data.at( 1 ).toString();
   const QString path_without_ext = Shared::remove_extension( in_event->d_data.at( 2 ).toString() );

   QString fun_name;
   QString hpp_params;

   QString file = "";
   qint32 pos = -1;
   
   // WYZNACZENIE NAZWY FUNKCJI I PARAMETROW PRZYSLANEGO OPISU FUNKCJI (TEXT)
   QRegExp expr( "(~{0,1}\\w+)\\s*\\((.*)\\)$" );
   if( expr.indexIn( text ) != -1 ) {
      const qint32 n = expr.numCaptures();
      if( n == 2 ) {
         fun_name = expr.cap( 1 );
         hpp_params = expr.cap( 2 );
      }
   }

   //------- SZUKANIE W PLIKACH C/CPP ---------------------------------

   // SPRAWDZENIE CZY SYGNATURA FUNKCJI ODPOWIADA KTOREJ Z FUNKCJI
   // ZNAJDUJACEJ SIE W PLIKACH C/CPP.
   const QVector< ParsedItem >& items = Classer::instance()->data()[ "" ];
   QVector< ParsedItem >::const_iterator it = items.begin();
   const QVector< ParsedItem >::const_iterator end = items.end();
   /*
   while( it != items.end() ) {
      if( it->name() == "set_data" ) {
         qDebug() << "----------------------------------------";
         qDebug() << it->text() << " , " << it->file();
         qDebug() << text;
      }
      ++it;
   }
   return;
   */
   while( it != end ) {
      if( ( ParsedItem::METHOD == it->type() ) && ( fun_name == it->name() ) ) {
         if( it->file().startsWith( path_without_ext ) ) {
            if( ( ParsedItem::IMPL == it->what() ) || ( ParsedItem::DECL_IMPL == it->what() ) ) {
               // WYZNACZENIE PARAMETROW WYWOLANIA W/G FUNKCJI CPP
               QString cpp_params;
               QRegExp expr( "\\((.*)\\)" );
               if( expr.indexIn( it->text() ) != -1 ) {
                  const qint32 n = expr.numCaptures();
                  if( n >= 1 ) {
                     cpp_params = expr.cap( n );
                     //qDebug() << hpp_params << "," << cpp_params;
                     // SPRAWDZENIE ZGODNOSCI PARAMETROW WYWOLANIA.
                     if( equ_param_sign( hpp_params, cpp_params ) ) {
                        file = it->file();
                        pos = it->pos();
                        break;
                     }
                  }
               }
            }
         }
      }
      ++it;
   }

   //------- SZUKANIE W PLIKACH H/HPP ---------------------------------
   if( -1 == pos ) {
      const QVector< ParsedItem >& items = Classer::instance()->data()[ class_name ];
      QVector< ParsedItem >::const_iterator it = items.begin();
      const QVector< ParsedItem >::const_iterator end = items.end();
      while( it != end ) {
         if( ( ParsedItem::METHOD == it->type() ) && ( text == it->text() ) ) {
            file = it->file();
            pos = it->pos();
            break;
         }
         ++it;
      }
   }

   if( pos != -1 ) {
      Editor* const edt = document_selected( file );
      if( edt ) {
         edt->set_cursor_position( pos );
      }
   }
}
// end of method_selected

//*******************************************************************
// equ_param_sign                                            PRIVATE
//-------------------------------------------------------------------
// Sprawdzenie czy dwie sygnatury parametrow maja to samo znaczenie.
// Najpierw rozkladamy obie sygnatury na skladowe (QStringList).
// Jesli jest ich tyle samo, to sprawdzamy czy elementy z
// 'in_sign1' zawieraja sie w 'in_sign2'.
//*******************************************************************
bool QBtWorkspace::equ_param_sign ( const QString& in_sign1, const QString& in_sign2 )
{
   bool retval = false;

   QString hpp_item;
   QString cpp_item;
   const QStringList list1 = in_sign1.split( ",", QString::SkipEmptyParts );
   const QStringList list2 = in_sign2.split( ",", QString::SkipEmptyParts );

   // Jest sens sprawdzac parametry jesli jest ich tyle samo.
   // Jesli nie to od razu jasne, ze sygnatury sie nie zgadzaja.
   if( list1.size() == list2.size() ) {
      retval = true;
      for( qint32 i = 0; i < list1.size(); ++i ) {
         hpp_item = list1.at( i );
         cpp_item = list2.at( i ).trimmed();
         remove_default_value( hpp_item );

         hpp_item.replace( "*", "\\s*\\*\\s*[const\\s*]*" );
         hpp_item.replace( "&", "\\s*&\\s*" );

         if( -1 == cpp_item.indexOf( QRegExp( hpp_item ) ) ) {
            retval = false;
            break;
         }
      }
   }
   return retval;
}
// end of equ_param_sign

//*******************************************************************
// remove_default_value                                      PRIVATE
//-------------------------------------------------------------------
// Usuniecie z deklaracji parametru wartosci domyslnej ( np. = 0 ).
//*******************************************************************
void QBtWorkspace::remove_default_value( QString& inout_param )
{
   const qint32 idx = inout_param.indexOf( '=' );
   if( idx != -1 ) inout_param.resize( idx );
   inout_param = inout_param.trimmed();
}
// end of remove_default_value

//*******************************************************************
// switch_files                                              PRIVATE
//*******************************************************************
void QBtWorkspace::switch_files()
{
   Editor* const editor = dynamic_cast<Editor*>( activeWindow() );
   if( editor ) {
      const QString file_path = editor->file_path();
      const QFileInfo fi( file_path );
      QString new_file_path = fi.absolutePath();
      if( !new_file_path.endsWith( QBtParser::SLASH ) ) new_file_path += QBtParser::SLASH;
      new_file_path += fi.baseName();
      new_file_path += ".";
      
      if( file_path.endsWith( "cpp" ) || file_path.endsWith( "c" ) || file_path.endsWith( "cxx" ) ) {
         new_file_path += "h";
      }
      else if( file_path.endsWith( "hpp" ) || file_path.endsWith( "h" ) || file_path.endsWith( "hxx" ) ) {
         new_file_path += "cpp";
      }

      document_selected( new_file_path );
   }
}
// end of switch_files

//*******************************************************************
// remove_file_from_project                                  PRIVATE
//*******************************************************************
void QBtWorkspace::remove_file_from_project()
{
   Editor* const editor = dynamic_cast<Editor*>( activeWindow() );
   if( editor ) {
      EventsController::instance()->send_event( Event::REMOVE_FILE_FROM_PROJECT, editor->file_path() );
   }
}
// end of remove_file_from_project

//*******************************************************************
// document_selected                                         PRIVATE
//*******************************************************************
Editor* QBtWorkspace::document_selected( const QString& in_file_path )
{
   Editor* retval = 0;
   
   if( !in_file_path.isEmpty() ) {
      const QFileInfo file_info( in_file_path );
      if( file_info.exists() ) {
         retval = search_document( in_file_path );
         if( retval ) {   // taki dokument jest juz otwartych w ktoryms z edytorow.
            retval->setWindowState( ( retval->windowState() & ~Qt::WindowMinimized ) | Qt::WindowActive );
         }
         else {         // trzeba otworzyc nowy edytor ze wskazanym plikiem
            retval = new Editor( this );
            if( retval ) {
               if( retval->open_file( in_file_path ) ) {
                  retval->show();
                  EventsController::instance()->send_event( Event::FILE_OPENED, windowList().size(), retval->file_path() );
               }
               else {
                  retval->close();
               }
            }
         }
      }
   }
   return retval;
}
// end of document_selected

//*******************************************************************
// selected_file_and_row                                     PRIVATE
//*******************************************************************
void QBtWorkspace::selected_file_and_row( Event* const in_event )
{
   const QString fpath = in_event->d_data.at( 0 ).toString();
   const int row = in_event->d_data.at( 1 ).toInt();

   Editor* const editor = document_selected( fpath );
   if( editor ) {
      editor->set_cursor_position( row, 1 );
   }
}
// end of selected_file_and_row

//*******************************************************************
// search_document                                           PRIVATE
//-------------------------------------------------------------------
// Wyszukiwanie wsrod OTWARTYCH edytorow, takiego ktorego
// plik odpowiadal by wskazanej sciezce.
//*******************************************************************
Editor* QBtWorkspace::search_document( const QString& in_file_path )
{
   Editor* retval = 0;

   if( !in_file_path.isEmpty() ) {   
      const QWidgetList windows = windowList( CreationOrder );
      if( !windows.isEmpty() ) {
         QWidgetList::const_iterator it = windows.begin();
         const QWidgetList::const_iterator end = windows.end();
         while( it != end ) {
            Editor* const editor = dynamic_cast<Editor*>( *it );
            if( editor->file_path() == in_file_path ) {
               retval = editor;
               break;            
            }
            ++it;
         }
      }
   }
   return retval;
}
// end of search_document
