#ifndef INCLUDED_QBtWorkspace_h
#define INCLUDED_QBtWorkspace_h

/********************************************************************
 * Copyright (C) Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : QBtWorkspace.h
 * DATE   : 3.09.2006
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "ProjectController.h"
#include <QWorkspace>

/*------- forward declarations:
-------------------------------------------------------------------*/
class Editor;
class Event;

/*------- class declaration:
-------------------------------------------------------------------*/
class QBtWorkspace : public QWorkspace
{
   Q_OBJECT

// ******* CONSTRUCTION/DESTRUCTION *******
public:
   QBtWorkspace( QWidget* = 0 );
   ~QBtWorkspace();
private:
   QBtWorkspace( const QBtWorkspace& );
   QBtWorkspace& operator=( const QBtWorkspace& );

// ******* MEMBERS *******
private:
   QString        d_last_save_dir;
   QString        d_last_open_dir;
   ProjectController* const d_project_ctrl;

// ******* METHODS *******
public:
   void    save_info_about_windows ();
   void    restore_all_windows ();
   const QString&  last_save_dir    () const;
   void    last_save_dir            ( const QString& );
   const QString&  last_open_dir    () const;
   void    last_open_dir            ( const QString& );
   void    properties               ();
   void    close_all                ();
   void    class_selected           ( Event* );
   void    member_selected          ( Event* );
   void    method_selected          ( Event* );
private:
   bool    equ_param_sign           (const QString& in_sign1, const QString& in_sign2 );
   void    remove_default_value     ( QString& inout_param );
   void    bookmark_all             ();
   void    customEvent              ( QEvent* );
   void    open_file                ();
   void    new_file                 ();
   void    save_file                ();
   void    save_file_as             ();
   void    save_all_files           ();
   void    print                    ( bool ) const;
   void    file_self_closed         ( Event* );
   void    switch_files             ();
   void    remove_file_from_project ();
   void    close                    ();
   Editor* document_selected        ( const QString& );
   void    selected_file_and_row    ( Event* );
   Editor* search_document          ( const QString& );
   void    next_window_focus        ();
};

inline const QString& QBtWorkspace::last_save_dir() const {
   return d_last_save_dir;
}
inline void QBtWorkspace::last_save_dir( const QString& in_save_dir ) {
   d_last_save_dir = in_save_dir;
}
inline const QString& QBtWorkspace::last_open_dir() const {
   return d_last_open_dir;
}
inline void QBtWorkspace::last_open_dir( const QString& in_open_dir ) {
   d_last_open_dir = in_open_dir;
}

#endif // INCLUDED_QBtWorkspace_h
