/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : Renumer.cpp
 * DATE   : 23.09.2006
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "Renumer.h"
#include "Editor.h"
#include "Margin.h"
#include "Folder.h"
#include "Bookmarks.h"
#include "Breakpoints.h"
#include <QtDebug>

//*******************************************************************
// run                                                        PUBLIC
//*******************************************************************
void Renumer::run( QTextEdit* const in_editor, const bool in_points )
{
   Editor* const editor = dynamic_cast< Editor* >( in_editor );
   if( editor ) {
      const QString fpath = editor->file_path();
      
      if( in_points ) {
         Bookmarks::instance()->clear( fpath );
         Breakpoints::instance()->clear( fpath );
      }

      //qint32 counter = 0;
      QTextBlock block = in_editor->document()->begin();
      while( block.isValid() ) {
         // Jesli linia nie ma jeszcze bloku danych to go dodajemy.
         if( !block.userData() ) block.setUserData( new LineInfo );
         const LineInfo* const inf = dynamic_cast<LineInfo*>( block.userData() );
         
         if( inf ) {
            //inf->nr = counter++;    // <<<=========== zmaina numeru
            if( in_points ) {
               const int nr = block.blockNumber();
               const QString txt = block.text().trimmed();
               if( inf->bookmark ) {
                  Bookmarks::instance()->add( fpath, nr, Points::SET, txt );
               }
               if( inf->breakpoint ) {
                  Breakpoints::instance()->add( fpath, nr, Points::SET, txt );
               }
            }
         }
         block = block.next();
      }
      editor->cursor_position_changed( false );
      editor->set_margin();
   }
}
// end of run
