/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : Renumer.h
 * DATE   : 23.09.2006
 *******************************************************************/
#ifndef INCLUDED_Renumer_h
#define INCLUDED_Renumer_h

/*------- include files:
-------------------------------------------------------------------*/
#ifndef INCLUDED_QTextEdit
#define INCLUDED_QTextEdit
#include <QTextEdit>
#endif // INCLUDED_QTextEdit

#ifndef INCLUDED_QTextDocument
#define INCLUDED_QTextDocument
#include <QTextDocument>
#endif // INCLUDED_QTextDocument

#ifndef INCLUDED_QTextCursor
#define INCLUDED_QTextCursor
#include <QTextCursor>
#endif // INCLUDED_QTextCursor

#ifndef INCLUDED_QTextBlock
#define INCLUDED_QTextBlock
#include <QTextBlock>
#endif // INCLUDED_QTextBlock

/*------- class declaration:
-------------------------------------------------------------------*/
class Renumer : public QObject
{
	Q_OBJECT

// ******* CONSTRUCTION / DESTRUCTION *******
public:
	Renumer( QTextEdit* const in_editor )
	: QObject ( in_editor )
	, editor_ ( in_editor )
	{
		if( editor_ ) {
			previous_block_ = editor_->textCursor().block().previous();
			next_block_     = editor_->textCursor().block().next();
		}
	}

	~Renumer()
	{
		if( editor_ ) {
			const QTextBlock previous = editor_->textCursor().block().previous();
			const QTextBlock next     = editor_->textCursor().block().next();
			if( ( previous != previous_block_ ) || ( next != next_block_ ) ) {
				run( editor_ );
			}
		}
	}
private:
	Renumer( const Renumer& );
	Renumer& operator=( const Renumer& );

// ******* MEMBERS *******
private:
	QTextEdit* const editor_;
	QTextBlock       previous_block_;
	QTextBlock       next_block_;

// ******* METHODS *******
public:
	static void run( QTextEdit*, bool = true );
};

#endif // INCLUDED_Renumer_h
