/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : Shared.cpp
 * DATE   : 2.09.2006
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "Shared.h"
#include "Config.h"
#include "Settings.h"
#include <QDir>
#include <QApplication>
#include <QDesktopWidget>
#include <QMessageBox>
#include <QtDebug>


/*------- local constants:
-------------------------------------------------------------------*/
const char* const Shared::ORGANISATION             = "beesoft.org";
const char* const Shared::VERSION                  = "0.18";
const char* const Shared::PROGRAM_NAME             = "Cobras IDE";
const char* const Shared::PROGRAM_SUBNAME          = QT_TR_NOOP( "programming environment" );
const QString     Shared::SRC                      = "SOURCES";
const QString     Shared::HDR                      = "HEADERS";
const QString     Shared::TS                       = "TRANSLATIONS";
const QString     Shared::UI                       = "FORMS";
const QString     Shared::RSC                      = "RESOURCES";

/*------- global variables:
-------------------------------------------------------------------*/
QString Shared::d_buffer( "" );

//*******************************************************************
// full_program_name
//*******************************************************************
const QString& Shared::full_program_name()
{
   d_buffer  = PROGRAM_NAME;
   d_buffer += " - ";
   d_buffer += tr( PROGRAM_SUBNAME );
   d_buffer += ", ver. ";
   d_buffer += VERSION;

   return d_buffer;
}
// end of full_program_name

//*******************************************************************
// resize
//*******************************************************************
void Shared::resize( QWidget* const in_widget, const int in_width, const int in_height )
{
   const int   screen_width = qApp->desktop()->availableGeometry().width();
   const int   screen_height = qApp->desktop()->availableGeometry().height();
   const float width_percent = static_cast<float>( in_width ) / 100.0;
   const float height_percent = static_cast<float>( in_height ) / 100.0;    
   const int   dx = static_cast<int>( screen_width * width_percent );
   const int   dy = static_cast<int>( screen_height * height_percent );

   resize( in_widget, QSize( dx, dy ));
}
void Shared::resize( QWidget* const in_widget, const QSize& in_size )
{
    const int screen_width  = qApp->desktop()->availableGeometry().width();
    const int screen_height = qApp->desktop()->availableGeometry().height();
    const int x  = ( screen_width  - in_size.width()  + 1 ) >> 1;
    const int y  = ( screen_height - in_size.height() + 1 ) >>  1;
    
    in_widget->move( x, y );
    in_widget->resize( in_size.width(), in_size.height() );
}
void Shared::resize_width( QWidget* const in_widget, const int in_width )
{
    const int screen_width  = qApp->desktop()->screenGeometry().width();
    const int screen_height = qApp->desktop()->screenGeometry().height();
    const float width_percent = ( static_cast<float>( in_width ) ) / 100.0;
    
    QSize new_size = in_widget->size();
    new_size.setWidth( static_cast<int>( screen_width * width_percent ));
    in_widget->resize( new_size );
    
    const int x  = ( screen_width  - new_size.width()  + 1 ) >> 1;
    const int y  = ( screen_height - new_size.height() + 1 ) >>  1;
    in_widget->move( x, y );
}
// end of resize

//*******************************************************************
// extension
//*******************************************************************
const QString& Shared::extension( const QString& in_path )
{
    d_buffer.clear();
    
    const qint32 idx1 = in_path.lastIndexOf( '/' );
    const qint32 idx2 = in_path.lastIndexOf( '.' );

    bool clip = false;
    if( idx2 != -1 ) clip = ( idx2 > idx1 );

    if( clip )     d_buffer = in_path.mid( idx2 + 1 );
    return d_buffer;
}
// end of extension

//*******************************************************************
// remove_extension
//*******************************************************************
const QString& Shared::remove_extension( const QString& in_path )
{
    const qint32 idx1 = in_path.lastIndexOf( '/' );
    const qint32 idx2 = in_path.lastIndexOf( '.' );

    bool clip = false;
    if( idx2 != -1 ) {
        clip = ( idx2 > idx1 );
    }

    d_buffer = ( clip ) ? in_path.left( idx2 ) : in_path;
    return d_buffer;
}
// end of remove_extension

//*******************************************************************
// is_src_ext
//*******************************************************************
bool Shared::is_src_ext( const QString& in_ext )
{
    bool retval = false;
    if( !in_ext.isEmpty() ) {
        if( ( "cpp" == in_ext ) || ( "cxx" == in_ext ) || ( "cc" == in_ext ) || ( "c" == in_ext ) ) {
            retval = true;
        }
    }
    return retval;
}
// end of is_src_ext

//*******************************************************************
// is_hdr_ext
//*******************************************************************
bool Shared::is_hdr_ext( const QString& in_ext )
{
    bool retval = false;
    if( !in_ext.isEmpty() ) {
        if( ( "h" == in_ext ) || ( "hpp" == in_ext ) || ( "hxx" == in_ext ) || ( "hh" == in_ext ) ) {
            retval = true;
        }
    }
    return retval;
}
// end of is_hdr_ext

//*******************************************************************
// is_ui_ext
//*******************************************************************
bool Shared::is_ui_ext( const QString& in_ext )
{
    bool retval = false;
    if( !in_ext.isEmpty() ) {
        retval = ( "ui" == in_ext );
    }
    return retval;
}
// end of is_img_ext

//*******************************************************************
// is_qrc_ext
//*******************************************************************
bool Shared::is_qrc_ext( const QString& in_ext )
{
    bool retval = false;
    if( !in_ext.isEmpty() ) {
        retval = ( "qrc" == in_ext );
    }
    return retval;
}
// end of is_img_ext

//*******************************************************************
// complete_dir
//*******************************************************************
void Shared::complete_dir( QString& in_dir )
{
    if( !in_dir.isEmpty() ) {
        if( !in_dir.endsWith( QDir::separator() ) ) {
            in_dir += QDir::separator();
        }
    }
}
// end of complete_dir

//*******************************************************************
// create_empty_file
//*******************************************************************
bool Shared::create_empty_file( const QString& in_file_path )
{
   bool retval = false;

    QFile file( in_file_path );
    if( !file.exists() ) {
        if( file.open( QIODevice::WriteOnly | QIODevice::Truncate ) ) {
            file.close();
            retval = true;
       }
    }
    return retval;
}
// end of create_empty_file

//*******************************************************************
// critical
//*******************************************************************
void Shared::critical( QWidget* const in_parent, const QString& in_caption, const QString& in_message )
{
    QMessageBox::critical( in_parent,
        in_caption,
        in_message,
        QMessageBox::Cancel,
        QMessageBox::NoButton );
}
// end of critical

//*******************************************************************
// relative_path                                       PUBLIC static
//*******************************************************************
const QString& Shared::relative_path( const QString& in_dir, const QString& in_path )
{
    d_buffer = in_path;
    
    if( !in_dir.isEmpty() && !in_path.isEmpty() ) {
        QString dir  = in_dir;
        qint32 idx   = -1;;
        qint32 level = 0;
        bool clipped = false;

        while( !dir.isEmpty() ) {
            if( d_buffer.startsWith( dir ) ) {
                d_buffer.remove( 0, dir.length() );
                clipped = true;
                break;
            }
            else {
                if( ( idx = dir.lastIndexOf( '/' ) ) > 0 ) {
                    if( ( idx = dir.lastIndexOf( '/', idx - 1 ) ) > 0 ) {
                        dir.resize( idx + 1 );
                        ++level;
                    }
                    else {
                        break;
                    }
                }
                else {
                    break;
                }
            }
        }

        if( clipped ) {
            QString prefix( "" );
            while( level ) {
                prefix += "../";
                --level;
            }
            d_buffer.insert( 0, prefix );
        }
    }

    return d_buffer;
}
// end of relative_path

//*******************************************************************
// qmake_varname
//*******************************************************************
//const QString& Shared::qmake_varname
