/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : Shared.h
 * DATE   : 2.09.2006
 *******************************************************************/
#ifndef INCLUDED_Shared_h
#define INCLUDED_Shared_h

/*------- include files:
-------------------------------------------------------------------*/
#include <QObject>
#include <QString>
#include <QWidget>

const quint32 None      = 0x00000000;
const quint32 Public    = 0x00000001;
const quint32 Protected = 0x00000010;
const quint32 Private   = 0x00000100;
const quint32 Slots     = 0x00010000;
const quint32 Signals   = 0x00100000;


/*------- BookmarkInfo declaration:
-------------------------------------------------------------------*/
class BookmarkInfo : public QObject
{
public:
   BookmarkInfo() : QObject()
   , line_ ( -1 )
   , text_ ( "" )
   {}
   BookmarkInfo( const qint32 in_line, const QString& in_text ) : QObject()
   , line_ ( in_line )
   , text_ ( in_text )
   {}
   BookmarkInfo( const BookmarkInfo& in_other ) : QObject()
   {
      line_ = in_other.line_;
      text_ = in_other.text_;
   }
   BookmarkInfo& operator=( const BookmarkInfo& in_other )
   {
      if( this != &in_other ) {
         line_ = in_other.line_;
         text_ = in_other.text_;
      }
      return *this;      
   }

//******* METHODS *******
public:
   qint32         line() const { return line_; }
   const QString& text() const { return text_; }

// ******* MEMBERS *******
private:
   qint32  line_;
   QString text_;
};

/*------- FileBookmarks declaration:
-------------------------------------------------------------------*/
class FileBookmarks : public QObject
{
public:
   FileBookmarks() : QObject()
   , file_ ( "" )
   {}
   FileBookmarks( const QString& in_file, const QVector<BookmarkInfo>& in_bookmarks ) : QObject()
   , file_      ( in_file )
   , bookmarks_ ( in_bookmarks )
   {}
   FileBookmarks( const FileBookmarks& in_other ) : QObject()
   {
      file_      = in_other.file_;
      bookmarks_ = in_other.bookmarks_;
   }
   FileBookmarks& operator=( const FileBookmarks& in_other )
   {
      if( this != &in_other ) {
         file_      = in_other.file_;
         bookmarks_ = in_other.bookmarks_;
      }
      return *this;      
   }

// ******* METHODS *******
public:
   const QString&               file      () const { return file_;      }
   const QVector<BookmarkInfo>& bookmarks () const { return bookmarks_; }
// ******* MEMBERS *******
private:
   QString               file_;
   QVector<BookmarkInfo> bookmarks_;
};

/*------- Shared declaration:
-------------------------------------------------------------------*/
class Shared : public QObject
{
// ******* CONSTRUCTION/DESTRUCTION *******
public:
   Shared() {}
   ~Shared() {}
private:
   Shared( const Shared& );
   Shared& operator=( const Shared& );

// ******* TYPES *******
public:
   enum {
      UnknownRole = 0,
      Class,
      Struct,
      Enum,
      Member,
      Method,
   };

// ******* CONSTANTS *******
public:
   static const char* const ORGANISATION;
   static const char* const VERSION;
   static const char* const PROGRAM_NAME;
   static const QString     SRC;
   static const QString     HDR;
   static const QString     TS;
   static const QString     UI;
   static const QString     RSC;

private:
   static const char* const PROGRAM_SUBNAME;
   static QString           d_buffer;

// ******* METHODS *******
public:
   static const QString& full_program_name ();
   static void                      resize ( QWidget* const, const int, const int );
   static void                      resize ( QWidget* const, const QSize& );
   static void                resize_width ( QWidget* const, const int );
   static const QString&         extension ( const QString& );
   static const QString&  remove_extension ( const QString& );
   static bool                  is_src_ext ( const QString& );
   static bool                  is_hdr_ext ( const QString& );
   static bool                   is_ui_ext ( const QString& );
   static bool                  is_qrc_ext ( const QString& );
   static void                complete_dir ( QString& );
   static bool           create_empty_file ( const QString& );
   static void                    critical ( QWidget*, const QString&, const QString& );
//   static bool                   cmp_items ( const ClassItem&, const ClassItem& );
   static const QString&     relative_path ( const QString&, const QString& );
};


#endif // INCLUDED_Shared_h

