/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : ToolsTable.cpp
 * DATE   : 21.05.2007
 *******************************************************************/

/*------- include files:
-------------------------------------------------------------------*/
#include "ToolsTable.h"

//*******************************************************************
// ToolTable                                             CONSTRUCTOR
//*******************************************************************
ToolsTable::ToolsTable( QWidget* const in_parent )
: QListWidget( in_parent )
{
   setSortingEnabled( false );
   setAlternatingRowColors( true );
}
// end of ToolsTable

//*******************************************************************
// get_data                                                   PUBLIC
//*******************************************************************
bool ToolsTable::get_data( QStringList& out_strings ) const
{
   out_strings.clear();

   const qint32 n = count();
   if( n > 0 ) {
      for( qint32 i = 0; i < n; ++i ) {
         const QListWidgetItem* const itm = item( i );
         if( itm ) {
            const QString txt = itm->text();
            if( !txt.isEmpty() ) {
               out_strings.push_back( txt );
            }
         }
      }
   }

   return ( !out_strings.empty() );
}
// end of get_data

//*******************************************************************
// set_data                                                   PUBLIC
//*******************************************************************
void ToolsTable::set_data( const QStringList& in_strings )
{
   clear();

   const qint32 n = in_strings.count();
   if( n > 0 ) {
      for( qint32 i = 0; i < n; ++i ) {
         QListWidgetItem* const itm = new_item();
         if( itm ) {
            itm->setText( in_strings[ i ] );
            addItem( itm );
         }
      }

      QListWidgetItem* const itm = item( 0 );
      if( itm ) {
         scrollToItem( itm, PositionAtTop );
         setCurrentItem( itm );
      }
   }
}
// end of set_data

//*******************************************************************
// new_item                                                  PRIVATE
//*******************************************************************
QListWidgetItem* ToolsTable::new_item() const
{
   QListWidgetItem* const itm = new QListWidgetItem;
   if( itm ) {
      itm->setFlags( itm->flags() | Qt::ItemIsEditable );
   }
   return itm;
}
// end of append

//*******************************************************************
// append                                               PRIVATE slot
//*******************************************************************
void ToolsTable::append()
{
   QListWidgetItem* const itm = new_item();
   if( itm ) {
      addItem( itm );
      editItem( itm );
      setCurrentItem( itm );
   }
}
// end of append

//*******************************************************************
// insert                                               PRIVATE slot
//*******************************************************************
void ToolsTable::insert()
{
   if( 0 == count() ) {
      append();
   }
   else {
      QListWidgetItem* const itm = new_item();
      if( itm ) {
         insertItem( currentRow(), itm );
         editItem( itm );
         setCurrentItem( itm );
      }
   }
}
// end of insert

//*******************************************************************
// remove                                               PRIVATE slot
//*******************************************************************
void ToolsTable::remove()
{
   QListWidgetItem* const item = takeItem( currentRow() );
   if( item ) delete item;
}
// end of remove

//*******************************************************************
// up                                                   PRIVATE slot
//*******************************************************************
void ToolsTable::up()
{
   if( count() > 0 ) {
      const qint32 row = currentRow();
      if( row > 0 ) {
         QListWidgetItem* const item = takeItem( row );
         if( item ) {
            insertItem( row - 1, item );
            scrollToItem( item, PositionAtCenter );
            setCurrentItem( item );
         }
      }
   }
}
// end of up

//*******************************************************************
// down                                                 PRIVATE slot
//*******************************************************************
void ToolsTable::down()
{
   const qint32 n = count();
   if( n > 0 ) {
      const qint32 row = currentRow();
      if( row < ( n - 1 ) ) {
         QListWidgetItem* const item = takeItem( row );
         if( item ) {
            insertItem( row + 1, item );
            scrollToItem( item, PositionAtCenter );
            setCurrentItem( item );
         }
      }
   }
}
// end of down
