/********************************************************************
 * Copyright (C) 2006 Piotr Pszczolkowski
 *-------------------------------------------------------------------
 * This file is part of Cobras.
 *
 * Cobras is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Cobras is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Cobras; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *-------------------------------------------------------------------
 * AUTHOR : Piotr Pszczolkowski (piotr@beesoft.org)
 * PROJECT: Cobras
 * FILE   : Translator.h
 * DATE   : 30.07.2007
 *-------------------------------------------------------------------
 * Translator (singleton) ma za zadanie obslugiwac wszystkie sprawy
 * zwiazane ze tlumaczeniami napisow na inne jezyki.
 *******************************************************************/
#ifndef INCLUDED_Translator_h
#define INCLUDED_Translator_h

/*------- include files:
-------------------------------------------------------------------*/
#include <QTranslator>

/*------- class declaration:
-------------------------------------------------------------------*/
class Translator : public QTranslator
{
//******* TYPES *******
public:
   typedef enum {
      ENGLISH = 0,
      POLISH
   } Language;
   
//******* CONSTRUCTION / DESTRUCTION *******
private:
   Translator();
   Translator( const Translator& );
   Translator& operator= ( const Translator& );
public:
   ~Translator();

//******* CONASTANTS *******
private:
   static const char* const TRANS_FNAME;
   static const char* const SETTINGS_INDEX;

//******* MEMBERS *******
private:
   static Translator* instance_;
   Language current_language_;

//******* METHODS *******
public:
   static Translator* instance();
   Language current_lang () const;
   void     change_to    ( Language );
   void     restore      ();
private:
   QString  get_fname    ( Language ) const;
   void     read_settings();
   void     save_settings() const;
};

#endif // INCLUDED_Translator_h
